@@ -46,6 +46,13 @@ public:
 	virtual bool setGraphicsMode(int mode) = 0;
 	virtual void resetGraphicsScale() = 0;
 	virtual int getGraphicsMode() const = 0;
+	virtual const OSystem::GraphicsMode *getSupportedShaders() const {
+		static const OSystem::GraphicsMode no_shader[2] = {{"NONE", "Normal (no shader)", 0}, {0, 0, 0}};
+		return no_shader;
+	};
+	virtual bool setShader(int id) { return false; }
+	virtual int getShader() const { return 0; }
+
 #ifdef USE_RGB_COLOR
 	virtual Graphics::PixelFormat getScreenFormat() const = 0;
 	virtual Common::List<Graphics::PixelFormat> getSupportedFormats() const = 0;
@@ -57,9 +64,11 @@ public:
 	virtual OSystem::TransactionError endGFXTransaction() = 0;
 
 	// ResidualVM specific method
-	virtual void launcherInitSize(uint w, uint h) = 0;
+	virtual void setupScreen(uint screenW, uint screenH, bool fullscreen, bool accel3d) = 0;
 	// ResidualVM specific method
-	virtual Graphics::PixelBuffer setupScreen(uint screenW, uint screenH, bool fullscreen, bool accel3d) = 0;
+	virtual Graphics::PixelBuffer getScreenPixelBuffer() = 0;
+	// ResidualVM specific method
+	virtual void suggestSideTextures(Graphics::Surface *left, Graphics::Surface *right) = 0;
 
 	virtual int16 getHeight() = 0;
 	virtual int16 getWidth() = 0;
@@ -92,6 +101,8 @@ public:
 	virtual bool lockMouse(bool lock) = 0;
 
 	virtual void displayMessageOnOSD(const char *msg) {}
+	virtual void displayActivityIconOnOSD(const Graphics::Surface *icon) {}
+
 
 	// Graphics::PaletteManager interface
 	//virtual void setPalette(const byte *colors, uint start, uint num) = 0;
