@@ -82,6 +82,18 @@ int ModularBackend::getGraphicsMode() const {
 	return _graphicsManager->getGraphicsMode();
 }
 
+const OSystem::GraphicsMode *ModularBackend::getSupportedShaders() const {
+	return _graphicsManager->getSupportedShaders();
+}
+
+bool ModularBackend::setShader(int id) {
+	return _graphicsManager->setShader(id);
+}
+
+int ModularBackend::getShader() const {
+	return _graphicsManager->getShader();
+}
+
 void ModularBackend::resetGraphicsScale() {
 	_graphicsManager->resetGraphicsScale();
 }
@@ -99,13 +111,18 @@ Common::List<Graphics::PixelFormat> ModularBackend::getSupportedFormats() const
 #endif
 
 // ResidualVM specific method
-void ModularBackend::launcherInitSize(uint w, uint h) {
-	_graphicsManager->launcherInitSize(w, h);
+void ModularBackend::setupScreen(uint screenW, uint screenH, bool fullscreen, bool accel3d) {
+	_graphicsManager->setupScreen(screenW, screenH, fullscreen, accel3d);
 }
 
 // ResidualVM specific method
-Graphics::PixelBuffer ModularBackend::setupScreen(uint screenW, uint screenH, bool fullscreen, bool accel3d) {
-	return _graphicsManager->setupScreen(screenW, screenH, fullscreen, accel3d);
+Graphics::PixelBuffer ModularBackend::getScreenPixelBuffer() {
+	return _graphicsManager->getScreenPixelBuffer();
+}
+
+// ResidualVM specific method
+void ModularBackend::suggestSideTextures(Graphics::Surface *left, Graphics::Surface *right) {
+	_graphicsManager->suggestSideTextures(left, right);
 }
 
 void ModularBackend::initSize(uint w, uint h, const Graphics::PixelFormat *format ) {
@@ -257,6 +274,10 @@ void ModularBackend::displayMessageOnOSD(const char *msg) {
 	_graphicsManager->displayMessageOnOSD(msg);
 }
 
+void ModularBackend::displayActivityIconOnOSD(const Graphics::Surface *icon) {
+	_graphicsManager->displayActivityIconOnOSD(icon);
+}
+
 void ModularBackend::quit() {
 	exit(0);
 }
