@@ -27,11 +27,13 @@
 
 #ifdef MACOSX
 
-#include "backends/platform/sdl/macosx/macosx.h"
+#include "backends/audiocd/macosx/macosx-audiocd.h"
 #include "backends/mixer/doublebuffersdl/doublebuffersdl-mixer.h"
 #include "backends/platform/sdl/macosx/appmenu_osx.h"
+#include "backends/platform/sdl/macosx/macosx.h"
 #include "backends/updates/macosx/macosx-updates.h"
 #include "backends/taskbar/macosx/macosx-taskbar.h"
+#include "backends/platform/sdl/macosx/macosx_wrapper.h"
 
 #include "common/archive.h"
 #include "common/config-manager.h"
@@ -47,11 +49,14 @@ OSystem_MacOSX::OSystem_MacOSX()
 }
 
 void OSystem_MacOSX::init() {
+	// Use an iconless window on OS X, as we use a nicer external icon there.
+	_window = new SdlIconlessWindow();
+
 #if defined(USE_TASKBAR)
 	// Initialize taskbar manager
 	_taskbarManager = new MacOSXTaskbarManager();
 #endif
-	
+
 	// Invoke parent implementation of this method
 	OSystem_POSIX::init();
 }
@@ -101,12 +106,8 @@ void OSystem_MacOSX::addSysArchivesToSearchSet(Common::SearchSet &s, int priorit
 	}
 }
 
-void OSystem_MacOSX::setupIcon() {
-	// Don't set icon on OS X, as we use a nicer external icon there.
-}
-
 bool OSystem_MacOSX::hasFeature(Feature f) {
-	if (f == kFeatureDisplayLogFile)
+	if (f == kFeatureDisplayLogFile || f == kFeatureClipboardSupport || f == kFeatureOpenUrl)
 		return true;
 	return OSystem_POSIX::hasFeature(f);
 }
@@ -124,6 +125,21 @@ bool OSystem_MacOSX::displayLogFile() {
 	return err != noErr;
 }
 
+bool OSystem_MacOSX::hasTextInClipboard() {
+	return hasTextInClipboardMacOSX();
+}
+
+Common::String OSystem_MacOSX::getTextFromClipboard() {
+	return getTextFromClipboardMacOSX();
+}
+
+bool OSystem_MacOSX::openUrl(const Common::String &url) {
+	CFURLRef urlRef = CFURLCreateWithBytes (NULL, (UInt8*)url.c_str(), url.size(), kCFStringEncodingASCII, NULL);
+	OSStatus err = LSOpenCFURLRef(urlRef, NULL);
+	CFRelease(urlRef);
+	return err == noErr;
+}
+
 Common::String OSystem_MacOSX::getSystemLanguage() const {
 #if defined(USE_DETECTLANG) && defined(USE_TRANSLATION)
 	CFArrayRef availableLocalizations = CFBundleCopyBundleLocalizations(CFBundleGetMainBundle());
@@ -171,4 +187,17 @@ Common::String OSystem_MacOSX::getSystemLanguage() const {
 #endif // USE_DETECTLANG
 }
 
+Common::String OSystem_MacOSX::getScreenshotsPath() {
+	Common::String path = ConfMan.get("screenshotpath");
+	if (path.empty())
+		path = getDesktopPathMacOSX();
+	if (!path.empty() && !path.hasSuffix("/"))
+		path += "/";
+	return path;
+}
+
+AudioCDManager *OSystem_MacOSX::createAudioCDManager() {
+	return createMacOSXAudioCDManager();
+}
+
 #endif
