@@ -33,12 +33,24 @@ public:
 
 	virtual bool displayLogFile();
 
+	virtual bool hasTextInClipboard();
+	virtual Common::String getTextFromClipboard();
+
+	virtual bool openUrl(const Common::String &url);
+
 	virtual Common::String getSystemLanguage() const;
 
 	virtual void init();
 	virtual void initBackend();
 	virtual void addSysArchivesToSearchSet(Common::SearchSet &s, int priority = 0);
-	virtual void setupIcon();
+
+	//Screenshots
+	virtual Common::String getScreenshotsPath();
+
+protected:
+	// Override createAudioCDManager() to get our Mac-specific
+	// version.
+	virtual AudioCDManager *createAudioCDManager();
 };
 
 #endif
