@@ -28,16 +28,20 @@
 class OSystem_POSIX : public OSystem_SDL {
 public:
 	// Let the subclasses be able to change _baseConfigName in the constructor
-	OSystem_POSIX(Common::String baseConfigName = ".residualvmrc");
+	OSystem_POSIX(Common::String baseConfigName = "residualvm.ini");
 	virtual ~OSystem_POSIX() {}
 
 	virtual bool hasFeature(Feature f);
 
 	virtual bool displayLogFile();
 
+	virtual bool openUrl(const Common::String &url);
+
 	virtual void init();
 	virtual void initBackend();
 
+	virtual void addSysArchivesToSearchSet(Common::SearchSet &s, int priority = 0);
+
 protected:
 	/**
 	 * Base string for creating the default path and filename for the
@@ -59,6 +63,10 @@ protected:
 	virtual Common::String getDefaultConfigFileName();
 
 	virtual Common::WriteStream *createLogFile();
+
+	virtual AudioCDManager *createAudioCDManager();
+
+	bool launchBrowser(const Common::String& client, const Common::String &url);
 };
 
 #endif
