@@ -31,7 +31,6 @@
 #include "backends/saves/default/default-saves.h"
 #include "backends/fs/ps3/ps3-fs-factory.h"
 #include "backends/events/ps3sdl/ps3sdl-events.h"
-#include "backends/mixer/sdl13/sdl13-mixer.h"
 
 #include <dirent.h>
 #include <sys/stat.h>
@@ -68,14 +67,6 @@ void OSystem_PS3::initBackend() {
 	if (_savefileManager == 0)
 		_savefileManager = new DefaultSaveFileManager(PREFIX "/saves");
 
-	// Create the mixer manager
-	if (_mixer == 0) {
-		_mixerManager = new Sdl13MixerManager();
-
-		// Setup and start mixer
-		_mixerManager->init();
-	}
-
 	// Event source
 	if (_eventSource == 0)
 		_eventSource = new PS3SdlEventSource();
