@@ -29,6 +29,8 @@
 #include "backends/mixer/sdl/sdl-mixer.h"
 #include "backends/events/sdl/sdl-events.h"
 #include "backends/log/log.h"
+#include "backends/platform/sdl/sdl-window.h"
+#include "backends/graphics/sdl/resvm-sdl-graphics.h"
 
 #include "common/array.h"
 
@@ -54,6 +56,8 @@ public:
 	 */
 	virtual SdlMixerManager *getMixerManager();
 
+	virtual bool hasFeature(Feature f);
+
 	// Override functions from ModularBackend and OSystem
 	virtual void initBackend();
 #if defined(USE_TASKBAR)
@@ -68,6 +72,10 @@ public:
 
 	virtual Common::String getSystemLanguage() const;
 
+	// Clipboard
+	virtual bool hasTextInClipboard();
+	virtual Common::String getTextFromClipboard();
+
 	virtual void setWindowCaption(const char *caption);
 	virtual void addSysArchivesToSearchSet(Common::SearchSet &s, int priority = 0);
 	virtual uint32 getMillis(bool skipRecord = false);
@@ -75,10 +83,22 @@ public:
 	virtual void getTimeAndDate(TimeDate &td) const;
 	virtual Audio::Mixer *getMixer();
 	virtual Common::TimerManager *getTimerManager();
+	virtual Common::SaveFileManager *getSavefileManager();
+
+	//Screenshots
+	virtual Common::String getScreenshotsPath();
+
+	// ResidualVM specific code
+	virtual void setupScreen(uint screenW, uint screenH, bool fullscreen, bool accel3d) override;
+	// ResidualVM specific code
+	virtual void launcherInitSize(uint w, uint h) override;
 
 protected:
 	bool _inited;
 	bool _initedSDL;
+#ifdef USE_SDL_NET
+	bool _initedSDLnet;
+#endif
 
 	/**
 	 * Mixer manager that configures and setups SDL for
@@ -91,6 +111,18 @@ protected:
 	 */
 	SdlEventSource *_eventSource;
 
+	/**
+	 * The SDL output window.
+	 */
+	SdlWindow *_window;
+
+	// ResidualVM specific code
+	// Graphics capabilities
+	void detectDesktopResolution();
+	void detectFramebufferSupport();
+	ResVmSdlGraphicsManager::Capabilities _capabilities;
+	// End of ResidualVM specific code
+
 	virtual Common::EventSource *getDefaultEventSource() { return _eventSource; }
 
 	/**
@@ -99,9 +131,9 @@ protected:
 	virtual void initSDL();
 
 	/**
-	 * Setup the window icon.
+	 * Create the audio CD manager
 	 */
-	virtual void setupIcon();
+	virtual AudioCDManager *createAudioCDManager();
 
 	// Logging
 	virtual Common::WriteStream *createLogFile() { return 0; }
