@@ -42,6 +42,8 @@
 
 #include "audio/musicplugin.h"
 
+#include "graphics/renderer.h" // ResidualVM
+
 #define DETECTOR_TESTING_HACK
 #define UPGRADE_ALL_TARGETS_HACK
 
@@ -57,7 +59,7 @@ static const char USAGE_STRING[] =
 ;
 
 // DONT FIXME: DO NOT ORDER ALPHABETICALLY, THIS IS ORDERED BY IMPORTANCE/CATEGORY! :)
-#if defined(__SYMBIAN32__) || defined(__GP32__) || defined(ANDROID) || defined(__DS__)
+#if defined(__SYMBIAN32__) || defined(__GP32__) || defined(ANDROID) || defined(__DS__) || defined(__3DS__)
 static const char HELP_STRING[] = "NoUsageString"; // save more data segment space
 #else
 static const char HELP_STRING[] =
@@ -68,6 +70,9 @@ static const char HELP_STRING[] =
 	"  -z, --list-games         Display list of supported games and exit\n"
 	"  -t, --list-targets       Display list of configured targets and exit\n"
 	"  --list-saves=TARGET      Display a list of saved games for the game (TARGET) specified\n"
+	"  --auto-detect            Display a list of games from current or specified directory\n"
+	"                           and start the first one. Use --path=PATH before --auto-detect\n"
+	"                           to specify a directory.\n"
 #if defined(WIN32) && !defined(_WIN32_WCE) && !defined(__SYMBIAN32__)
 	"  --console                Enable the console window (default:enabled)\n"
 #endif
@@ -94,11 +99,13 @@ static const char HELP_STRING[] =
 	"  -d, --debuglevel=NUM     Set debug verbosity level\n"
 	"  --debugflags=FLAGS       Enable engine specific debug flags\n"
 	"                           (separated by commas)\n"
+	"  --debug-channels-only    Show only the specified debug channels\n"
 	"  -u, --dump-scripts       Enable script dumping if a directory called 'dumps'\n"
 	"                           exists in the current directory\n"
 	"\n"
-	"  --cdrom=NUM              CD drive to play CD audio from (default: 0 = first\n"
-	"                           drive)\n"
+	"  --cdrom=DRIVE            CD drive to play CD audio from; can either be a\n"
+	"                           drive, path, or numeric index (default: 0 = best\n"
+	"                           choice drive)\n"
 	"  --joystick[=NUM]         Enable joystick input (default: 0 = first joystick)\n"
 	"  --platform=WORD          Specify platform of game (allowed values: 2gs, 3do,\n"
 	"                           acorn, amiga, atari, c64, fmtowns, nes, mac, pc, pc98,\n"
@@ -115,9 +122,12 @@ static const char HELP_STRING[] =
 	"  --talkspeed=NUM          Set talk speed for games (default: 179)\n"
 	"  --show-fps               Set the turn on display FPS info\n"
 	"  --no-show-fps            Set the turn off display FPS info\n"
-	"  --soft-renderer          Switch to 3D software renderer\n"
-	"  --no-soft-renderer       Switch to 3D hardware renderer\n"
+	"  --renderer=RENDERER      Select renderer (software, opengl, opengl_shaders)\n"
 	"  --aspect-ratio           Enable aspect ratio correction\n"
+	"  --bpp=NUM                Select number of bits per pixel, 0 (auto-detect), 16, 32\n"
+	"                           (default: 0) (only supported by software renderer)\n"
+	"  --[no-]dirtyrects        Enable dirty rectangles optimisation in software renderer\n"
+	"                           (default: enabled)\n"
 #ifdef ENABLE_EVENTRECORDER
 	"  --record-mode=MODE       Specify record mode for event recorder (record, playback,\n"
 	"                           passthrough [default])\n"
@@ -130,6 +140,8 @@ static const char HELP_STRING[] =
 	"  --dimuse-tempo=NUM       Set internal Digital iMuse tempo (10 - 100) per second\n"
 	"                           (default: 10)\n"
 #endif
+	"  --engine-speed=NUM       Set frame per second limit (0 - 100), 0 = no limit\n"
+	"                           (default: 60)\n"
 ;
 #endif
 
@@ -158,9 +170,11 @@ void registerDefaults() {
 
 	// Graphics
 	ConfMan.registerDefault("fullscreen", false);
-	ConfMan.registerDefault("soft_renderer", false);
+	ConfMan.registerDefault("filtering", false);
 	ConfMan.registerDefault("show_fps", false);
 	ConfMan.registerDefault("aspect_ratio", false);
+	ConfMan.registerDefault("dirtyrects", true);
+	ConfMan.registerDefault("bpp", 0);
 
 	// Sound & Music
 	ConfMan.registerDefault("music_volume", 192);
@@ -200,6 +214,7 @@ void registerDefaults() {
 #ifdef ENABLE_GRIM
 	ConfMan.registerDefault("dimuse_tempo", 10);
 #endif
+	ConfMan.registerDefault("engine_speed", 60);
 
 	// Miscellaneous
 	ConfMan.registerDefault("joystick_num", -1);
@@ -339,6 +354,12 @@ Common::String parseCommandLine(Common::StringMap &settings, int argc, const cha
 			// We defer checking whether this is a valid target to a later point.
 			return s;
 		} else {
+			// On MacOS X prior to 10.9 the OS is sometimes adding a -psn_X_XXXXXX argument (where X are digits)
+			// to pass the process serial number. We need to ignore it to avoid an error.
+#ifdef MACOSX
+			if (strncmp(s, "-psn_", 5) == 0)
+				continue;
+#endif
 
 			bool isLongCmd = (s[0] == '-' && s[1] == '-');
 
@@ -354,6 +375,9 @@ Common::String parseCommandLine(Common::StringMap &settings, int argc, const cha
 			DO_COMMAND('z', "list-games")
 			END_COMMAND
 
+			DO_LONG_COMMAND("auto-detect")
+			END_COMMAND
+
 #ifdef DETECTOR_TESTING_HACK
 			// HACK FIXME TODO: This command is intentionally *not* documented!
 			DO_LONG_COMMAND("test-detector")
@@ -385,6 +409,9 @@ Common::String parseCommandLine(Common::StringMap &settings, int argc, const cha
 			DO_LONG_OPTION("debugflags")
 			END_OPTION
 
+			DO_LONG_OPTION_BOOL("debug-channels-only")
+			END_OPTION
+
 			DO_OPTION('e', "music-driver")
 			END_OPTION
 
@@ -397,6 +424,9 @@ Common::String parseCommandLine(Common::StringMap &settings, int argc, const cha
 			DO_OPTION_BOOL('f', "fullscreen")
 			END_OPTION
 
+			DO_LONG_OPTION_BOOL("filtering")
+			END_OPTION
+
 #ifdef ENABLE_EVENTRECORDER
 			DO_LONG_OPTION_INT("disable-display")
 			END_OPTION
@@ -472,9 +502,6 @@ Common::String parseCommandLine(Common::StringMap &settings, int argc, const cha
 			DO_LONG_OPTION_BOOL("disable-sdl-parachute")
 			END_OPTION
 
-			DO_LONG_OPTION("engine-speed")
-			END_OPTION
-
 			DO_LONG_OPTION_BOOL("multi-midi")
 			END_OPTION
 
@@ -487,13 +514,23 @@ Common::String parseCommandLine(Common::StringMap &settings, int argc, const cha
 			DO_LONG_OPTION_BOOL("aspect-ratio")
 			END_OPTION
 
-			DO_LONG_OPTION("gamma")
+			DO_LONG_OPTION_INT("bpp")
 			END_OPTION
 
-			DO_LONG_OPTION_BOOL("soft-renderer")
+			DO_LONG_OPTION_BOOL("dirtyrects")
+			END_OPTION
+
+			DO_LONG_OPTION("gamma")
+			END_OPTION
+// ResidualVM specific start
+			DO_LONG_OPTION("renderer")
+				Graphics::RendererType renderer = Graphics::parseRendererTypeCode(option);
+				if (renderer == Graphics::kRendererTypeDefault)
+					usage("Unrecognized renderer type '%s'", option);
 			END_OPTION
 
 			DO_LONG_OPTION_BOOL("show-fps")
+// ResidualVM specific end
 			END_OPTION
 
 			DO_LONG_OPTION("savepath")
@@ -541,6 +578,9 @@ Common::String parseCommandLine(Common::StringMap &settings, int argc, const cha
 #endif
 
 
+			DO_LONG_OPTION_INT("engine-speed")
+			END_OPTION
+
 			DO_LONG_OPTION("speech-mode")
 			END_OPTION
 
@@ -703,6 +743,34 @@ static void listAudioDevices() {
 	}
 }
 
+/** Display all games in the given directory, or current directory if empty */
+static bool autoDetect(Common::String path) {
+	if (path.empty())
+		path = ".";
+	//Current directory
+	Common::FSNode dir(path);
+
+	Common::FSList files;
+
+	//Collect all files from directory
+	dir.getChildren(files, Common::FSNode::kListAll);
+
+	GameList candidates(EngineMan.detectGames(files));
+	if (candidates.empty()) {
+		printf("ScummVM could not find any game in %s\n", path.c_str());
+		return false;
+	}
+
+	// Print all the candidate found
+	printf("ID                   Description\n");
+	printf("-------------------- ---------------------------------------------------------\n");
+	for (GameList::iterator v = candidates.begin(); v != candidates.end(); ++v) {
+		printf("%-20s %s\n", v->gameid().c_str(), v->description().c_str());
+	}
+	// Set the active domain to the first one to start it.
+	ConfMan.setActiveDomain(candidates.begin()->gameid());
+	return true;
+}
 
 #ifdef DETECTOR_TESTING_HACK
 static void runDetectorTest() {
@@ -929,6 +997,10 @@ bool processSettings(Common::String &command, Common::StringMap &settings, Commo
 	} else if (command == "help") {
 		printf(HELP_STRING, s_appName);
 		return true;
+	} else if (command == "auto-detect") {
+		// If auto-detects succeed, we want to return false so that the game is started
+		return !autoDetect(settings["path"]);
+		//return true;
 	}
 #ifdef DETECTOR_TESTING_HACK
 	else if (command == "test-detector") {
