@@ -51,13 +51,13 @@
  * header file, analog to internal_version.h, maybe called svn_rev.h or so.)
  *
  * Drawback: This only works on systems which can run suitable scripts as part
- * of the build proces (so I guess Visual C++ would be out of the game here?
+ * of the build process (so I guess Visual C++ would be out of the game here?
  * I don't know VC enough to be sure). And of course it must be robust enough
  * to properly work in exports (i.e. release tar balls etc.).
  */
 const char *gScummVMVersion = SCUMMVM_VERSION;
 #ifdef __amigaos4__
-static const char *version_cookie __attribute__((used)) = "$VER: ResidualVM " SCUMMVM_VERSION " (" __DATE__ ", " __TIME__ ")";
+static const char *version_cookie __attribute__((used)) = "$VER: ResidualVM " SCUMMVM_VERSION " (" AMIGA_DATE ")";
 #endif
 #ifdef __PLAYSTATION2__
 const char *gScummVMBuildDate = "Git Master"; /* ScummVM Git Master */
@@ -153,5 +153,26 @@ const char *gScummVMFeatures = ""
 
 #ifdef ENABLE_VKEYBD
 	"virtual keyboard "
+#endif
+
+#ifdef USE_CLOUD
+	"cloud ("
+#ifdef USE_LIBCURL
+	"servers"
+#ifdef USE_SDL_NET
+	", "
+#endif
+#endif
+#ifdef USE_SDL_NET
+	"local"
+#endif
+	") "
+#else
+#ifdef USE_LIBCURL
+	"libcurl "
+#endif
+#ifdef USE_SDL_NET
+	"SDL_net "
+#endif
 #endif
 	;
