@@ -23,13 +23,17 @@
 #ifndef COMMON_SCUMMSYS_H
 #define COMMON_SCUMMSYS_H
 
-#ifndef __has_feature         // Optional of course.
-  #define __has_feature(x) 0  // Compatibility with non-clang compilers.
+#ifndef __has_feature           // Optional of course.
+	#define __has_feature(x) 0  // Compatibility with non-clang compilers.
 #endif
 
 // This is a convenience macro to test whether the compiler used is a GCC
 // version, which is at least major.minor.
-#define GCC_ATLEAST(major, minor) (defined(__GNUC__) && (__GNUC__ > (major) || (__GNUC__ == (major) && __GNUC_MINOR__ >= (minor))))
+#ifdef __GNUC__
+	#define GCC_ATLEAST(major, minor) (__GNUC__ > (major) || (__GNUC__ == (major) && __GNUC_MINOR__ >= (minor)))
+#else
+	#define GCC_ATLEAST(major, minor) 0
+#endif
 
 #if defined(_WIN32_WCE) && _WIN32_WCE < 300
 	#define NONSTANDARD_PORT
@@ -46,7 +50,7 @@
 
 	#if defined(WIN32)
 
-		#ifdef _MSC_VER
+		#if defined(_MSC_VER) && _MSC_VER <= 1800
 
 		// FIXME: The placement of the workaround functions for MSVC below
 		// require us to include stdio.h and stdarg.h for MSVC here. This
@@ -128,6 +132,7 @@
 	#include <stdlib.h>
 	#include <string.h>
 	#include <stdarg.h>
+	#include <stddef.h>
 	#include <assert.h>
 	#include <ctype.h>
 	// MSVC does not define M_PI, M_SQRT2 and other math defines by default.
@@ -148,6 +153,19 @@
 	#endif
 #endif
 
+#ifndef STATIC_ASSERT
+	/**
+	 * Generates a compile-time assertion.
+	 *
+	 * @param expression An expression that can be evaluated at compile time.
+	 * @param message An underscore-delimited message to be presented at compile
+	 * time if the expression evaluates to false.
+	 */
+	#define STATIC_ASSERT(expression, message) \
+		extern int STATIC_ASSERT_##message[(expression) ? 1 : -1]; \
+		(void)(STATIC_ASSERT_##message);
+#endif
+
 // The following math constants are usually defined by the system math.h header, but
 // they are not part of the ANSI C++ standards and so can NOT be relied upon to be
 // present i.e. when -std=c++11 is passed to GCC, enabling strict ANSI compliance.
@@ -251,6 +269,7 @@
 
 	#if defined(__DC__) || \
 		  defined(__DS__) || \
+		  defined(__3DS__) || \
 		  defined(__GP32__) || \
 		  defined(IPHONE) || \
 		  defined(__PLAYSTATION2__) || \
@@ -290,6 +309,25 @@
 	#endif
 #endif
 
+//
+// Determine 64 bitness
+// Reference: http://nadeausoftware.com/articles/2012/02/c_c_tip_how_detect_processor_type_using_compiler_predefined_macros
+//
+#if !defined(HAVE_CONFIG_H)
+
+	#if defined(__x86_64__) || \
+		  defined(_M_X64) || \
+		  defined(__ppc64__) || \
+		  defined(__powerpc64__) || \
+		  defined(__LP64__)
+
+		#if !defined(SCUMM_64BITS)
+			#define SCUMM_64BITS
+		#endif
+
+	#endif
+
+#endif
 
 //
 // Some more system specific settings.
@@ -367,11 +405,11 @@
 #endif
 
 #ifndef STRINGBUFLEN
-  #if defined(__N64__) || defined(__DS__)
-    #define STRINGBUFLEN 256
-  #else
-    #define STRINGBUFLEN 1024
-  #endif
+	#if defined(__N64__) || defined(__DS__) || defined(__3DS__)
+		#define STRINGBUFLEN 256
+	#else
+		#define STRINGBUFLEN 1024
+	#endif
 #endif
 
 // ResidualVM specific:
