@@ -103,6 +103,14 @@ public:
 		flush();
 	}
 
+	/**
+	* Obtains the current value of the stream position indicator of the
+	* stream.
+	*
+	* @return the current position indicator, or -1 if an error occurred.
+	 */
+	virtual int32 pos() const = 0;
+
 
 	// The remaining methods all have default implementations; subclasses
 	// need not (and should not) overload them.
@@ -125,6 +133,13 @@ public:
 		write(&value, 4);
 	}
 
+#ifdef HAVE_INT64
+	void writeUint64LE(uint64 value) {
+		value = TO_LE_64(value);
+		write(&value, 8);
+	}
+#endif
+
 	void writeUint16BE(uint16 value) {
 		value = TO_BE_16(value);
 		write(&value, 2);
@@ -135,6 +150,13 @@ public:
 		write(&value, 4);
 	}
 
+#ifdef HAVE_INT64
+	void writeUint64BE(uint64 value) {
+		value = TO_BE_64(value);
+		write(&value, 8);
+	}
+#endif
+
 	FORCEINLINE void writeSint16LE(int16 value) {
 		writeUint16LE((uint16)value);
 	}
@@ -143,6 +165,12 @@ public:
 		writeUint32LE((uint32)value);
 	}
 
+#ifdef HAVE_INT64
+	FORCEINLINE void writeSint64LE(int64 value) {
+		writeUint64LE((uint64)value);
+	}
+#endif
+
 	FORCEINLINE void writeSint16BE(int16 value) {
 		writeUint16BE((uint16)value);
 	}
@@ -151,6 +179,38 @@ public:
 		writeUint32BE((uint32)value);
 	}
 
+#ifdef HAVE_INT64
+	FORCEINLINE void writeSint64BE(int64 value) {
+		writeUint64BE((uint64)value);
+	}
+#endif
+
+
+	/**
+	 * Write the given 32-bit floating point value stored
+	 * in little endian(LSB first) order into the stream.
+	 */
+	FORCEINLINE void writeFloatLE(float value) {
+		uint32 n;
+
+		memcpy(&n, &value, 4);
+
+		writeUint32LE(n);
+	}
+
+
+	/**
+	 * Write the given 32-bit floating point value stored
+	 * in big endian order into the stream.
+	 */
+	FORCEINLINE void writeFloatBE(float value) {
+		uint32 n;
+
+		memcpy(&n, &value, 4);
+
+		writeUint32BE(n);
+	}
+
 	/**
 	 * Write the given string to the stream.
 	 * This writes str.size() characters, but no terminating zero byte.
@@ -241,6 +301,21 @@ public:
 		return FROM_LE_32(val);
 	}
 
+#ifdef HAVE_INT64
+	/**
+	 * Read an unsigned 64-bit word stored in little endian (LSB first) order
+	 * from the stream and return it.
+	 * Performs no error checking. The return value is undefined
+	 * if a read error occurred (for which client code can check by
+	 * calling err() and eos() ).
+	 */
+	uint64 readUint64LE() {
+		uint64 val;
+		read(&val, 8);
+		return FROM_LE_64(val);
+	}
+#endif
+
 	/**
 	 * Read an unsigned 16-bit word stored in big endian (MSB first) order
 	 * from the stream and return it.
@@ -267,6 +342,21 @@ public:
 		return FROM_BE_32(val);
 	}
 
+#ifdef HAVE_INT64
+	/**
+	 * Read an unsigned 64-bit word stored in big endian (MSB first) order
+	 * from the stream and return it.
+	 * Performs no error checking. The return value is undefined
+	 * if a read error occurred (for which client code can check by
+	 * calling err() and eos() ).
+	 */
+	uint64 readUint64BE() {
+		uint64 val;
+		read(&val, 8);
+		return FROM_BE_64(val);
+	}
+#endif
+
 	/**
 	 * Read a signed 16-bit word stored in little endian (LSB first) order
 	 * from the stream and return it.
@@ -289,6 +379,19 @@ public:
 		return (int32)readUint32LE();
 	}
 
+#ifdef HAVE_INT64
+	/**
+	 * Read a signed 64-bit word stored in little endian (LSB first) order
+	 * from the stream and return it.
+	 * Performs no error checking. The return value is undefined
+	 * if a read error occurred (for which client code can check by
+	 * calling err() and eos() ).
+	 */
+	FORCEINLINE int64 readSint64LE() {
+		return (int64)readUint64LE();
+	}
+#endif
+
 	/**
 	 * Read a signed 16-bit word stored in big endian (MSB first) order
 	 * from the stream and return it.
@@ -311,6 +414,51 @@ public:
 		return (int32)readUint32BE();
 	}
 
+#ifdef HAVE_INT64
+	/**
+	 * Read a signed 64-bit word stored in big endian (MSB first) order
+	 * from the stream and return it.
+	 * Performs no error checking. The return value is undefined
+	 * if a read error occurred (for which client code can check by
+	 * calling err() and eos() ).
+	 */
+	FORCEINLINE int64 readSint64BE() {
+		return (int64)readUint64BE();
+	}
+#endif
+
+	/**
+	 * Read a 32-bit floating point value stored in little endian (LSB first)
+	 * order from the stream and return it.
+	 * Performs no error checking. The return value is undefined
+	 * if a read error occurred (for which client code can check by
+	 * calling err() and eos() ).
+	 */
+	FORCEINLINE float readFloatLE() {
+		uint32 n = readUint32LE();
+		float f;
+
+		memcpy(&f, &n, 4);
+
+		return f;
+	}
+
+	/**
+	 * Read a 32-bit floating point value stored in big endian
+	 * order from the stream and return it.
+	 * Performs no error checking. The return value is undefined
+	 * if a read error occurred (for which client code can check by
+	 * calling err() and eos() ).
+	 */
+	FORCEINLINE float readFloatBE() {
+		uint32 n = readUint32BE();
+		float f;
+
+		memcpy(&f, &n, 4);
+
+		return f;
+	}
+
 	/**
 	 * Read the specified amount of data into a malloc'ed buffer
 	 * which then is wrapped into a MemoryReadStream.
@@ -321,6 +469,14 @@ public:
 	 */
 	SeekableReadStream *readStream(uint32 dataSize);
 
+	/**
+	 * Read stream in Pascal format, that is, one byte is
+	 * string length, followed by string data
+	 *
+	 * @param transformCR	if set (default), then transform \r into \n
+	 */
+	Common::String readPascalString(bool transformCR = true);
+
 };
 
 
@@ -408,6 +564,15 @@ public:
 	 * err() or eos() to determine whether an exception occurred.
 	 */
 	virtual String readLine();
+
+	/**
+	 * Print a hexdump of the stream while maintaing position. The number
+	 * of bytes per line is customizable.
+	 * @param len	the length of that data
+	 * @param bytesPerLine	number of bytes to print per line (default: 16)
+	 * @param startOffset	shift the shown offsets by the starting offset (default: 0)
+	 */
+	void hexdump(int len, int bytesPerLine = 16, int startOffset = 0);
 };
 
 /**
@@ -435,6 +600,14 @@ public:
 		return (_bigEndian) ? TO_BE_32(val) : TO_LE_32(val);
 	}
 
+#ifdef HAVE_INT64
+	uint64 readUint64() {
+		uint64 val;
+		read(&val, 8);
+		return (_bigEndian) ? TO_BE_64(val) : TO_LE_64(val);
+	}
+#endif
+
 	FORCEINLINE int16 readSint16() {
 		return (int16)readUint16();
 	}
@@ -442,6 +615,12 @@ public:
 	FORCEINLINE int32 readSint32() {
 		return (int32)readUint32();
 	}
+
+#ifdef HAVE_INT64
+	FORCEINLINE int64 readSint64() {
+		return (int64)readUint64();
+	}
+#endif
 };
 
 /**
