@@ -287,7 +287,7 @@ sub begin_section {
 			# headlines...
 			my $ascii_title = html_entities_to_ascii($title);
 			$title = html_entities_to_cpp($title);
-			if ($ascii_title ne $title) {	
+			if ($ascii_title ne $title) {
 				print '"A1""'.$ascii_title.'",' . "\n";
 			}
 			print '"C1""'.$title.'",' . "\n";
@@ -295,7 +295,7 @@ sub begin_section {
 		} else {
 			my $ascii_title = html_entities_to_ascii($title);
 			$title = html_entities_to_cpp($title);
-			if ($ascii_title ne $title) {	
+			if ($ascii_title ne $title) {
 				print '"A1""'.$ascii_title.'",' . "\n";
 			}
 			print '"C1""'.$title.'",' . "\n";
@@ -428,7 +428,7 @@ sub add_person {
 		if (length $desc > 0) {
 			my $ascii_desc = html_entities_to_ascii($desc);
 			$desc = html_entities_to_cpp($desc);
-			if ($ascii_desc ne $desc) {	
+			if ($ascii_desc ne $desc) {
 				print '"A2""'.$ascii_desc.'",' . "\n";
 			}
 			print '"C2""'.$desc.'",' . "\n";
@@ -505,6 +505,11 @@ begin_credits("Credits");
 	    begin_section("Myst 3 Team");
 		add_person("Bastien Bouclet", "bgK", "Core developer");
 	    end_section();
+
+	    begin_section("Stark Team");
+		add_person("Bastien Bouclet", "bgK", "Core developer");
+		add_person("Einar Johan T. S&oslash;m&aring;en", "somaen", "Core developer (retired)");
+	    end_section();
 	end_section();
 
 	begin_section("Contributors");
@@ -517,6 +522,7 @@ begin_credits("Credits");
 		add_person("Ori Avtalion", "salty-horse", "Lipsync, LAF support, various code fixes");
 		add_person("Robert Biro", "DarthJDG", "Antialiasing support");
 		add_person("Bastien Bouclet", "bgK", "Various fixes to engine");
+		add_person("David Cardwell", "d356", "Few fixes to EMI");
 		add_person("Marcus Comstedt", "marcus_c", "Initial Dreamcast port");
 		add_person("Andrea Corna", "Yak Bizzarro", "Patcher module, various engine improvements");
 		add_person("Jonathan Gray", "khalek", "Various code fixes");
@@ -524,7 +530,7 @@ begin_credits("Credits");
 		add_person("Azamat H. Hackimov", "winterheart", "Configure fix");
 		add_person("Vincent Hamm", "yazoo", "Various engine code fixes and improvements");
 		add_person("Sven Hesse", "DrMcCoy", "Various compilation fixes");
-		add_person("Matthew Hoops", "clone2727", "Some engine improvements, some work on EMI support");
+		add_person("Matthew Hoops", "clone2727", "Smush codec48, Grim and EMI engine improvements");
 		add_person("Erich Hoover", "Compholio", "x86-64 fixes, various code fixes and improvements");
 		add_person("Max Horn", "fingolfin", "Few code fixes");
 		add_person("Travis Howell", "Kirben", "Various code fixes, Windows port");
@@ -549,6 +555,7 @@ begin_credits("Credits");
 		add_person("", "cmayer0087", "Various engine code fixes");
 		add_person("", "JenniBee", "Compilation fixes");
 		add_person("", "karjonas", "Various engine code fixes");
+		add_person("", "PoulpiFr", "Few fixes to Android port");
 		add_person("", "sietschie", "Few fixes to engine");
 	    end_section();
 
@@ -556,6 +563,14 @@ begin_credits("Credits");
 		add_person("Matthew Hoops", "clone2727", "Various engine improvements and code fixes");
 		add_person("Stefano Musumeci", "subr3v", "TinyGL engine driver support");
 	    end_section();
+
+	    begin_section("Stark");
+		add_person("Matthew Hoops", "clone2727", "ADPCM decoder");
+		add_person("Pawe&#322; Ko&#322;odziejski", "aquadran", "Various engine code fixes");
+		add_person("Vincent Pelletier", "vpelletier", "Raw sound support");
+		add_person("Jordi Vilalta Prat", "jvprat", "Initial engine contributor");
+		add_person("Scott Thomas", "ST", "Initial engine author");
+	    end_section();
 	end_section();
 
     end_section();
@@ -563,7 +578,7 @@ begin_credits("Credits");
     begin_section("ScummVM code");
 	  add_paragraph(
 	  "ResidualVM uses a lot of ScummVM code. ".
-	  "For a list of authors look into COPYRIGHT file. ");
+	  "For a list of authors look into COPYRIGHT file.");
     end_section();
 
   begin_section("Website code");
@@ -578,7 +593,7 @@ begin_credits("Credits");
   begin_section("Special thanks to");
 
 	  add_paragraph(
-	  "The LUA developers, for creating a nice compact script interpreter. ");
+	  "The LUA developers, for creating a nice compact script interpreter.");
 
 	  add_paragraph(
 	  "Tim Schafer, for obvious reasons, and everybody else who helped make ".
@@ -590,7 +605,7 @@ begin_credits("Credits");
 	  "and avoiding the horrible hack it could have been.");
 
 	  add_paragraph(
-	  "Benjamin Haisch, for emimeshviewer, which our EMI code borrows heavily from. ");
+	  "Benjamin Haisch, for emimeshviewer, which our EMI code borrows heavily from.");
 
   end_section();
 
