@@ -41,6 +41,7 @@ class Set;
 class Material;
 struct SetShadow;
 struct Joint;
+class EMIModel;
 
 struct Plane {
 	Common::String setName;
@@ -364,6 +365,14 @@ public:
 	void putInSet(const Common::String &setName);
 	/**
 	 * Returns true if the actor is in the given set.
+	 * For engine internal use only, do not expose via lua API.
+	 *
+	 * @param setName The name of the set.
+	 */
+	bool isDrawableInSet(const Common::String &setName) const;
+	/**
+	 * Returns true if the actor is in the given set.
+	 * Can be exposed via lua API.
 	 *
 	 * @param setName The name of the set.
 	 */
@@ -538,8 +547,8 @@ public:
 	float getGlobalAlpha() const { return _globalAlpha; }
 	AlphaMode getAlphaMode() const { return _alphaMode; }
 	float getEffectiveAlpha() const { return _alphaMode != AlphaOff ? _globalAlpha : 1.f; }
-	void setGlobalAlpha(float alpha) { _globalAlpha = alpha; }
-	void setAlphaMode(AlphaMode mode) { _alphaMode = mode; }
+	void setGlobalAlpha(float alpha, const Common::String &mesh);
+	void setAlphaMode(AlphaMode mode, const Common::String &mesh);
 
 	int getSortOrder() const;
 	void setSortOrder(const int order);
@@ -548,9 +557,7 @@ public:
 	void activateShadow(bool active, const char *shadowName);
 	void activateShadow(bool active, SetShadow *shadow);
 
-	void restoreCleanBuffer();
 	void drawToCleanBuffer();
-	void clearCleanBuffer();
 
 	bool isTalkingForeground() const;
 
@@ -587,6 +594,7 @@ private:
 	void calculateOrientation(const Math::Vector3d &pos, Math::Angle *pitch, Math::Angle *yaw, Math::Angle *roll);
 
 	bool getSphereInfo(bool adjustZ, float &size, Math::Vector3d &pos) const;
+	EMIModel *findModelWithMesh(const Common::String &mesh);
 
 	Common::String _name;
 	Common::String _setName;    // The actual current set
@@ -713,7 +721,7 @@ private:
 	int _sectorSortOrder;
 	bool _useParentSortOrder;
 
-	int _cleanBuffer;
+	bool _fakeUnbound;
 	bool _drawnToClean;
 
 	LightMode _lightMode;
