@@ -232,7 +232,10 @@ BitmapData::~BitmapData() {
 }
 
 void BitmapData::freeData() {
-	if (!_keepData) {
+	if (!_keepData && _data) {
+		for (int i = 0; i < _numImages; ++i) {
+			_data[i].free();
+		}
 		delete[] _data;
 		_data = nullptr;
 	}
@@ -288,7 +291,7 @@ bool BitmapData::loadTile(Common::SeekableReadStream *o) {
 	for (uint32 i = 0; i < _numCoords * 4; ++i) {
 		char f[4];
 		o->read(f, 4);
-		_texc[i] = get_float(f);
+		_texc[i] = READ_LE_FLOAT(f);
 	}
 
 	_layers = new Layer[_numLayers];
@@ -306,8 +309,6 @@ bool BitmapData::loadTile(Common::SeekableReadStream *o) {
 
 	o->seek(16, SEEK_CUR);
 	int numSubImages = o->readUint32LE();
-	if (numSubImages < 5)
-		error("Can not handle a tile with less than 5 sub images");
 
 	char **data = new char *[numSubImages];
 
@@ -451,6 +452,11 @@ int Bitmap::getNumImages() const {
 	return _data->_numImages;
 }
 
+int Bitmap::getNumLayers() const {
+	_data->load();
+	return _data->_numLayers;
+}
+
 void Bitmap::freeData() {
 	--_data->_refCount;
 	if (_data->_refCount < 1) {
@@ -473,14 +479,7 @@ void BitmapData::convertToColorFormat(int num, const Graphics::PixelFormat &form
 	}
 
 	Graphics::PixelBuffer dst(format, _width * _height, DisposeAfterUse::NO);
-
-	for (int i = 0; i < _width * _height; ++i) {
-		if (_data[num].getValueAt(i) == 0xf81f) { //transparency
-			dst.setPixelAt(i, 0xf81f);
-		} else {
-			dst.setPixelAt(i, _data[num]);
-		}
-	}
+	dst.copyBuffer(0, _width * _height, _data[num]);
 	_data[num].free();
 	_data[num] = dst;
 }
