@@ -8,12 +8,12 @@
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
-
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
@@ -27,21 +27,20 @@
 
 namespace Grim {
 
-struct Sprite;
+class Sprite;
 
 class SpriteComponent : public Component {
 public:
 	SpriteComponent(Component *parent, int parentID, const char *filename, tag32 tag);
 	~SpriteComponent();
 
-	void init();
-	void setKey(int val);
-	void reset();
-	void saveState(SaveGame *state);
-	void restoreState(SaveGame *state);
+	void init() override;
+	void setKey(int val) override;
+	void reset() override;
+	void saveState(SaveGame *state) override;
+	void restoreState(SaveGame *state) override;
 
 private:
-	Common::String _filename;
 	Sprite *_sprite;
 };
 
