@@ -37,7 +37,7 @@ public:
 	bool cmd_checkFiles(int argc, const char **argv);
 	bool cmd_lua_do(int argc, const char **argv);
 	bool cmd_jump(int argc, const char **argv);
-	bool cmd_swap_renderer(int argc, const char **argv);
+	bool cmd_set_renderer(int argc, const char **argv);
 	bool cmd_save(int argc, const char **argv);
 	bool cmd_load(int argc, const char **argv);
 };
