@@ -455,7 +455,7 @@ void Lua_V2::FileFindFirst() {
 	// _menus.lua: saveload_menu.get_file_list searches for *.gsv.
 	// This avoids conflicts with grim saves.
 	if (0 == strcmp(extension, "*.gsv"))
-		extension = "efmi???.gsv";
+		extension = "efmi###.gsv";
 
 	Common::SaveFileManager *saveFileMan = g_system->getSavefileManager();
 	g_grim->_listFiles = saveFileMan->listSavefiles(extension);
@@ -477,15 +477,19 @@ void Lua_V2::ThumbnailFromFile() {
 		return;
 	}
 	int index = (int)lua_getnumber(texIdObj);
-	const char *filename = lua_getstring(filenameObj);
+	Common::String filename(lua_getstring(filenameObj));
+	
+	if (g_grim->getGameType() == GType_MONKEY4 &&
+		g_grim->getGamePlatform() == Common::kPlatformPS2) {
+		filename += ".ps2";
+	}
 
 	int width = 256, height = 128;
-	Bitmap *screenshot;
 
 	SaveGame *savedState = SaveGame::openForLoading(filename);
 	if (!savedState || !savedState->isCompatible()) {
 		delete savedState;
-		warning("Lua_V2::ThumbnailFromFile: savegame %s not compatible", filename);
+		warning("Lua_V2::ThumbnailFromFile: savegame %s not compatible", filename.c_str());
 		lua_pushnil();
 		return;
 	}
@@ -501,10 +505,11 @@ void Lua_V2::ThumbnailFromFile() {
 		data[l] = savedState->readLEUint16();
 	}
 	Graphics::PixelBuffer buf(Graphics::createPixelFormat<565>(), (byte *)data);
-	screenshot = new Bitmap(buf, width, height, "screenshot");
+	Bitmap *screenshot = new Bitmap(buf, width, height, "screenshot");
 	if (!screenshot) {
 		lua_pushnil();
-		warning("Lua_V2::ThumbnailFromFile: Could not restore screenshot from file %s", filename);
+		warning("Lua_V2::ThumbnailFromFile: Could not restore screenshot from file %s", filename.c_str());
+		delete screenshot;
 		delete[] data;
 		delete savedState;
 		return;
@@ -512,6 +517,7 @@ void Lua_V2::ThumbnailFromFile() {
 
 	screenshot->_data->convertToColorFormat(Graphics::PixelFormat(4, 8, 8, 8, 8, 0, 8, 16, 24));
 	g_driver->createSpecialtyTexture(index, screenshot->getData(0).getRawBuffer(), width, height);
+	delete screenshot;
 	delete[] data;
 	savedState->endSection();
 	delete savedState;
@@ -521,8 +527,10 @@ void Lua_V2::ThumbnailFromFile() {
 
 void Lua_V2::GetMemoryCardId() {
 	// 0 - No mem card
-	lua_pushnumber(0);
-	warning("GetMemoryCardId: Currently just returning 0");
+	// 1 - Not formatted
+	// 2 - Not enough space
+	// 3 - Error occured
+	lua_pushnumber(4);
 }
 
 void Lua_V2::SetReplayMode() {
@@ -668,11 +676,13 @@ void Lua_V2::SetLayerFrame() {
 void Lua_V2::SetLayerSortOrder() {
 	lua_Object param1 = lua_getparam(1);
 	lua_Object param2 = lua_getparam(2);
-	if (lua_isuserdata(param1) && lua_tag(param1) != MKTAG('L','A','Y','R') && lua_isnumber(param2)) {
+	if (lua_isuserdata(param1) && lua_tag(param1) == MKTAG('L','A','Y','R') && lua_isnumber(param2)) {
 		int layer = (int)lua_getuserdata(param1);
 		int sortorder = (int)lua_getnumber(param2);
 		Layer *l = Layer::getPool().getObject(layer);
 		l->setSortOrder(sortorder);
+	} else {
+		warning("Lua_V2::SetLayerSortOrder: wrong parameters");
 	}
 }
 
