@@ -105,7 +105,13 @@ void EMIModel::loadMesh(Common::SeekableReadStream *data) {
 
 	char f[4];
 	data->read(f, 4);
-	_radius = get_float(f);
+	for (uint l = 0; l < nameString.size(); ++l) {
+		if (nameString[l] == '\\') {
+			nameString.setChar('/', l);
+		}
+	}
+	_meshName = nameString;
+	_radius = READ_LE_FLOAT(f);
 	_center->readFromStream(data);
 
 	_boxData->readFromStream(data);
@@ -202,7 +208,7 @@ void EMIModel::loadMesh(Common::SeekableReadStream *data) {
 			_boneInfos[i]._incFac = data->readUint32LE();
 			_boneInfos[i]._joint = data->readUint32LE();
 			data->read(f, 4);
-			_boneInfos[i]._weight = get_float(f);
+			_boneInfos[i]._weight = READ_LE_FLOAT(f);
 		}
 	} else {
 		_numBones = 0;
@@ -424,6 +430,8 @@ Math::AABB EMIModel::calculateWorldBounds(const Math::Matrix4 &matrix) const {
 
 EMIModel::EMIModel(const Common::String &filename, Common::SeekableReadStream *data, EMICostume *costume) :
 		_fname(filename), _costume(costume) {
+	_meshAlphaMode = Actor::AlphaOff;
+	_meshAlpha = 1.0;
 	_numVertices = 0;
 	_vertices = nullptr;
 	_drawVertices = nullptr;
@@ -457,6 +465,8 @@ EMIModel::EMIModel(const Common::String &filename, Common::SeekableReadStream *d
 }
 
 EMIModel::~EMIModel() {
+	g_driver->destroyEMIModel(this);
+
 	delete[] _vertices;
 	delete[] _drawVertices;
 	delete[] _normals;
