@@ -26,7 +26,6 @@
 #include "math/vector3d.h"
 #include "math/quat.h"
 
-#include "graphics/pixelformat.h"
 #include "graphics/pixelbuffer.h"
 #include "common/str.h"
 #include "common/rect.h"
@@ -94,12 +93,7 @@ public:
 	* @return true if supports shaders, false otherwise
 	*/
 	virtual bool supportsShaders() = 0;
-	/**
-	 * Query whether the current context is fullscreen.
-	 *
-	 * @return true if fullscreen, false otherwise
-	 */
-	virtual bool isFullscreen() { return _isFullscreen; }
+
 	virtual uint getScreenWidth() { return _screenWidth; }
 	virtual uint getScreenHeight() { return _screenHeight; }
 
@@ -134,6 +128,7 @@ public:
 	virtual bool isShadowModeActive();
 	virtual void setShadowColor(byte r, byte g, byte b) = 0;
 	virtual void getShadowColor(byte *r, byte *g, byte *b) = 0;
+	virtual void destroyShadow(Shadow *shadow) {}
 
 	virtual void set3DMode() = 0;
 
@@ -266,13 +261,7 @@ public:
 	virtual void destroyMesh(const Mesh *mesh) {}
 	virtual void createEMIModel(EMIModel *model) {}
 	virtual void updateEMIModel(const EMIModel *model) {}
-
-	virtual int genBuffer() { return 0; }
-	virtual void delBuffer(int buffer) {}
-	virtual void selectBuffer(int buffer) {}
-	virtual void clearBuffer(int buffer) {}
-	virtual void drawBuffers() {}
-	virtual void refreshBuffers() {}
+	virtual void destroyEMIModel(EMIModel *model) {}
 
 	virtual void createSpecialtyTexture(uint id, const uint8 *data, int width, int height);
 	virtual void createSpecialtyTextureFromScreen(uint id, uint8 *data, int x, int y, int width, int height) = 0;
@@ -291,7 +280,6 @@ protected:
 	static const int _gameWidth = 640;
 	float _scaleW, _scaleH;
 	int _screenWidth, _screenHeight;
-	bool _isFullscreen;
 	Shadow *_currentShadowArray;
 	unsigned char _shadowColorR;
 	unsigned char _shadowColorG;
@@ -299,7 +287,6 @@ protected:
 	bool _renderBitmaps;
 	bool _renderZBitmaps;
 	bool _shadowModeActive;
-	Graphics::PixelFormat _pixelFormat;
 	Math::Vector3d _currentPos;
 	Math::Matrix4 _currentRot;
 	float _dimLevel;
@@ -308,6 +295,7 @@ protected:
 // Factory-like functions:
 
 GfxBase *CreateGfxOpenGL();
+GfxBase *CreateGfxOpenGLShader();
 GfxBase *CreateGfxTinyGL();
 
 extern GfxBase *g_driver;
