@@ -30,7 +30,7 @@
 #include "common/system.h"
 #include "common/config-manager.h"
 
-#if defined(USE_OPENGL) && !defined(USE_OPENGL_SHADERS)
+#if defined(USE_OPENGL) && !defined(USE_GLES2)
 
 #include "graphics/surface.h"
 #include "graphics/pixelbuffer.h"
@@ -52,7 +52,7 @@
 #include "engines/grim/registry.h"
 
 
-#if defined (SDL_BACKEND) && defined(GL_ARB_fragment_program)
+#if defined (SDL_BACKEND) && defined(GL_ARB_fragment_program) && !defined(USE_GLEW)
 
 // We need SDL.h for SDL_GL_GetProcAddress.
 #include "backends/platform/sdl/sdl-sys.h"
@@ -102,7 +102,6 @@ GfxOpenGL::GfxOpenGL() : _smushNumTex(0),
 		_useDepthShader(false), _fragmentProgram(0), _useDimShader(0),
 		_dimFragProgram(0), _maxLights(0), _storedDisplay(nullptr), 
 		_emergFont(0), _alpha(1.f) {
-	g_driver = this;
 	// GL_LEQUAL as glDepthFunc ensures that subsequent drawing attempts for
 	// the same triangles are not ignored by the depth test.
 	// That's necessary for EMI where some models have multiple faces which
@@ -124,17 +123,17 @@ GfxOpenGL::~GfxOpenGL() {
 	if (_useDimShader)
 		glDeleteProgramsARB(1, &_dimFragProgram);
 #endif
+	for (unsigned int i = 0; i < _numSpecialtyTextures; i++) {
+		destroyTexture(&_specialtyTextures[i]);
+	}
 }
 
 byte *GfxOpenGL::setupScreen(int screenW, int screenH, bool fullscreen) {
-	_pixelFormat = g_system->setupScreen(screenW, screenH, fullscreen, true).getFormat();
-
 	_screenWidth = screenW;
 	_screenHeight = screenH;
 	_scaleW = _screenWidth / (float)_gameWidth;
 	_scaleH = _screenHeight / (float)_gameHeight;
 
-	_isFullscreen = g_system->getFeatureState(OSystem::kFeatureFullscreenMode);
 	_useDepthShader = false;
 	_useDimShader = false;
 
@@ -142,9 +141,6 @@ byte *GfxOpenGL::setupScreen(int screenW, int screenH, bool fullscreen) {
 
 	g_system->setWindowCaption("ResidualVM: OpenGL Renderer");
 
-	// Load emergency built-in font
-	loadEmergFont();
-
 	int screenSize = _screenWidth * _screenHeight * 4;
 	_storedDisplay = new byte[screenSize];
 	memset(_storedDisplay, 0, screenSize);
@@ -155,8 +151,8 @@ byte *GfxOpenGL::setupScreen(int screenW, int screenH, bool fullscreen) {
 
 	GLfloat ambientSource[] = { 0.0f, 0.0f, 0.0f, 1.0f };
 	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambientSource);
-	GLfloat specularReflectance[] = { 0.3f, 0.3f, 0.3f, 1.0f };
-	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specularReflectance);
+	GLfloat diffuseReflectance[] = { 1.0f, 1.0f, 1.0f, 1.0f };
+	glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuseReflectance);
 
 	if (g_grim->getGameType() == GType_GRIM) {
 		glPolygonOffset(-6.0, -6.0);
@@ -174,6 +170,7 @@ void GfxOpenGL::initExtensions() {
 	}
 
 #if defined (SDL_BACKEND) && defined(GL_ARB_fragment_program)
+#ifndef USE_GLEW
 	union {
 		void *obj_ptr;
 		void (APIENTRY *func_ptr)();
@@ -191,9 +188,10 @@ void GfxOpenGL::initExtensions() {
 	glDeleteProgramsARB = (PFNGLDELETEPROGRAMSARBPROC)u.func_ptr;
 	u.obj_ptr = SDL_GL_GetProcAddress("glProgramLocalParameter4fARB");
 	glProgramLocalParameter4fARB = (PFNGLPROGRAMLOCALPARAMETER4FARBPROC)u.func_ptr;
+#endif
 
 	const char *extensions = (const char *)glGetString(GL_EXTENSIONS);
-	if (strstr(extensions, "ARB_fragment_program")) {
+	if (extensions && strstr(extensions, "ARB_fragment_program")) {
 		_useDepthShader = true;
 		_useDimShader = true;
 	}
@@ -756,6 +754,11 @@ void GfxOpenGL::drawEMIModelFace(const EMIModel *model, const EMIMeshFace *face)
 		glEnable(GL_BLEND);
 
 	glBegin(GL_TRIANGLES);
+	float alpha = _alpha;
+	if (model->_meshAlphaMode == Actor::AlphaReplace) {
+		alpha *= model->_meshAlpha;
+	}
+	Math::Vector3d noLighting(1.f, 1.f, 1.f);
 	for (uint j = 0; j < face->_faceLength * 3; j++) {
 		int index = indices[j];
 
@@ -763,11 +766,11 @@ void GfxOpenGL::drawEMIModelFace(const EMIModel *model, const EMIMeshFace *face)
 			if (face->_hasTexture) {
 				glTexCoord2f(model->_texVerts[index].getX(), model->_texVerts[index].getY());
 			}
-			Math::Vector3d lighting = model->_lighting[index];
+			Math::Vector3d lighting = (face->_flags & EMIMeshFace::kNoLighting) ? noLighting : model->_lighting[index];
 			byte r = (byte)(model->_colorMap[index].r * lighting.x());
 			byte g = (byte)(model->_colorMap[index].g * lighting.y());
 			byte b = (byte)(model->_colorMap[index].b * lighting.z());
-			byte a = (int)(model->_colorMap[index].a * _alpha * _currentActor->getLocalAlpha(index));
+			byte a = (int)(model->_colorMap[index].a * alpha * _currentActor->getLocalAlpha(index));
 			glColor4ub(r, g, b, a);
 		}
 
@@ -962,17 +965,17 @@ void GfxOpenGL::setupLight(Light *light, int lightId) {
 	}
 
 	glEnable(GL_LIGHTING);
-	GLfloat diffuse[] = { 0.0f, 0.0f, 0.0f, 1.0f };
-	GLfloat specular[] = { 0.0f, 0.0f, 0.0f, 1.0f };
+	GLfloat lightColor[] = { 0.0f, 0.0f, 0.0f, 1.0f };
 	GLfloat lightPos[] = { 0.0f, 0.0f, 0.0f, 1.0f };
 	GLfloat lightDir[] = { 0.0f, 0.0f, -1.0f };
 	GLfloat cutoff = 180.0f;
 	GLfloat spot_exp = 0.0f;
+	GLfloat q_attenuation = 1.0f;
 
-	GLfloat intensity = light->_intensity;
-	diffuse[0] = ((GLfloat)light->_color.getRed() / 15.0f) * intensity;
-	diffuse[1] = ((GLfloat)light->_color.getGreen() / 15.0f) * intensity;
-	diffuse[2] = ((GLfloat)light->_color.getBlue() / 15.0f) * intensity;
+	GLfloat intensity = light->_scaledintensity;
+	lightColor[0] = (GLfloat)light->_color.getRed() * intensity;
+	lightColor[1] = (GLfloat)light->_color.getGreen() * intensity;
+	lightColor[2] = (GLfloat)light->_color.getBlue() * intensity;
 
 	if (light->_type == Light::Omni) {
 		lightPos[0] = light->_pos.x();
@@ -990,21 +993,18 @@ void GfxOpenGL::setupLight(Light *light, int lightId) {
 		lightDir[0] = light->_dir.x();
 		lightDir[1] = light->_dir.y();
 		lightDir[2] = light->_dir.z();
-		specular[0] = diffuse[0];
-		specular[1] = diffuse[1];
-		specular[2] = diffuse[2];
 		spot_exp = 2.0f;
 		cutoff = light->_penumbraangle;
+		q_attenuation = 0.0f;
 	}
 
 	glDisable(GL_LIGHT0 + lightId);
-	glLightfv(GL_LIGHT0 + lightId, GL_DIFFUSE, diffuse);
-	glLightfv(GL_LIGHT0 + lightId, GL_SPECULAR, specular);
+	glLightfv(GL_LIGHT0 + lightId, GL_DIFFUSE, lightColor);
 	glLightfv(GL_LIGHT0 + lightId, GL_POSITION, lightPos);
 	glLightfv(GL_LIGHT0 + lightId, GL_SPOT_DIRECTION, lightDir);
 	glLightf(GL_LIGHT0 + lightId, GL_SPOT_EXPONENT, spot_exp);
 	glLightf(GL_LIGHT0 + lightId, GL_SPOT_CUTOFF, cutoff);
-	glLightf(GL_LIGHT0 + lightId, GL_QUADRATIC_ATTENUATION, 0.2f);
+	glLightf(GL_LIGHT0 + lightId, GL_QUADRATIC_ATTENUATION, q_attenuation);
 	glEnable(GL_LIGHT0 + lightId);
 }
 
@@ -1129,8 +1129,8 @@ void GfxOpenGL::createBitmap(BitmapData *bitmap) {
 void GfxOpenGL::drawBitmap(const Bitmap *bitmap, int dx, int dy, uint32 layer) {
 	// The PS2 version of EMI uses a TGA for it's splash-screen
 	// avoid using the TIL-code below for that, by checking
-	// numImages here:
-	if (g_grim->getGameType() == GType_MONKEY4 && bitmap->_data->_numImages > 1) {
+	// for texture coordinates set by BitmapData::loadTile
+	if (g_grim->getGameType() == GType_MONKEY4 && bitmap->_data && bitmap->_data->_texc) {
 		glMatrixMode(GL_MODELVIEW);
 		glPushMatrix();
 		glLoadIdentity();
@@ -1153,6 +1153,7 @@ void GfxOpenGL::drawBitmap(const Bitmap *bitmap, int dx, int dy, uint32 layer) {
 		GLuint *textures = (GLuint *)bitmap->getTexIds();
 		float *texc = data->_texc;
 
+		assert(layer < data->_numLayers);
 		uint32 offset = data->_layers[layer]._offset;
 		for (uint32 i = offset; i < offset + data->_layers[layer]._numImages; ++i) {
 			glBindTexture(GL_TEXTURE_2D, textures[data->_verts[i]._texid]);
@@ -1585,6 +1586,9 @@ void GfxOpenGL::prepareMovieFrame(Graphics::Surface *frame) {
 	if (frame->format == Graphics::PixelFormat(4, 8, 8, 8, 0, 8, 16, 24, 0)) {
 		format = GL_BGRA;
 		dataType = GL_UNSIGNED_INT_8_8_8_8;
+	} else if (frame->format == Graphics::PixelFormat(4, 8, 8, 8, 0, 16, 8, 0, 0)) {
+		format = GL_BGRA;
+		dataType = GL_UNSIGNED_INT_8_8_8_8_REV;
 	} else if (frame->format == Graphics::PixelFormat(2, 5, 6, 5, 0, 11, 5, 0, 0)) {
 		format = GL_RGB;
 		dataType = GL_UNSIGNED_SHORT_5_6_5;
@@ -1913,10 +1917,11 @@ void GfxOpenGL::irisAroundRegion(int x1, int y1, int x2, int y2) {
 		fx1, fy1
 	};
 
-	glEnableClientState(GL_VERTEX_ARRAY);
-	glVertexPointer(2, GL_FLOAT, 0, points);
-	glDrawArrays(GL_TRIANGLE_STRIP, 0, 10);
-	glDisableClientState(GL_VERTEX_ARRAY);
+	glBegin(GL_TRIANGLE_STRIP);
+	for (int i = 0 ;i < 10; ++i) {
+		glVertex2fv(points+2*i);
+	}
+	glEnd();
 
 	glColor3f(1.0f, 1.0f, 1.0f);
 	glEnable(GL_DEPTH_TEST);
@@ -1951,33 +1956,12 @@ void GfxOpenGL::drawRectangle(const PrimitiveObject *primitive) {
 		glVertex2f(x1, y2 + 1);
 		glEnd();
 	} else {
-		glBegin(GL_QUADS);
-
-		// top line
+		glLineWidth(_scaleW);
+		glBegin(GL_LINE_LOOP);
 		glVertex2f(x1, y1);
 		glVertex2f(x2 + 1, y1);
-		glVertex2f(x2 + 1, y1 + 1);
-		glVertex2f(x1, y1 + 1);
-
-
-		// right line
-		glVertex2f(x2, y1);
-		glVertex2f(x2 + 1, y1);
-		glVertex2f(x2 + 1, y2 + 1);
-		glVertex2f(x2, y2);
-
-		// bottom line
-		glVertex2f(x1, y2);
-		glVertex2f(x2 + 1, y2);
 		glVertex2f(x2 + 1, y2 + 1);
 		glVertex2f(x1, y2 + 1);
-
-		// left line
-		glVertex2f(x1, y1);
-		glVertex2f(x1 + 1, y1);
-		glVertex2f(x1 + 1, y2 + 1);
-		glVertex2f(x1, y2);
-
 		glEnd();
 	}
 
@@ -2085,9 +2069,6 @@ void GfxOpenGL::drawPolygon(const PrimitiveObject *primitive) {
 	glBegin(GL_LINES);
 	glVertex2f(x1, y1);
 	glVertex2f(x2 + 1, y2 + 1);
-	glEnd();
-
-	glBegin(GL_LINES);
 	glVertex2f(x3, y3 + 1);
 	glVertex2f(x4 + 1, y4);
 	glEnd();
