@@ -27,7 +27,9 @@
 
 #ifdef USE_OPENGL
 
-#if defined (SDL_BACKEND) && !defined(__amigaos4__)
+#ifdef USE_GLEW
+#include <GL/glew.h>
+#elif defined (SDL_BACKEND) && !defined(__amigaos4__)
 #include <SDL_opengl.h>
 #undef ARRAYSIZE
 #else
