@@ -48,7 +48,7 @@
 #include "common/system.h"
 #include "common/textconsole.h"
 
-#ifdef USE_OPENGL_SHADERS
+#if defined(USE_GLES2) || defined(USE_OPENGL_SHADERS)
 
 #include "graphics/surface.h"
 #include "graphics/pixelbuffer.h"
@@ -110,7 +110,7 @@ struct GrimVertex {
 };
 
 struct TextUserData {
-	Graphics::Shader * shader;
+	OpenGL::Shader * shader;
 	uint32 characters;
 	Color  color;
 	GLuint texture;
@@ -122,7 +122,7 @@ struct FontUserData {
 };
 
 struct EMIModelUserData {
-	Graphics::Shader *_shader;
+	OpenGL::Shader *_shader;
 	uint32 _texCoordsVBO;
 	uint32 _colorMapVBO;
 	uint32 _verticesVBO;
@@ -130,7 +130,7 @@ struct EMIModelUserData {
 };
 
 struct ModelUserData {
-	Graphics::Shader *_shader;
+	OpenGL::Shader *_shader;
 	uint32 _meshInfoVBO;
 };
 
@@ -205,7 +205,7 @@ Math::Matrix4 makeFrustumMatrix(double left, double right, double bottom, double
 	return proj;
 }
 
-GfxBase *CreateGfxOpenGL() {
+GfxBase *CreateGfxOpenGLShader() {
 	return new GfxOpenGLS();
 }
 
@@ -239,6 +239,9 @@ GfxOpenGLS::GfxOpenGLS() {
 }
 
 GfxOpenGLS::~GfxOpenGLS() {
+	for (unsigned int i = 0; i < _numSpecialtyTextures; i++) {
+		destroyTexture(&_specialtyTextures[i]);
+	}
 	delete[] _lights;
 
 	delete _backgroundProgram;
@@ -254,6 +257,7 @@ GfxOpenGLS::~GfxOpenGLS() {
 	delete _dimPlaneProgram;
 	delete _dimRegionProgram;
 	glDeleteTextures(1, &_storedDisplay);
+	glDeleteTextures(1, &_emergTexture);
 }
 
 void GfxOpenGLS::setupZBuffer() {
@@ -287,11 +291,11 @@ void GfxOpenGLS::setupQuadEBO() {
 		p[5] = start++;
 	}
 
-	_quadEBO = Graphics::Shader::createBuffer(GL_ELEMENT_ARRAY_BUFFER, sizeof(quad_indices), quad_indices, GL_STATIC_DRAW);
+	_quadEBO = OpenGL::Shader::createBuffer(GL_ELEMENT_ARRAY_BUFFER, sizeof(quad_indices), quad_indices, GL_STATIC_DRAW);
 }
 
 void GfxOpenGLS::setupTexturedQuad() {
-	_smushVBO = Graphics::Shader::createBuffer(GL_ARRAY_BUFFER, sizeof(textured_quad), textured_quad, GL_STATIC_DRAW);
+	_smushVBO = OpenGL::Shader::createBuffer(GL_ARRAY_BUFFER, sizeof(textured_quad), textured_quad, GL_STATIC_DRAW);
 	_smushProgram->enableVertexAttribute("position", _smushVBO, 2, GL_FLOAT, GL_FALSE, 4 * sizeof(float), 0);
 	_smushProgram->enableVertexAttribute("texcoord", _smushVBO, 2, GL_FLOAT, GL_FALSE, 4 * sizeof(float), 2 * sizeof(float));
 
@@ -307,7 +311,7 @@ void GfxOpenGLS::setupTexturedQuad() {
 }
 
 void GfxOpenGLS::setupTexturedCenteredQuad() {
-	_spriteVBO = Graphics::Shader::createBuffer(GL_ARRAY_BUFFER, sizeof(textured_quad_centered), textured_quad_centered, GL_STATIC_DRAW);
+	_spriteVBO = OpenGL::Shader::createBuffer(GL_ARRAY_BUFFER, sizeof(textured_quad_centered), textured_quad_centered, GL_STATIC_DRAW);
 	_spriteProgram->enableVertexAttribute("position", _spriteVBO, 3, GL_FLOAT, GL_FALSE, 5 * sizeof(float), 0);
 	_spriteProgram->enableVertexAttribute("texcoord", _spriteVBO, 2, GL_FLOAT, GL_FALSE, 5 * sizeof(float), 3 * sizeof(float));
 	_spriteProgram->disableVertexAttribute("color", Math::Vector4d(1.0f, 1.0f, 1.0f, 1.0f));
@@ -322,6 +326,9 @@ void GfxOpenGLS::setupPrimitives() {
 		glBufferData(GL_ARRAY_BUFFER, 8 * sizeof(float), NULL, GL_DYNAMIC_DRAW);
 	}
 
+	if (g_grim->getGameType() == GType_MONKEY4)
+		return;
+
 	glGenBuffers(1, &_irisVBO);
 	glBindBuffer(GL_ARRAY_BUFFER, _irisVBO);
 	glBufferData(GL_ARRAY_BUFFER, 20 * sizeof(float), NULL, GL_DYNAMIC_DRAW);
@@ -366,48 +373,44 @@ void GfxOpenGLS::setupShaders() {
 	bool isEMI = g_grim->getGameType() == GType_MONKEY4;
 
 	static const char* commonAttributes[] = {"position", "texcoord", NULL};
-	_backgroundProgram = Graphics::Shader::fromFiles(isEMI ? "emi_background" : "grim_background", commonAttributes);
-	_smushProgram = Graphics::Shader::fromFiles("smush", commonAttributes);
-	_textProgram = Graphics::Shader::fromFiles("text", commonAttributes);
-	_emergProgram = Graphics::Shader::fromFiles("emerg", commonAttributes);
+	_backgroundProgram = OpenGL::Shader::fromFiles(isEMI ? "emi_background" : "grim_background", commonAttributes);
+	_smushProgram = OpenGL::Shader::fromFiles("smush", commonAttributes);
+	_textProgram = OpenGL::Shader::fromFiles("text", commonAttributes);
+	_emergProgram = OpenGL::Shader::fromFiles("emerg", commonAttributes);
 
 	static const char* actorAttributes[] = {"position", "texcoord", "color", "normal", NULL};
-	_actorProgram = Graphics::Shader::fromFiles(isEMI ? "emi_actor" : "grim_actor", actorAttributes);
-	_spriteProgram = _actorProgram->clone();
+	_actorProgram = OpenGL::Shader::fromFiles(isEMI ? "emi_actor" : "grim_actor", actorAttributes);
+	_spriteProgram = OpenGL::Shader::fromFiles(isEMI ? "emi_actor" : "grim_actor", actorAttributes);
 
 	static const char* primAttributes[] = { "position", NULL };
-	_shadowPlaneProgram = Graphics::Shader::fromFiles("shadowplane", primAttributes);
+	_shadowPlaneProgram = OpenGL::Shader::fromFiles("shadowplane", primAttributes);
+	_primitiveProgram = OpenGL::Shader::fromFiles("grim_primitive", primAttributes);
 
 	if (!isEMI) {
-		_primitiveProgram = Graphics::Shader::fromFiles("grim_primitive", primAttributes);
 		_irisProgram = _primitiveProgram->clone();
 
-		_dimProgram = Graphics::Shader::fromFiles("dim", commonAttributes);
+		_dimProgram = OpenGL::Shader::fromFiles("dim", commonAttributes);
 		_dimRegionProgram = _dimProgram->clone();
 	} else {
-		_dimPlaneProgram = Graphics::Shader::fromFiles("emi_dimplane", primAttributes);
+		_dimPlaneProgram = OpenGL::Shader::fromFiles("emi_dimplane", primAttributes);
 	}
 
 	setupQuadEBO();
 	setupTexturedQuad();
 	setupTexturedCenteredQuad();
+	setupPrimitives();
 
 	if (!isEMI) {
-		setupPrimitives();
-
-		_blastVBO = Graphics::Shader::createBuffer(GL_ARRAY_BUFFER, 128 * 16 * sizeof(float), NULL, GL_DYNAMIC_DRAW);
+		_blastVBO = OpenGL::Shader::createBuffer(GL_ARRAY_BUFFER, 128 * 16 * sizeof(float), NULL, GL_DYNAMIC_DRAW);
 	}
 }
 
 byte *GfxOpenGLS::setupScreen(int screenW, int screenH, bool fullscreen) {
-	_pixelFormat = g_system->setupScreen(screenW, screenH, fullscreen, true).getFormat();
-
 	_screenWidth = screenW;
 	_screenHeight = screenH;
 	_scaleW = _screenWidth / (float)_gameWidth;
 	_scaleH = _screenHeight / (float)_gameHeight;
 
-	_isFullscreen = g_system->getFeatureState(OSystem::kFeatureFullscreenMode);
 #ifdef USE_GLES2
 	g_system->setFeatureState(OSystem::kFeatureVirtControls, true);
 #endif
@@ -419,9 +422,6 @@ byte *GfxOpenGLS::setupScreen(int screenW, int screenH, bool fullscreen) {
 	setupZBuffer();
 	setupShaders();
 
-	// Load emergency built-in font
-	loadEmergFont();
-
 	glViewport(0, 0, _screenWidth, _screenHeight);
 
 	glGenTextures(1, &_storedDisplay);
@@ -700,11 +700,36 @@ void GfxOpenGLS::startActorDraw(const Actor *actor) {
 		}
 		_actorProgram->setUniform("normalMatrix", normalMatrix);
 
-		_actorProgram->setUniform("actorPos", pos);
 		_actorProgram->setUniform("isBillboard", GL_FALSE);
 		_actorProgram->setUniform("useVertexAlpha", GL_FALSE);
 		_actorProgram->setUniform("uniformColor", color);
 		_actorProgram->setUniform1f("alphaRef", 0.0f);
+		_actorProgram->setUniform1f("meshAlpha", 1.0f);
+
+		// set the uniform parameter for _spriteProgram
+		// since they are needed by emi_actor.{fragment,vertex}
+		// in drawSprite()
+		_spriteProgram->use();
+		_spriteProgram->setUniform("modelMatrix", modelMatrix);
+		if (actor->isInOverworld()) {
+			_spriteProgram->setUniform("viewMatrix", viewMatrix);
+			_spriteProgram->setUniform("projMatrix", _overworldProjMatrix);
+			_spriteProgram->setUniform("cameraPos", Math::Vector3d(0,0,0));
+		} else {
+			_spriteProgram->setUniform("viewMatrix", viewRot);
+			_spriteProgram->setUniform("projMatrix", _projMatrix);
+			_spriteProgram->setUniform("cameraPos", _currentPos);
+		}
+		_spriteProgram->setUniform("normalMatrix", normalMatrix);
+
+		_spriteProgram->setUniform("actorPos", pos);
+		_spriteProgram->setUniform("isBillboard", GL_FALSE);
+		_spriteProgram->setUniform("useVertexAlpha", GL_FALSE);
+		_spriteProgram->setUniform("uniformColor", color);
+		_spriteProgram->setUniform1f("alphaRef", 0.0f);
+		_spriteProgram->setUniform1f("meshAlpha", 1.0f);
+
+		_actorProgram->use();
 	} else {
 		Math::Matrix4 modelMatrix = quat.toMatrix();
 		bool hasZBuffer = g_grim->getCurrSet()->getCurrSetup()->_bkgndZBm;
@@ -713,19 +738,17 @@ void GfxOpenGLS::startActorDraw(const Actor *actor) {
 		modelMatrix.transpose();
 		modelMatrix.setPosition(pos);
 		modelMatrix.transpose();
-		_mvpMatrix = _viewMatrix * modelMatrix;
-		_mvpMatrix.transpose();
 
 		_actorProgram->setUniform("modelMatrix", modelMatrix);
 		_actorProgram->setUniform("viewMatrix", _viewMatrix);
 		_actorProgram->setUniform("projMatrix", _projMatrix);
 		_actorProgram->setUniform("extraMatrix", extraMatrix);
-		_actorProgram->setUniform("mvpMatrix", _mvpMatrix);
 		_actorProgram->setUniform("tex", 0);
 		_actorProgram->setUniform("texZBuf", 1);
 		_actorProgram->setUniform("hasZBuffer", hasZBuffer);
 		_actorProgram->setUniform("texcropZBuf", _zBufTexCrop);
 		_actorProgram->setUniform("screenSize", Math::Vector2d(_screenWidth, _screenHeight));
+		_actorProgram->setUniform1f("alphaRef", 0.5f);
 	}
 
 	if (_currentShadowArray) {
@@ -775,10 +798,8 @@ void GfxOpenGLS::startActorDraw(const Actor *actor) {
 			uniform = Common::String::format("lights[%u]._color", i);
 			_actorProgram->setUniform(uniform.c_str(), l._color);
 
-			if (g_grim->getGameType() == GType_MONKEY4) {
-				uniform = Common::String::format("lights[%u]._params", i);
-				_actorProgram->setUniform(uniform.c_str(), l._params);
-			}
+			uniform = Common::String::format("lights[%u]._params", i);
+			_actorProgram->setUniform(uniform.c_str(), l._params);
 		}
 	}
 }
@@ -805,7 +826,6 @@ void GfxOpenGLS::drawShadowPlanes() {
 	glEnable(GL_STENCIL_TEST);
 	glStencilFunc(GL_ALWAYS, 1, (GLuint)~0);
 	glStencilOp(GL_REPLACE, GL_REPLACE, GL_REPLACE);
-	glDisable(GL_TEXTURE_2D);
 
 	if (!_currentShadowArray->userData) {
 		uint32 numVertices = 0;
@@ -836,8 +856,8 @@ void GfxOpenGLS::drawShadowPlanes() {
 		ShadowUserData *sud = new ShadowUserData;
 		_currentShadowArray->userData = sud;
 		sud->_numTriangles = numTriangles;
-		sud->_verticesVBO = Graphics::Shader::createBuffer(GL_ARRAY_BUFFER, 3 * numVertices * sizeof(float), vertBuf, GL_STATIC_DRAW);
-		sud->_indicesVBO = Graphics::Shader::createBuffer(GL_ELEMENT_ARRAY_BUFFER, 3 * numTriangles * sizeof(uint16), idxBuf, GL_STATIC_DRAW);
+		sud->_verticesVBO = OpenGL::Shader::createBuffer(GL_ARRAY_BUFFER, 3 * numVertices * sizeof(float), vertBuf, GL_STATIC_DRAW);
+		sud->_indicesVBO = OpenGL::Shader::createBuffer(GL_ELEMENT_ARRAY_BUFFER, 3 * numTriangles * sizeof(uint16), idxBuf, GL_STATIC_DRAW);
 
 		delete[] vertBuf;
 		delete[] idxBuf;
@@ -888,6 +908,17 @@ void GfxOpenGLS::getShadowColor(byte *r, byte *g, byte *b) {
 	*b = _shadowColorB;
 }
 
+void GfxOpenGLS::destroyShadow(Shadow *shadow) {
+	ShadowUserData *sud = static_cast<ShadowUserData *>(shadow->userData);
+	if (sud) {
+		OpenGL::Shader::freeBuffer(sud->_verticesVBO);
+		OpenGL::Shader::freeBuffer(sud->_indicesVBO);
+		delete sud;
+	}
+
+	shadow->userData = nullptr;
+}
+
 void GfxOpenGLS::set3DMode() {
 
 }
@@ -933,9 +964,10 @@ void GfxOpenGLS::drawEMIModelFace(const EMIModel* model, const EMIMeshFace* face
 	mud->_shader->use();
 	bool textured = face->_hasTexture && !_currentShadowArray;
 	mud->_shader->setUniform("textured", textured ? GL_TRUE : GL_FALSE);
-	mud->_shader->setUniform("lightsEnabled", _lightsEnabled);
+	mud->_shader->setUniform("lightsEnabled", (face->_flags & EMIMeshFace::kNoLighting) ? false : _lightsEnabled);
 	mud->_shader->setUniform("swapRandB", _selectedTexture->_colorFormat == BM_BGRA || _selectedTexture->_colorFormat == BM_BGR888);
 	mud->_shader->setUniform("useVertexAlpha", _selectedTexture->_colorFormat == BM_BGRA);
+	mud->_shader->setUniform1f("meshAlpha", (model->_meshAlphaMode == Actor::AlphaReplace) ? model->_meshAlpha : 1.0f);
 
 	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, face->_indicesEBO);
 
@@ -947,16 +979,15 @@ void GfxOpenGLS::drawMesh(const Mesh *mesh) {
 	const ModelUserData *mud = (const ModelUserData *)mesh->_userData;
 	if (!mud)
 		return;
-	Graphics::Shader *actorShader = mud->_shader;
+	OpenGL::Shader *actorShader = mud->_shader;
 
 	actorShader->use();
 	actorShader->setUniform("extraMatrix", _matrixStack.top());
+	actorShader->setUniform("lightsEnabled", _lightsEnabled && !isShadowModeActive());
 
 	const Material *curMaterial = NULL;
 	for (int i = 0; i < mesh->_numFaces;) {
 		const MeshFace *face = &mesh->_faces[i];
-		if (face->getLight() == 0 && !isShadowModeActive())
-			disableLights();
 
 		curMaterial = face->getMaterial();
 		curMaterial->select();
@@ -973,9 +1004,6 @@ void GfxOpenGLS::drawMesh(const Mesh *mesh) {
 		actorShader->setUniform("texScale", Math::Vector2d(_selectedTexture->_width, _selectedTexture->_height));
 
 		glDrawArrays(GL_TRIANGLES, *(int *)face->_userData, faces);
-
-		if (face->getLight() == 0 && !isShadowModeActive())
-			enableLights();
 	}
 }
 
@@ -1088,19 +1116,10 @@ void GfxOpenGLS::setupLight(Grim::Light *light, int lightId) {
 	Math::Vector4d &lightDir    = _lights[lightId]._direction;
 	Math::Vector4d &lightParams = _lights[lightId]._params;
 
-	if (g_grim->getGameType() == GType_MONKEY4) {
-		float intensity = light->_intensity;
-		lightColor.x() = ((float)light->_color.getRed() / 255.0f) * intensity;
-		lightColor.y() = ((float)light->_color.getGreen() / 255.0f) * intensity;
-		lightColor.z() = ((float)light->_color.getBlue() / 255.0f) * intensity;
-		lightColor.w() = 1.0f;
-	} else {
-		float intensity = light->_intensity / 1.3f;
-		lightColor.x() = ((float)light->_color.getRed() / 15.0f) * intensity;
-		lightColor.y() = ((float)light->_color.getGreen() / 15.0f) * intensity;
-		lightColor.z() = ((float)light->_color.getBlue() / 15.0f) * intensity;
-		lightColor.w() = 1.0f;
-	}
+	lightColor.x() = (float)light->_color.getRed();
+	lightColor.y() = (float)light->_color.getGreen();
+	lightColor.z() = (float)light->_color.getBlue();
+	lightColor.w() = light->_scaledintensity;
 
 	if (light->_type == Grim::Light::Omni) {
 		lightPos = Math::Vector4d(light->_pos.x(), light->_pos.y(), light->_pos.z(), 1.0f);
@@ -1110,11 +1129,9 @@ void GfxOpenGLS::setupLight(Grim::Light *light, int lightId) {
 		lightPos = Math::Vector4d(-light->_dir.x(), -light->_dir.y(), -light->_dir.z(), 0.0f);
 		lightDir = Math::Vector4d(0.0f, 0.0f, 0.0f, -1.0f);
 	} else if (light->_type == Grim::Light::Spot) {
-		float cosPenumbra = cos(light->_penumbraangle);
-		float cosUmbra = cos(light->_umbraangle);
 		lightPos = Math::Vector4d(light->_pos.x(), light->_pos.y(), light->_pos.z(), 1.0f);
 		lightDir = Math::Vector4d(light->_dir.x(), light->_dir.y(), light->_dir.z(), 1.0f);
-		lightParams = Math::Vector4d(light->_falloffNear, light->_falloffFar, cosPenumbra, cosUmbra);
+		lightParams = Math::Vector4d(light->_falloffNear, light->_falloffFar, light->_cospenumbraangle, light->_cosumbraangle);
 	} else if (light->_type == Grim::Light::Ambient) {
 		lightPos = Math::Vector4d(0.0f, 0.0f, 0.0f, -1.0f);
 		lightDir = Math::Vector4d(0.0f, 0.0f, 0.0f, -1.0f);
@@ -1215,9 +1232,10 @@ void GfxOpenGLS::selectTexture(const Texture *texture) {
 
 void GfxOpenGLS::destroyTexture(Texture *texture) {
 	GLuint *textures = static_cast<GLuint *>(texture->_texture);
-	glDeleteTextures(1, textures);
-
-	delete[] textures;
+	if (textures) {
+		glDeleteTextures(1, textures);
+		delete[] textures;
+	}
 }
 
 void GfxOpenGLS::createBitmap(BitmapData *bitmap) {
@@ -1298,11 +1316,11 @@ void GfxOpenGLS::createBitmap(BitmapData *bitmap) {
 			delete[] texData;
 		bitmap->freeData();
 
-		Graphics::Shader *shader = _backgroundProgram->clone();
+		OpenGL::Shader *shader = _backgroundProgram->clone();
 		bitmap->_userData = shader;
 
 		if (g_grim->getGameType() == GType_MONKEY4) {
-			GLuint vbo = Graphics::Shader::createBuffer(GL_ARRAY_BUFFER, bitmap->_numCoords * 4 * sizeof(float), bitmap->_texc, GL_STATIC_DRAW);
+			GLuint vbo = OpenGL::Shader::createBuffer(GL_ARRAY_BUFFER, bitmap->_numCoords * 4 * sizeof(float), bitmap->_texc, GL_STATIC_DRAW);
 			shader->enableVertexAttribute("position", vbo, 2, GL_FLOAT, GL_FALSE, 4 * sizeof(float), 0);
 			shader->enableVertexAttribute("texcoord", vbo, 2, GL_FLOAT, GL_FALSE, 4 * sizeof(float), 2*sizeof(float));
 		}
@@ -1314,9 +1332,9 @@ void GfxOpenGLS::createBitmap(BitmapData *bitmap) {
 }
 
 void GfxOpenGLS::drawBitmap(const Bitmap *bitmap, int dx, int dy, uint32 layer) {
-	if (g_grim->getGameType() == GType_MONKEY4 && bitmap->_data->_numImages > 1) {
+	if (g_grim->getGameType() == GType_MONKEY4 && bitmap->_data && bitmap->_data->_texc) {
 		BitmapData *data = bitmap->_data;
-		Graphics::Shader *shader = (Graphics::Shader *)data->_userData;
+		OpenGL::Shader *shader = (OpenGL::Shader *)data->_userData;
 		GLuint *textures = (GLuint *)bitmap->getTexIds();
 
 		glDisable(GL_DEPTH_TEST);
@@ -1326,6 +1344,7 @@ void GfxOpenGLS::drawBitmap(const Bitmap *bitmap, int dx, int dy, uint32 layer)
 
 		shader->use();
 		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, _quadEBO);
+		assert(layer < data->_numLayers);
 		uint32 offset = data->_layers[layer]._offset;
 		for (uint32 i = offset; i < offset + data->_layers[layer]._numImages; ++i) {
 			glBindTexture(GL_TEXTURE_2D, textures[data->_verts[i]._texid]);
@@ -1351,7 +1370,7 @@ void GfxOpenGLS::drawBitmap(const Bitmap *bitmap, int dx, int dy, uint32 layer)
 			glDisable(GL_BLEND);
 		}
 
-		Graphics::Shader *shader = (Graphics::Shader *)bitmap->_data->_userData;
+		OpenGL::Shader *shader = (OpenGL::Shader *)bitmap->_data->_userData;
 		shader->use();
 		glDisable(GL_DEPTH_TEST);
 		glDepthMask(GL_FALSE);
@@ -1406,7 +1425,7 @@ void GfxOpenGLS::destroyBitmap(BitmapData *bitmap) {
 		delete[] textures;
 		bitmap->_texIds = 0;
 	}
-	Graphics::Shader *shader = (Graphics::Shader *)bitmap->_userData;
+	OpenGL::Shader *shader = (OpenGL::Shader *)bitmap->_userData;
 	if (g_grim->getGameType() == GType_MONKEY4) {
 		glDeleteBuffers(1, &shader->getAttributeAt(0)._vbo);
 	}
@@ -1566,10 +1585,10 @@ void GfxOpenGLS::createTextObject(TextObject *text) {
 		glBindBuffer(GL_ARRAY_BUFFER, vbo);
 		glBufferSubData(GL_ARRAY_BUFFER, 0, numCharacters * 16 * sizeof(float), bufData);
 	} else {
-		vbo = Graphics::Shader::createBuffer(GL_ARRAY_BUFFER, numCharacters * 16 * sizeof(float), bufData, GL_STATIC_DRAW);
+		vbo = OpenGL::Shader::createBuffer(GL_ARRAY_BUFFER, numCharacters * 16 * sizeof(float), bufData, GL_STATIC_DRAW);
 	}
 
-	Graphics::Shader * textShader = _textProgram->clone();
+	OpenGL::Shader * textShader = _textProgram->clone();
 	glBindBuffer(GL_ARRAY_BUFFER, vbo);
 
 	textShader->enableVertexAttribute("position", vbo, 2, GL_FLOAT, GL_FALSE, 4 * sizeof(float), 0);
@@ -1806,11 +1825,7 @@ void GfxOpenGLS::drawGenericPrimitive(const float *vertices, uint32 numVertices,
 
 	switch (primitive->getType()) {
 		case PrimitiveObject::RectangleType:
-			if (primitive->isFilled()) {
-				glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
-			} else {
-				glDrawArrays(GL_LINE_LOOP, 0, 4);
-			}
+			glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
 			break;
 		case PrimitiveObject::LineType:
 			glDrawArrays(GL_LINES, 0, 2);
@@ -1834,9 +1849,20 @@ void GfxOpenGLS::drawRectangle(const PrimitiveObject *primitive) {
 	float x2 = primitive->getP2().x * _scaleW;
 	float y2 = primitive->getP2().y * _scaleH;
 
-	float data[] = { x1, y1, x2, y1, x2, y2, x1, y2 };
+	if (primitive->isFilled()) {
+		float data[] = { x1, y1, x2 + 1, y1, x1, y2 + 1, x2 + 1, y2 + 1 };
+		drawGenericPrimitive(data, 8, primitive);
+	} else {
+		float top[] =    { x1, y1, x2 + 1, y1, x1, y1 + 1, x2 + 1, y1 + 1 };
+		float right[] =  { x2, y1, x2 + 1, y1, x2, y2 + 1, x2 + 1, y2 + 1 };
+		float bottom[] = { x1, y2, x2 + 1, y2, x1, y2 + 1, x2 + 1, y2 + 1 };
+		float left[] =   { x1, y1, x1 + 1, y1, x1, y2 + 1, x1 + 1, y2 + 1 };
+		drawGenericPrimitive(top, 8, primitive);
+		drawGenericPrimitive(right, 8, primitive);
+		drawGenericPrimitive(bottom, 8, primitive);
+		drawGenericPrimitive(left, 8, primitive);
+	}
 
-	drawGenericPrimitive(data, 8, primitive);
 }
 
 void GfxOpenGLS::drawLine(const PrimitiveObject *primitive) {
@@ -1860,7 +1886,7 @@ void GfxOpenGLS::drawPolygon(const PrimitiveObject *primitive) {
 	float x4 = primitive->getP4().x * _scaleW;
 	float y4 = primitive->getP4().y * _scaleH;
 
-	const float data[] = { x1, y1, x2, y2, x3, y3, x4, y4 };
+	const float data[] = { x1, y1, x2 + 1, y2 + 1, x3, y3 + 1, x4 + 1, y4 };
 
 	drawGenericPrimitive(data, 8, primitive);
 }
@@ -1872,20 +1898,52 @@ void GfxOpenGLS::prepareMovieFrame(Graphics::Surface* frame) {
 
 	GLenum frameType, frameFormat;
 
-	switch (frame->format.bytesPerPixel) {
-	case 2:
-		frameType = GL_UNSIGNED_SHORT_5_6_5;
-		frameFormat = GL_RGB;
-		_smushSwizzle = false;
-		break;
-	case 4:
+	// GLES2 support is needed here, so:
+	// - frameFormat GL_BGRA is not supported, so use GL_RGBA
+	// - no format conversion, so same format is used for internal storage, so swizzle in shader
+	// - GL_UNSIGNED_INT_8_8_8_8[_REV] do not exist, so use _BYTE and fix
+	//   endianness in shader.
+	if (frame->format == Graphics::PixelFormat(4, 8, 8, 8, 0, 8, 16, 24, 0) || frame->format == Graphics::PixelFormat(4, 8, 8, 8, 8, 8, 16, 24, 0)) {
+		// frame->format: GBRA
+		// read in little endian: {A, R, G, B}, swap: {B, G, R, A}, swizzle: {R, G, B, A}
+		// read in big endian: {B, G, R, A}, swizzle: {R, G, B, A}
 		frameType = GL_UNSIGNED_BYTE;
 		frameFormat = GL_RGBA;
 		_smushSwizzle = true;
-		break;
-	default:
-		error("Video decoder returned invalid pixel format!");
-		return;
+#ifdef SCUMM_LITTLE_ENDIAN
+		_smushSwap = true;
+#else
+		_smushSwap = false;
+#endif
+
+	} else if (frame->format == Graphics::PixelFormat(4, 8, 8, 8, 0, 16, 8, 0, 0) || frame->format == Graphics::PixelFormat(4, 8, 8, 8, 8, 16, 8, 0, 24)) {
+		// frame->format: ARGB
+		// read in little endian: {B, G, R, A}, swizzle: {R, G, B, A}
+		// read in big endian: {A, R, G, B}, swap: {B, G, R, A}, swizzle: {R, G, B, A}
+		frameType = GL_UNSIGNED_BYTE;
+		frameFormat = GL_RGBA;
+		_smushSwizzle = true;
+#ifdef SCUMM_LITTLE_ENDIAN
+		_smushSwap = false;
+#else
+		_smushSwap = true;
+#endif
+	} else if (frame->format == Graphics::PixelFormat(2, 5, 6, 5, 0, 11, 5, 0, 0)) {
+		frameType = GL_UNSIGNED_SHORT_5_6_5;
+		frameFormat = GL_RGB;
+		_smushSwizzle = false;
+		_smushSwap = false;
+	} else {
+		error("Unknown pixelformat: Bpp: %d RBits: %d GBits: %d BBits: %d ABits: %d RShift: %d GShift: %d BShift: %d AShift: %d",
+			frame->format.bytesPerPixel,
+			-(frame->format.rLoss - 8),
+			-(frame->format.gLoss - 8),
+			-(frame->format.bLoss - 8),
+			-(frame->format.aLoss - 8),
+			frame->format.rShift,
+			frame->format.gShift,
+			frame->format.bShift,
+			frame->format.aShift);
 	}
 
 	// create texture
@@ -1915,6 +1973,7 @@ void GfxOpenGLS::drawMovieFrame(int offsetX, int offsetY) {
 	_smushProgram->setUniform("texcrop", Math::Vector2d(float(_smushWidth) / nextHigher2(_smushWidth), float(_smushHeight) / nextHigher2(_smushHeight)));
 	_smushProgram->setUniform("scale", Math::Vector2d(float(_smushWidth)/ float(_gameWidth), float(_smushHeight) / float(_gameHeight)));
 	_smushProgram->setUniform("offset", Math::Vector2d(float(offsetX) / float(_gameWidth), float(offsetY) / float(_gameHeight)));
+	_smushProgram->setUniform("swap", _smushSwap);
 	_smushProgram->setUniform("swizzle", _smushSwizzle);
 	glBindTexture(GL_TEXTURE_2D, _smushTexId);
 
@@ -1949,15 +2008,15 @@ void GfxOpenGLS::renderZBitmaps(bool render) {
 void GfxOpenGLS::createEMIModel(EMIModel *model) {
 	EMIModelUserData *mud = new EMIModelUserData;
 	model->_userData = mud;
-	mud->_verticesVBO = Graphics::Shader::createBuffer(GL_ARRAY_BUFFER, model->_numVertices * 3 * sizeof(float), model->_vertices, GL_STREAM_DRAW);
+	mud->_verticesVBO = OpenGL::Shader::createBuffer(GL_ARRAY_BUFFER, model->_numVertices * 3 * sizeof(float), model->_vertices, GL_STREAM_DRAW);
 
-	mud->_normalsVBO = Graphics::Shader::createBuffer(GL_ARRAY_BUFFER, model->_numVertices * 3 * sizeof(float), model->_normals, GL_STREAM_DRAW);
+	mud->_normalsVBO = OpenGL::Shader::createBuffer(GL_ARRAY_BUFFER, model->_numVertices * 3 * sizeof(float), model->_normals, GL_STREAM_DRAW);
 
-	mud->_texCoordsVBO = Graphics::Shader::createBuffer(GL_ARRAY_BUFFER, model->_numVertices * 2 * sizeof(float), model->_texVerts, GL_STATIC_DRAW);
+	mud->_texCoordsVBO = OpenGL::Shader::createBuffer(GL_ARRAY_BUFFER, model->_numVertices * 2 * sizeof(float), model->_texVerts, GL_STATIC_DRAW);
 
-	mud->_colorMapVBO = Graphics::Shader::createBuffer(GL_ARRAY_BUFFER, model->_numVertices * 4 * sizeof(byte), model->_colorMap, GL_STATIC_DRAW);
+	mud->_colorMapVBO = OpenGL::Shader::createBuffer(GL_ARRAY_BUFFER, model->_numVertices * 4 * sizeof(byte), model->_colorMap, GL_STATIC_DRAW);
 
-	Graphics::Shader * actorShader = _actorProgram->clone();
+	OpenGL::Shader * actorShader = _actorProgram->clone();
 	actorShader->enableVertexAttribute("position", mud->_verticesVBO, 3, GL_FLOAT, GL_FALSE, 3 * sizeof(float), 0);
 	actorShader->enableVertexAttribute("normal", mud->_normalsVBO, 3, GL_FLOAT, GL_FALSE, 3 * sizeof(float), 0);
 	actorShader->enableVertexAttribute("texcoord", mud->_texCoordsVBO, 2, GL_FLOAT, GL_FALSE, 2 * sizeof(float), 0);
@@ -1966,12 +2025,34 @@ void GfxOpenGLS::createEMIModel(EMIModel *model) {
 
 	for (uint32 i = 0; i < model->_numFaces; ++i) {
 		EMIMeshFace * face = &model->_faces[i];
-		face->_indicesEBO = Graphics::Shader::createBuffer(GL_ELEMENT_ARRAY_BUFFER, face->_faceLength * 3 * sizeof(uint32), face->_indexes, GL_STATIC_DRAW);
+		face->_indicesEBO = OpenGL::Shader::createBuffer(GL_ELEMENT_ARRAY_BUFFER, face->_faceLength * 3 * sizeof(uint32), face->_indexes, GL_STATIC_DRAW);
 	}
 
 	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
 }
 
+void GfxOpenGLS::destroyEMIModel(EMIModel *model) {
+	for (uint32 i = 0; i < model->_numFaces; ++i) {
+		EMIMeshFace *face = &model->_faces[i];
+		OpenGL::Shader::freeBuffer(face->_indicesEBO);
+		face->_indicesEBO = 0;
+	}
+
+	EMIModelUserData *mud = static_cast<EMIModelUserData *>(model->_userData);
+
+	if (mud) {
+		OpenGL::Shader::freeBuffer(mud->_verticesVBO);
+		OpenGL::Shader::freeBuffer(mud->_normalsVBO);
+		OpenGL::Shader::freeBuffer(mud->_texCoordsVBO);
+		OpenGL::Shader::freeBuffer(mud->_colorMapVBO);
+
+		delete mud->_shader;
+		delete mud;
+	}
+
+	model->_userData = nullptr;
+}
+
 void GfxOpenGLS::createMesh(Mesh *mesh) {
 	Common::Array<GrimVertex> meshInfo;
 	meshInfo.reserve(mesh->_numVertices * 5);
@@ -2007,9 +2088,9 @@ void GfxOpenGLS::createMesh(Mesh *mesh) {
 	ModelUserData *mud = new ModelUserData;
 	mesh->_userData = mud;
 
-	mud->_meshInfoVBO = Graphics::Shader::createBuffer(GL_ARRAY_BUFFER, meshInfo.size() * sizeof(GrimVertex), &meshInfo[0], GL_STATIC_DRAW);
+	mud->_meshInfoVBO = OpenGL::Shader::createBuffer(GL_ARRAY_BUFFER, meshInfo.size() * sizeof(GrimVertex), &meshInfo[0], GL_STATIC_DRAW);
 
-	Graphics::Shader *shader = _actorProgram->clone();
+	OpenGL::Shader *shader = _actorProgram->clone();
 	mud->_shader = shader;
 	shader->enableVertexAttribute("position", mud->_meshInfoVBO, 3, GL_FLOAT, GL_FALSE, sizeof(GrimVertex), 0);
 	shader->enableVertexAttribute("texcoord", mud->_meshInfoVBO, 2, GL_FLOAT, GL_FALSE, sizeof(GrimVertex), 3 * sizeof(float));
@@ -2051,8 +2132,10 @@ Bitmap *GfxOpenGLS::getScreenshot(int w, int h, bool useStored) {
 		char *buffer = new char[_screenWidth * _screenHeight * 4];
 
 		glGetTexImage(GL_TEXTURE_2D, 0, GL_RGBA, GL_UNSIGNED_BYTE, buffer);
-		memcpy(src.getRawBuffer(), buffer, _screenWidth * _screenHeight * 4);
-
+		byte *rawBuf = src.getRawBuffer();
+		for (int i = 0; i < _screenHeight; i++) {
+			memcpy(&(rawBuf[(_screenHeight - i - 1) * _screenWidth * 4]), &buffer[4 * _screenWidth * i], _screenWidth * 4);
+		}
 		delete[] buffer;
 	} else
 #endif
