@@ -20,6 +20,7 @@
  *
  */
 
+#include "common/config-manager.h"
 #include "common/endian.h"
 #include "common/system.h"
 
@@ -43,100 +44,13 @@
 
 namespace Grim {
 
-/**
- * This class is used for blitting bitmaps with transparent pixels.
- * Instead of checking every pixel for transparency, it creates a list of 'lines'.
- * A line is, well, a line of non trasparent pixels, and itstores a pointer to the
- * first pixel, and the position of it, which can be used to memcpy the entire line
- * to the destination buffer.
- */
-class BlitImage {
-public:
-	BlitImage() {
-		_lines = nullptr;
-		_last = nullptr;
-		_width = 0;
-		_height = 0;
-	}
-	~BlitImage() {
-		Line *temp = _lines;
-		while (temp != nullptr) {
-			_lines = temp->next;
-			delete temp;
-			temp = _lines;
-		}
-	}
-	void create(const Graphics::PixelBuffer &buf, uint32 transparency, int x, int y, int width, int height) {
-		Graphics::PixelBuffer srcBuf = buf;
-		_width = width;
-		_height = height;
-		// A line of pixels can not wrap more that one line of the image, since it would break
-		// blitting of bitmaps with a non-zero x position.
-		for (int l = 0; l < height; l++) {
-			int start = -1;
-
-			for (int r = 0; r < width; ++r) {
-				// We found a transparent pixel, so save a line from 'start' to the pixel before this.
-				if (srcBuf.getValueAt(r) == transparency && start >= 0) {
-					newLine(start, l, r - start, srcBuf.getRawBuffer(start));
-
-					start = -1;
-				} else if (srcBuf.getValueAt(r) != transparency && start == -1) {
-					start = r;
-				}
-			}
-			// end of the bitmap line. if start is an actual pixel save the line.
-			if (start >= 0) {
-				newLine(start, l, width - start, srcBuf.getRawBuffer(start));
-			}
-
-			srcBuf.shiftBy(width);
-		}
-	}
-
-	void newLine(int x, int y, int length, byte *pixels) {
-		if (length < 1) {
-			return;
-		}
-
-		Line *line = new Line;
-
-		line->x = x;
-		line->y = y;
-		line->length = length;
-		line->pixels = pixels;
-		line->next = nullptr;
-
-		if (_last) {
-			_last->next = line;
-		}
-		if (!_lines) {
-			_lines = line;
-		}
-		_last = line;
-	}
-
-	struct Line {
-		int x;
-		int y;
-		int length;
-		byte *pixels;
-
-		Line *next;
-	};
-	Line *_lines;
-	Line *_last;
-	int _width, _height;
-};
-
 GfxBase *CreateGfxTinyGL() {
 	return new GfxTinyGL();
 }
 
 GfxTinyGL::GfxTinyGL() :
-		_smushWidth(0), _smushHeight(0), _zb(nullptr), _alpha(1.f),
-		_bufferId(0), _currentActor(nullptr) {
-	g_driver = this;
+		_zb(nullptr), _alpha(1.f),
+		_currentActor(nullptr), _smushImage(nullptr) {
 	_storedDisplay = nullptr;
 	// TGL_LEQUAL as tglDepthFunc ensures that subsequent drawing attempts for
 	// the same triangles are not ignored by the depth test.
@@ -144,18 +58,27 @@ GfxTinyGL::GfxTinyGL() :
 	// refer to the same vertices. The first face is usually using the
 	// color map and the following are using textures.
 	_depthFunc = (g_grim->getGameType() == GType_MONKEY4) ? TGL_LEQUAL : TGL_LESS;
+	for (int i = 0; i < 96; i++) {
+		_emergFont[i] = nullptr;
+	}
 }
 
 GfxTinyGL::~GfxTinyGL() {
+	releaseMovieFrame();
+	for (unsigned int i = 0; i < _numSpecialtyTextures; i++) {
+		destroyTexture(&_specialtyTextures[i]);
+	}
+	for (int i = 0; i < 96; i++) {
+		Graphics::tglDeleteBlitImage(_emergFont[i]);
+	}
 	if (_zb) {
-		delBuffer(1);
 		TinyGL::glClose();
 		delete _zb;
 	}
 }
 
 byte *GfxTinyGL::setupScreen(int screenW, int screenH, bool fullscreen) {
-	Graphics::PixelBuffer buf = g_system->setupScreen(screenW, screenH, fullscreen, false);
+	Graphics::PixelBuffer buf = g_system->getScreenPixelBuffer();
 	byte *buffer = buf.getRawBuffer();
 
 	_screenWidth = screenW;
@@ -163,15 +86,15 @@ byte *GfxTinyGL::setupScreen(int screenW, int screenH, bool fullscreen) {
 	_scaleW = _screenWidth / (float)_gameWidth;
 	_scaleH = _screenHeight / (float)_gameHeight;
 
-	_isFullscreen = g_system->getFeatureState(OSystem::kFeatureFullscreenMode);
-
 	g_system->showMouse(!fullscreen);
 
 	g_system->setWindowCaption("ResidualVM: Software 3D Renderer");
 
 	_pixelFormat = buf.getFormat();
+	debug("INFO: TinyGL front buffer pixel format: %s", _pixelFormat.toString().c_str());
 	_zb = new TinyGL::FrameBuffer(screenW, screenH, buf);
 	TinyGL::glInit(_zb, 256);
+	tglEnableDirtyRects(ConfMan.getBool("dirtyrects"));
 
 	_storedDisplay.create(_pixelFormat, _gameWidth * _gameHeight, DisposeAfterUse::YES);
 	_storedDisplay.clear(_gameWidth * _gameHeight);
@@ -180,11 +103,8 @@ byte *GfxTinyGL::setupScreen(int screenW, int screenH, bool fullscreen) {
 
 	TGLfloat ambientSource[] = { 0.0f, 0.0f, 0.0f, 1.0f };
 	tglLightModelfv(TGL_LIGHT_MODEL_AMBIENT, ambientSource);
-
-	// we now generate a buffer (id 1), which we will use as a backing buffer, where the actors' clean buffers
-	// will blit to. everu frame this will be blitted to screen, but the actors' buffers will be blitted to
-	// this only when they change.
-	genBuffer();
+	TGLfloat diffuseReflectance[] = { 1.0f, 1.0f, 1.0f, 1.0f };
+	tglMaterialfv(TGL_FRONT, TGL_DIFFUSE, diffuseReflectance);
 
 	return buffer;
 }
@@ -252,65 +172,18 @@ Math::Matrix4 GfxTinyGL::getProjection() {
 }
 
 void GfxTinyGL::clearScreen() {
-	_zb->clear(true, 0, true, 0, 0, 0);
+	tglClear(TGL_COLOR_BUFFER_BIT | TGL_DEPTH_BUFFER_BIT);
 }
 
 void GfxTinyGL::clearDepthBuffer() {
-	memset(_zb->getZBuffer(), 0, _gameWidth * _gameHeight * sizeof(uint32));
+	tglClear(TGL_DEPTH_BUFFER_BIT);
 }
 
 void GfxTinyGL::flipBuffer() {
+	TinyGL::tglPresentBuffer();
 	g_system->updateScreen();
 }
 
-int GfxTinyGL::genBuffer() {
-	TinyGL::Buffer *buf = _zb->genOffscreenBuffer();
-	_buffers[++_bufferId] = buf;
-
-	return _bufferId;
-}
-
-void GfxTinyGL::delBuffer(int id) {
-	_zb->delOffscreenBuffer(_buffers[id]);
-	_buffers.erase(id);
-}
-
-void GfxTinyGL::selectBuffer(int id) {
-	if (id == 0) {
-		_zb->selectOffscreenBuffer(NULL);
-	} else {
-		_zb->selectOffscreenBuffer(_buffers[id]);
-	}
-}
-
-void GfxTinyGL::clearBuffer(int id) {
-	TinyGL::Buffer *buf = _buffers[id];
-	_zb->clearOffscreenBuffer(buf);
-}
-
-void GfxTinyGL::drawBuffers() {
-	selectBuffer(1);
-	Common::HashMap<int, TinyGL::Buffer *>::iterator i = _buffers.begin();
-	for (++i; i != _buffers.end(); ++i) {
-		TinyGL::Buffer *buf = i->_value;
-		_zb->blitOffscreenBuffer(buf);
-		//this is not necessary, but it prevents the buffers to be blitted every frame, if it is not needed
-		buf->used = false;
-	}
-
-	selectBuffer(0);
-	_zb->blitOffscreenBuffer(_buffers[1]);
-}
-
-void GfxTinyGL::refreshBuffers() {
-	clearBuffer(1);
-	Common::HashMap<int, TinyGL::Buffer *>::iterator i = _buffers.begin();
-	for (++i; i != _buffers.end(); ++i) {
-		TinyGL::Buffer *buf = i->_value;
-		buf->used = true;
-	}
-}
-
 bool GfxTinyGL::isHardwareAccelerated() {
 	return false;
 }
@@ -750,27 +623,32 @@ void GfxTinyGL::drawEMIModelFace(const EMIModel *model, const EMIMeshFace *face)
 
 	tglEnable(TGL_DEPTH_TEST);
 	tglDisable(TGL_ALPHA_TEST);
-	//tglDisable(TGL_LIGHTING); // not apply here in TinyGL
+	tglDisable(TGL_LIGHTING);
 	if (!_currentShadowArray && face->_hasTexture)
 		tglEnable(TGL_TEXTURE_2D);
 	else
 		tglDisable(TGL_TEXTURE_2D);
-	if (face->_flags & EMIMeshFace::kAlphaBlend || face->_flags & EMIMeshFace::kUnknownBlend || _currentActor->hasLocalAlpha() || _alpha < 1.0f )
+	if (face->_flags & EMIMeshFace::kAlphaBlend || face->_flags & EMIMeshFace::kUnknownBlend || _currentActor->hasLocalAlpha() || _alpha < 1.0f)
 		tglEnable(TGL_BLEND);
 
 	tglBegin(TGL_TRIANGLES);
+	float alpha = _alpha;
+	if (model->_meshAlphaMode == Actor::AlphaReplace) {
+		alpha *= model->_meshAlpha;
+	}
+	Math::Vector3d noLighting(1.f, 1.f, 1.f);
 	for (uint j = 0; j < face->_faceLength * 3; j++) {
 		int index = indices[j];
+
 		if (!_currentShadowArray) {
 			if (face->_hasTexture) {
 				tglTexCoord2f(model->_texVerts[index].getX(), model->_texVerts[index].getY());
 			}
-
-			Math::Vector3d lighting = model->_lighting[index];
+			Math::Vector3d lighting = (face->_flags & EMIMeshFace::kNoLighting) ? noLighting : model->_lighting[index];
 			byte r = (byte)(model->_colorMap[index].r * lighting.x());
 			byte g = (byte)(model->_colorMap[index].g * lighting.y());
 			byte b = (byte)(model->_colorMap[index].b * lighting.z());
-			byte a = (int)(model->_colorMap[index].a * _alpha * _currentActor->getLocalAlpha(index));
+			byte a = (int)(model->_colorMap[index].a * alpha * _currentActor->getLocalAlpha(index));
 			tglColor4ub(r, g, b, a);
 		}
 
@@ -789,7 +667,7 @@ void GfxTinyGL::drawEMIModelFace(const EMIModel *model, const EMIMeshFace *face)
 	tglEnable(TGL_TEXTURE_2D);
 	tglEnable(TGL_DEPTH_TEST);
 	tglEnable(TGL_ALPHA_TEST);
-	//tglEnable(TGL_LIGHTING); // not apply here in TinyGL
+	tglEnable(TGL_LIGHTING);
 	tglDisable(TGL_BLEND);
 
 	if (!_currentShadowArray)
@@ -797,9 +675,13 @@ void GfxTinyGL::drawEMIModelFace(const EMIModel *model, const EMIMeshFace *face)
 }
 
 void GfxTinyGL::drawModelFace(const Mesh *mesh, const MeshFace *face) {
+	// Support transparency in actor objects, such as the message tube
+	// in Manny's Office
 	float *vertices = mesh->_vertices;
 	float *vertNormals = mesh->_vertNormals;
 	float *textureVerts = mesh->_textureVerts;
+	tglAlphaFunc(TGL_GREATER, 0.5);
+	tglEnable(TGL_ALPHA_TEST);
 	tglNormal3fv(const_cast<float *>(face->getNormal().getData()));
 	tglBegin(TGL_POLYGON);
 	for (int i = 0; i < face->getNumVertices(); i++) {
@@ -808,10 +690,11 @@ void GfxTinyGL::drawModelFace(const Mesh *mesh, const MeshFace *face) {
 		if (face->hasTexture())
 			tglTexCoord2fv(textureVerts + 2 * face->getTextureVertex(i));
 
-		tglColor4f(1.0f, 1.0f, 1.0f, _alpha);
 		tglVertex3fv(vertices + 3 * face->getVertex(i));
 	}
 	tglEnd();
+	// Done with transparency-capable objects
+	tglDisable(TGL_ALPHA_TEST);
 }
 
 void GfxTinyGL::drawSprite(const Sprite *sprite) {
@@ -858,9 +741,7 @@ void GfxTinyGL::drawSprite(const Sprite *sprite) {
 	tglDisable(TGL_LIGHTING);
 
 	if (g_grim->getGameType() == GType_GRIM) {
-		// Disable Alpha Test, doesn't work the same as OpenGL
-		// It was enabled to sync with OpenGL renderer while TinyGL refactoring
-		//tglEnable(TGL_ALPHA_TEST);
+		tglEnable(TGL_ALPHA_TEST);
 		tglAlphaFunc(TGL_GEQUAL, 0.5f);
 	} else if (sprite->_flags2 & Sprite::AlphaTest) {
 		tglEnable(TGL_ALPHA_TEST);
@@ -943,7 +824,7 @@ void GfxTinyGL::rotateViewpoint(const Math::Matrix4 &rot) {
 }
 
 void GfxTinyGL::translateViewpointFinish() {
-	//glMatrixMode(GL_MODELVIEW); // exist in opengl but doesn't work properly here
+	tglMatrixMode(TGL_MODELVIEW);
 	tglPopMatrix();
 }
 
@@ -962,11 +843,13 @@ void GfxTinyGL::setupLight(Light *light, int lightId) {
 	float lightPos[] = { 0.0f, 0.0f, 0.0f, 1.0f };
 	float lightDir[] = { 0.0f, 0.0f, -1.0f };
 	float cutoff = 180.0f;
+	float spot_exp = 0.0f;
+	float q_attenuation = 1.0f;
 
-	float intensity = light->_intensity / 1.3f;
-	lightColor[0] = ((float)light->_color.getRed() / 15.0f) * intensity;
-	lightColor[1] = ((float)light->_color.getGreen() / 15.0f) * intensity;
-	lightColor[2] = ((float)light->_color.getBlue() / 15.0f) * intensity;
+	float intensity = light->_scaledintensity;
+	lightColor[0] = (float)light->_color.getRed() * intensity;
+	lightColor[1] = (float)light->_color.getGreen() * intensity;
+	lightColor[2] = (float)light->_color.getBlue() * intensity;
 
 	if (light->_type == Light::Omni) {
 		lightPos[0] = light->_pos.x();
@@ -984,18 +867,18 @@ void GfxTinyGL::setupLight(Light *light, int lightId) {
 		lightDir[0] = light->_dir.x();
 		lightDir[1] = light->_dir.y();
 		lightDir[2] = light->_dir.z();
-		/* FIXME: TGL_SPOT_CUTOFF should be light->_penumbraangle, but there
-		   seems to be a bug in tinygl as it renders differently from OpenGL.
-		   Reproducing: turn off all lights (comment out), go to scene "al",
-		   and walk along left wall under the lamp. */
-		cutoff = 90.0f;
+		spot_exp = 2.0f;
+		cutoff = light->_penumbraangle;
+		q_attenuation = 0.0f;
 	}
 
 	tglDisable(TGL_LIGHT0 + lightId);
 	tglLightfv(TGL_LIGHT0 + lightId, TGL_DIFFUSE, lightColor);
 	tglLightfv(TGL_LIGHT0 + lightId, TGL_POSITION, lightPos);
 	tglLightfv(TGL_LIGHT0 + lightId, TGL_SPOT_DIRECTION, lightDir);
+	tglLightf(TGL_LIGHT0 + lightId, TGL_SPOT_EXPONENT, spot_exp);
 	tglLightf(TGL_LIGHT0 + lightId, TGL_SPOT_CUTOFF, cutoff);
+	tglLightf(TGL_LIGHT0 + lightId, TGL_QUADRATIC_ATTENUATION, q_attenuation);
 	tglEnable(TGL_LIGHT0 + lightId);
 }
 
@@ -1004,9 +887,8 @@ void GfxTinyGL::turnOffLight(int lightId) {
 }
 
 void GfxTinyGL::createBitmap(BitmapData *bitmap) {
-	if (bitmap->_format == 1) {
-		bitmap->convertToColorFormat(_pixelFormat);
-	}
+	Graphics::BlitImage **imgs = new Graphics::BlitImage*[bitmap->_numImages];
+	bitmap->_texIds = (void *)imgs;
 
 	if (bitmap->_format != 1) {
 		for (int pic = 0; pic < bitmap->_numImages; pic++) {
@@ -1022,202 +904,27 @@ void GfxTinyGL::createBitmap(BitmapData *bitmap) {
 			}
 			delete[] bufPtr;
 			bitmap->_data[pic] = Graphics::PixelBuffer(Graphics::PixelFormat(4, 8, 8, 8, 8, 0, 8, 16, 24), (byte *)buf);
+			imgs[pic] = Graphics::tglGenBlitImage();
+			const Graphics::PixelBuffer &imageBuffer = bitmap->getImageData(pic);
+			Graphics::Surface sourceSurface;
+			sourceSurface.setPixels(imageBuffer.getRawBuffer());
+			sourceSurface.format = imageBuffer.getFormat();
+			sourceSurface.w = bitmap->_width;
+			sourceSurface.h = bitmap->_height;
+			sourceSurface.pitch = sourceSurface.w * imageBuffer.getFormat().bytesPerPixel;
+			Graphics::tglUploadBlitImage(imgs[pic], sourceSurface, 0, false);
 		}
 	} else {
-		BlitImage *imgs = new BlitImage[bitmap->_numImages];
-		bitmap->_texIds = (void *)imgs;
-
 		for (int i = 0; i < bitmap->_numImages; ++i) {
-			imgs[i].create(bitmap->getImageData(i), 0xf81f, bitmap->_x, bitmap->_y, bitmap->_width, bitmap->_height);
-		}
-	}
-}
-
-void GfxTinyGL::blitScreen(const Graphics::PixelFormat &format, BlitImage *image, byte *src, int x, int y, int width, int height, bool trans, bool dimSprites) {
-	int srcX, srcY;
-
-	if (x < 0) {
-		srcX = -x;
-		x = 0;
-	} else {
-		srcX = 0;
-	}
-
-	if (y < 0) {
-		srcY = -y;
-		y = 0;
-	} else {
-		srcY = 0;
-	}
-
-	blitScreen(format, image, src, x, y, srcX, srcY, width, height, width, height, trans, dimSprites);
-}
-
-void GfxTinyGL::blit(const Graphics::PixelFormat &format, BlitImage *image, byte *dst, byte *src, int x, int y, int width, int height, bool trans) {
-	int srcX, srcY;
-
-	if (x < 0) {
-		srcX = -x;
-		x = 0;
-	} else {
-		srcX = 0;
-	}
-
-	if (y < 0) {
-		srcY = -y;
-		y = 0;
-	} else {
-		srcY = 0;
-	}
-
-	blit(format, image, dst, src, x, y, srcX, srcY, width, height, width, height, trans);
-}
-
-void GfxTinyGL::blit(const Graphics::PixelFormat &format, BlitImage *image, byte *dst, byte *src, int dstX, int dstY, int srcX, int srcY, int width, int height, int srcWidth, int srcHeight, bool trans) {
-	if (_dimLevel > 0.0f && _dimLevel < 1.0f) {
-		warning("TinyGL doesn't implement partial screen-dimming yet");
-	}
-
-	if (dstX >= _gameWidth || dstY >= _gameHeight)
-		return;
-
-	int clampWidth, clampHeight;
-
-	if (dstX + width > _gameWidth)
-		clampWidth = _gameWidth - dstX;
-	else
-		clampWidth = width;
-
-	if (dstY + height > _gameHeight)
-		clampHeight = _gameHeight - dstY;
-	else
-		clampHeight = height;
-
-	dst += (dstX + (dstY * _gameWidth)) * format.bytesPerPixel;
-	src += (srcX + (srcY * srcWidth)) * format.bytesPerPixel;
-
-	Graphics::PixelBuffer srcBuf(format, src);
-	Graphics::PixelBuffer dstBuf(format, dst);
-
-	if (!trans) {
-		for (int l = 0; l < clampHeight; l++) {
-			dstBuf.copyBuffer(0, clampWidth, srcBuf);
-			dstBuf.shiftBy(_gameWidth);
-			srcBuf.shiftBy(srcWidth);
-		}
-	} else {
-		if (image) {
-			BlitImage::Line *l = image->_lines;
-			int maxY = srcY + clampHeight;
-			int maxX = srcX + clampWidth;
-			while (l && l->y < srcY)
-				l = l->next;
-
-			while (l && l->y < maxY) {
-				if (l->x < maxX && l->x + l->length > srcX) {
-					int length = l->length;
-					int skipStart = (l->x < srcX) ? (srcX - l->x) : 0;
-					length -= skipStart;
-					int skipEnd   = (l->x + l->length > maxX) ? (l->x + l->length - maxX) : 0;
-					length -= skipEnd;
-					memcpy(dstBuf.getRawBuffer((l->y - srcY) * _gameWidth + MAX(l->x - srcX, 0)),
-						   l->pixels + skipStart * format.bytesPerPixel, length * format.bytesPerPixel);
-				}
-				l = l->next;
-			}
-		} else {
-			for (int l = 0; l < clampHeight; l++) {
-				for (int r = 0; r < clampWidth; ++r) {
-					if (srcBuf.getValueAt(r) != 0xf81f) {
-						dstBuf.setPixelAt(r, srcBuf);
-					}
-				}
-				dstBuf.shiftBy(_gameWidth);
-				srcBuf.shiftBy(srcWidth);
-			}
-		}
-	}
-}
-
-void GfxTinyGL::blitScreen(const Graphics::PixelFormat &format, BlitImage *image, byte *src, int dstX, int dstY, int srcX, int srcY, int width, int height, int srcWidth, int srcHeight, bool trans, bool dimSprites) {
-	if (dstX >= _gameWidth || dstY >= _gameHeight)
-		return;
-
-	int clampWidth, clampHeight;
-
-	if (dstX + width > _gameWidth)
-		clampWidth = _gameWidth - dstX;
-	else
-		clampWidth = width;
-
-	if (dstY + height > _gameHeight)
-		clampHeight = _gameHeight - dstY;
-	else
-		clampHeight = height;
-
-	int blendEnabled;
-	tglGetIntegerv(TGL_BLEND, &blendEnabled);
-	src += (srcX + (srcY * srcWidth)) * format.bytesPerPixel;
-	Graphics::PixelBuffer srcBuf(format, src);
-	bool hasDim = _dimLevel > 0.0f;
-	dimSprites &= hasDim;
-
-	if ((trans == false || blendEnabled == false) && (dimSprites == false)) {
-		byte *dst = _zb->getPixelBuffer();
-		dst += (dstX + (dstY * _gameWidth)) * format.bytesPerPixel;
-		Graphics::PixelBuffer dstBuf(format, dst);
-		if (!trans) {
-			for (int l = 0; l < clampHeight; l++) {
-				dstBuf.copyBuffer(0, clampWidth, srcBuf);
-				dstBuf.shiftBy(_gameWidth);
-				srcBuf.shiftBy(srcWidth);
-			}
-		} else {
-			if (image) {
-				BlitImage::Line *l = image->_lines;
-				int maxY = srcY + clampHeight;
-				int maxX = srcX + clampWidth;
-				while (l && l->y < srcY) {
-					l = l->next;
-				}
-				while (l && l->y < maxY) {
-					if (l->x < maxX && l->x + l->length > srcX) {
-						int length = l->length;
-						int skipStart = (l->x < srcX) ? (srcX - l->x) : 0;
-						length -= skipStart;
-						int skipEnd   = (l->x + l->length > maxX) ? (l->x + l->length - maxX) : 0;
-						length -= skipEnd;
-						memcpy(dstBuf.getRawBuffer((l->y - srcY) * _gameWidth + MAX(l->x - srcX, 0)),
-							l->pixels + skipStart * format.bytesPerPixel, length * format.bytesPerPixel);
-					}
-					l = l->next;
-				}
-			} else {
-				for (int l = 0; l < clampHeight; l++) {
-					for (int r = 0; r < clampWidth; ++r) {
-						if (srcBuf.getValueAt(r) != 0xf81f) {
-							dstBuf.setPixelAt(r, srcBuf);
-						}
-					}
-					dstBuf.shiftBy(_gameWidth);
-					srcBuf.shiftBy(srcWidth);
-				}
-			}
-		}
-	} else {
-		float colFactor = 1.0f - _dimLevel;
-		if (dimSprites == false) {
-			colFactor = 1.0f;
-		}
-		for (int l = 0; l < clampHeight; l++) {
-			for (int r = 0; r < clampWidth; ++r) {
-				byte aDst, rDst, gDst, bDst;
-				srcBuf.getARGBAt(r, aDst, rDst, gDst, bDst);
-				if (rDst == 248 && gDst == 0 && bDst == 248) 
-					continue;
-				_zb->writePixel((dstX + r) + (dstY + l) * _gameWidth, aDst, rDst * colFactor, gDst * colFactor, bDst * colFactor);
-			}
-			srcBuf.shiftBy(srcWidth);
+			imgs[i] = Graphics::tglGenBlitImage();
+			const Graphics::PixelBuffer &imageBuffer = bitmap->getImageData(i);
+			Graphics::Surface sourceSurface;
+			sourceSurface.setPixels(imageBuffer.getRawBuffer());
+			sourceSurface.format = imageBuffer.getFormat();
+			sourceSurface.w = bitmap->_width;
+			sourceSurface.h = bitmap->_height;
+			sourceSurface.pitch = sourceSurface.w * imageBuffer.getFormat().bytesPerPixel;
+			Graphics::tglUploadBlitImage(imgs[i], sourceSurface, sourceSurface.format.ARGBToColor(0, 255, 0, 255), true);
 		}
 	}
 }
@@ -1225,14 +932,17 @@ void GfxTinyGL::blitScreen(const Graphics::PixelFormat &format, BlitImage *image
 void GfxTinyGL::drawBitmap(const Bitmap *bitmap, int x, int y, uint32 layer) {
 	// PS2 EMI uses a TGA for it's splash-screen, avoid using the following
 	// code for drawing that (as it has no tiles).
-	if (g_grim->getGameType() == GType_MONKEY4 && bitmap->_data->_numImages > 1) {
-		tglColor3f(1.0f - _dimLevel, 1.0f - _dimLevel, 1.0f  - _dimLevel);
+	if (g_grim->getGameType() == GType_MONKEY4 && bitmap->_data && bitmap->_data->_texc) {
+		tglEnable(TGL_BLEND);
+		tglBlendFunc(TGL_SRC_ALPHA, TGL_ONE_MINUS_SRC_ALPHA);
+		tglColor3f(1.0f, 1.0f, 1.0f);
 
 		BitmapData *data = bitmap->_data;
 		float *texc = data->_texc;
 
-		BlitImage *b = (BlitImage *)bitmap->getTexIds();
+		Graphics::BlitImage **b = (Graphics::BlitImage **)bitmap->getTexIds();
 
+		assert(layer < data->_numLayers);
 		uint32 offset = data->_layers[layer]._offset;
 		for (uint32 i = offset; i < offset + data->_layers[layer]._numImages; ++i) {
 			const BitmapData::Vert &v = data->_verts[i];
@@ -1250,12 +960,15 @@ void GfxTinyGL::drawBitmap(const Bitmap *bitmap, int x, int y, uint32 layer) {
 				int srcX = texc[ntex + 2] * bitmap->getWidth();
 				int srcY = texc[ntex + 3] * bitmap->getHeight();
 
-				blitScreen(bitmap->getPixelFormat(texId), &b[texId], bitmap->getData(texId).getRawBuffer(),
-					 x + dx1, y + dy1, srcX, srcY, dx2 - dx1, dy2 - dy1, b[texId]._width, b[texId]._height, true, true);
+				Graphics::BlitTransform transform(x + dx1, y + dy1);
+				transform.sourceRectangle(srcX, srcY, dx2 - dx1, dy2 - dy1);
+				transform.tint(1.0f, 1.0f - _dimLevel, 1.0f - _dimLevel, 1.0f  - _dimLevel);
+				Graphics::tglBlit(b[texId], transform);
 				ntex += 16;
 			}
 		}
 
+		tglDisable(TGL_BLEND);
 		return;
 	}
 
@@ -1267,22 +980,28 @@ void GfxTinyGL::drawBitmap(const Bitmap *bitmap, int x, int y, uint32 layer) {
 	assert(bitmap->getActiveImage() > 0);
 	const int num = bitmap->getActiveImage() - 1;
 
-	BlitImage *b = (BlitImage *)bitmap->getTexIds();
+	Graphics::BlitImage **b = (Graphics::BlitImage **)bitmap->getTexIds();
 
-	if (bitmap->getFormat() == 1)
-		blitScreen(bitmap->getPixelFormat(num), &b[num], (byte *)bitmap->getData(num).getRawBuffer(),
-			 x, y, bitmap->getWidth(), bitmap->getHeight(), true, false);
-	else
-		blit(bitmap->getPixelFormat(num), nullptr, (byte *)_zb->getZBuffer(), (byte *)bitmap->getData(num).getRawBuffer(),
-			 x, y, bitmap->getWidth(), bitmap->getHeight(), false);
+	if (bitmap->getFormat() == 1) {
+		if (bitmap->getHasTransparency()) {
+			tglEnable(TGL_BLEND);
+			tglBlendFunc(TGL_SRC_ALPHA, TGL_ONE_MINUS_SRC_ALPHA);
+		}
+		Graphics::tglBlit(b[num], x, y);
+		if (bitmap->getHasTransparency()) {
+			tglDisable(TGL_BLEND);
+		}
+	} else {
+		Graphics::tglBlitZBuffer(b[num], x, y);
+	}
 }
 
 void GfxTinyGL::destroyBitmap(BitmapData *bitmap) {
- 	for (int pic = 0; pic < bitmap->_numImages; pic++) {
- 		if (bitmap->_data)
- 			bitmap->_data[pic].free();
- 	}
- 	delete[] (BlitImage*)bitmap->_texIds;
+	Graphics::BlitImage **imgs = (Graphics::BlitImage **)bitmap->_texIds;
+	for (int pic = 0; pic < bitmap->_numImages; pic++) {
+		Graphics::tglDeleteBlitImage(imgs[pic]);
+	}
+	delete[] imgs;
 }
 
 void GfxTinyGL::createFont(Font *font) {
@@ -1292,7 +1011,7 @@ void GfxTinyGL::destroyFont(Font *font) {
 }
 
 struct TextObjectData {
-	byte *data;
+	Graphics::BlitImage *image;
 	int width, height, x, y;
 };
 
@@ -1303,6 +1022,12 @@ void GfxTinyGL::createTextObject(TextObject *text) {
 	const Color &fgColor = text->getFGColor();
 	TextObjectData *userData = new TextObjectData[numLines];
 	text->setUserData(userData);
+	uint32 kKitmapColorkey = _pixelFormat.RGBToColor(0, 255, 0);
+	const uint32 blackColor = _pixelFormat.RGBToColor(0, 0, 0);
+	const uint32 color = _pixelFormat.RGBToColor(fgColor.getRed(), fgColor.getGreen(), fgColor.getBlue());
+	while (color == kKitmapColorkey || blackColor == kKitmapColorkey) {
+		kKitmapColorkey += 1;
+	}
 	for (int j = 0; j < numLines; j++) {
 		const Common::String &currentLine = lines[j];
 
@@ -1333,24 +1058,17 @@ void GfxTinyGL::createTextObject(TextObject *text) {
 			startColumn += font->getCharKernedWidth(ch);
 		}
 
-		Graphics::PixelBuffer buf(_pixelFormat, width * height, DisposeAfterUse::NO);
+		Graphics::PixelBuffer buf(_pixelFormat, width * height, DisposeAfterUse::YES);
 
 		uint8 *bitmapData = _textBitmap;
-		uint8 r = fgColor.getRed();
-		uint8 g = fgColor.getGreen();
-		uint8 b = fgColor.getBlue();
-		uint32 color = _zb->cmode.RGBToColor(r, g, b);
-
-		if (color == 0xf81f)
-			color = 0xf81e;
 
 		int txData = 0;
 		for (int i = 0; i < width * height; i++, txData++, bitmapData++) {
 			byte pixel = *bitmapData;
 			if (pixel == 0x00) {
-				buf.setPixelAt(txData, 0xf81f);
+				buf.setPixelAt(txData, kKitmapColorkey);
 			} else if (pixel == 0x80) {
-				buf.setPixelAt(txData, 0);
+				buf.setPixelAt(txData, blackColor);
 			} else if (pixel == 0xFF) {
 				buf.setPixelAt(txData, color);
 			}
@@ -1358,7 +1076,15 @@ void GfxTinyGL::createTextObject(TextObject *text) {
 
 		userData[j].width = width;
 		userData[j].height = height;
-		userData[j].data = buf.getRawBuffer();
+
+		Graphics::Surface sourceSurface;
+		sourceSurface.setPixels(buf.getRawBuffer());
+		sourceSurface.format = buf.getFormat();
+		sourceSurface.w = width;
+		sourceSurface.h = height;
+		sourceSurface.pitch = sourceSurface.w * buf.getFormat().bytesPerPixel;
+		userData[j].image = Graphics::tglGenBlitImage();
+		Graphics::tglUploadBlitImage(userData[j].image, sourceSurface, kKitmapColorkey, true);
 		userData[j].x = text->getLineX(j);
 		userData[j].y = text->getLineY(j);
 
@@ -1375,10 +1101,13 @@ void GfxTinyGL::createTextObject(TextObject *text) {
 void GfxTinyGL::drawTextObject(const TextObject *text) {
 	const TextObjectData *userData = (const TextObjectData *)text->getUserData();
 	if (userData) {
+		tglEnable(TGL_BLEND);
+		tglBlendFunc(TGL_SRC_ALPHA, TGL_ONE_MINUS_SRC_ALPHA);
 		int numLines = text->getNumLines();
 		for (int i = 0; i < numLines; ++i) {
-			blitScreen(_pixelFormat, nullptr, userData[i].data, userData[i].x, userData[i].y, userData[i].width, userData[i].height, true, false);
+			Graphics::tglBlit(userData[i].image, userData[i].x, userData[i].y);
 		}
+		tglDisable(TGL_BLEND);
 	}
 }
 
@@ -1387,7 +1116,7 @@ void GfxTinyGL::destroyTextObject(TextObject *text) {
 	if (userData) {
 		int numLines = text->getNumLines();
 		for (int i = 0; i < numLines; ++i) {
-			delete[] userData[i].data;
+			Graphics::tglDeleteBlitImage(userData[i].image);
 		}
 		delete[] userData;
 	}
@@ -1467,66 +1196,65 @@ void GfxTinyGL::selectTexture(const Texture *texture) {
 
 	// Grim has inverted tex-coords, EMI doesn't
 	if (g_grim->getGameType() != GType_MONKEY4) {
-		tglPushMatrix(); // removed in opengl but here doesn't work properly after remove
 		tglMatrixMode(TGL_TEXTURE);
 		tglLoadIdentity();
 		tglScalef(1.0f / texture->_width, 1.0f / texture->_height, 1);
-		tglMatrixMode(TGL_MODELVIEW); // removed in opengl but here doesn't work properly after remove
-		tglPopMatrix(); // removed in opengl but here doesn't work properly after remove
 	}
 }
 
 void GfxTinyGL::destroyTexture(Texture *texture) {
-	tglDeleteTextures(1, (TGLuint *)texture->_texture);
-	delete[] (TGLuint *)texture->_texture;
-}
-
-void GfxTinyGL::prepareMovieFrame(Graphics::Surface *frame) {
-	_smushWidth = frame->w;
-	_smushHeight = frame->h;
-
-	Graphics::PixelBuffer srcBuf(frame->format, (byte *)frame->getPixels());
-	_smushBitmap.create(_pixelFormat, frame->w * frame->h, DisposeAfterUse::YES);
-	_smushBitmap.copyBuffer(0, frame->w * frame->h, srcBuf);
-}
-
-void GfxTinyGL::drawMovieFrame(int offsetX, int offsetY) {
-	if (_smushWidth == _gameWidth && _smushHeight == _gameHeight) {
-		_zb->copyFromBuffer(_smushBitmap);
-	} else {
-		blitScreen(_pixelFormat, nullptr, _smushBitmap.getRawBuffer(), offsetX, offsetY, _smushWidth, _smushHeight, false, false);
+	TGLuint *textures = (TGLuint *)texture->_texture;
+	if (textures) {
+		tglDeleteTextures(1, textures);
+		delete[] textures;
 	}
 }
 
+void GfxTinyGL::prepareMovieFrame(Graphics::Surface *frame) {
+	if (_smushImage == nullptr)
+		_smushImage = Graphics::tglGenBlitImage();
+	Graphics::tglUploadBlitImage(_smushImage, *frame, 0, false);
+}
+
+void GfxTinyGL::drawMovieFrame(int offsetX, int offsetY) {
+	Graphics::tglBlitFast(_smushImage, offsetX, offsetY);
+}
+
 void GfxTinyGL::releaseMovieFrame() {
+	Graphics::tglDeleteBlitImage(_smushImage);
 }
 
 void GfxTinyGL::loadEmergFont() {
+	Graphics::Surface characterSurface;
+	Graphics::PixelFormat textureFormat(4, 8, 8, 8, 8, 0, 8, 16, 24);
+	characterSurface.create(8, 13, textureFormat);
+	uint32 color = textureFormat.ARGBToColor(255, 255, 255, 255);
+	uint32 colorTransparent = textureFormat.ARGBToColor(0, 255, 255, 255);
+	for (int i = 0; i < 96; i++) {
+		_emergFont[i] = Graphics::tglGenBlitImage();
+		const uint8 *ptr = Font::emerFont[i];
+		for (int py = 0; py < 13; py++) {
+				int line = ptr[12 - py];
+				for (int px = 0; px < 8; px++) {
+					int pixel = line & 0x80;
+					line <<= 1;
+					*(uint32 *)characterSurface.getBasePtr(px, py) = pixel ? color : colorTransparent;
+				}
+		}
+		Graphics::tglUploadBlitImage(_emergFont[i], characterSurface, 0, false);
+	}
+	characterSurface.free();
 }
 
 void GfxTinyGL::drawEmergString(int x, int y, const char *text, const Color &fgColor) {
-	uint32 color = _pixelFormat.RGBToColor(fgColor.getRed(), fgColor.getGreen(), fgColor.getBlue());
-
 	int length = strlen(text);
 
 	for (int l = 0; l < length; l++) {
 		int c = text[l];
 		assert(c >= 32 && c <= 127);
-		const uint8 *ptr = Font::emerFont[c - 32];
-		for (int py = 0; py < 13; py++) {
-			if ((py + y) < _gameHeight && (py + y) >= 0) {
-				int line = ptr[12 - py];
-				for (int px = 0; px < 8; px++) {
-					if ((px + x) < _gameWidth && (px + x) >= 0) {
-						int pixel = line & 0x80;
-						line <<= 1;
-						if (pixel) {
-							_zb->writePixel(((py + y) * _gameWidth) + (px + x), color);
-						}
-					}
-				}
-			}
-		}
+		Graphics::BlitTransform transform(x, y);
+		transform.tint(1.0f, fgColor.getRed() / 255.0f, fgColor.getGreen() / 255.0f, fgColor.getBlue() / 255.0f);
+		Graphics::tglBlit(_emergFont[c - 32], transform);
 		x += 10;
 	}
 }
@@ -1535,7 +1263,7 @@ Bitmap *GfxTinyGL::getScreenshot(int w, int h, bool useStored) {
 	if (useStored) {
 		return createScreenshotBitmap(_storedDisplay, w, h, true);
 	} else {
-		Graphics::PixelBuffer src(Graphics::PixelFormat(4, 8, 8, 8, 8, 0, 8, 16, 24), _screenWidth * _screenHeight, DisposeAfterUse::YES);
+		Graphics::PixelBuffer src(_pixelFormat, _screenWidth * _screenHeight, DisposeAfterUse::YES);
 		_zb->copyToBuffer(src);
 		return createScreenshotBitmap(src, w, h, true);
 	}
@@ -1547,102 +1275,170 @@ void GfxTinyGL::createSpecialtyTextureFromScreen(uint id, uint8 *data, int x, in
 }
 
 void GfxTinyGL::storeDisplay() {
+	TinyGL::tglPresentBuffer();
 	_zb->copyToBuffer(_storedDisplay);
 }
 
 void GfxTinyGL::copyStoredToDisplay() {
-	_zb->copyFromBuffer(_storedDisplay);
+	Bitmap *bitmap = getScreenshot(_gameWidth, _gameHeight, true);
+	drawBitmap(bitmap, 0, 0, 0);
+	delete bitmap;
 }
 
 void GfxTinyGL::dimScreen() {
-	for (int l = 0; l < _gameWidth * _gameHeight; l++) {
-		uint8 r, g, b;
-		_storedDisplay.getRGBAt(l, r, g, b);
-		uint32 color = (r + g + b) / 10;
-		_storedDisplay.setPixelAt(l, color, color, color);
-	}
+	dimRegion(0, 0, _gameWidth, _gameHeight, 0.2f);
 }
 
 void GfxTinyGL::dimRegion(int x, int y, int w, int h, float level) {
-	for (int ly = y; ly < y + h; ly++) {
-		for (int lx = x; lx < x + w; lx++) {
-			uint8 r, g, b;
-			_zb->readPixelRGB(ly * _gameWidth + lx, r, g, b);
-			uint32 color = (uint32)(((r + g + b) / 3) * level);
-			_zb->writePixel(ly * _gameWidth + lx, color, color, color);
-		}
-	}
+	tglMatrixMode(TGL_PROJECTION);
+	tglLoadIdentity();
+	tglOrtho(0, _gameWidth, _gameHeight, 0, 0, 1);
+	tglMatrixMode(TGL_MODELVIEW);
+	tglLoadIdentity();
+
+	tglDisable(TGL_LIGHTING);
+	tglDisable(TGL_DEPTH_TEST);
+	tglDepthMask(TGL_FALSE);
+	tglEnable(TGL_BLEND);
+	tglBlendFunc(TGL_SRC_ALPHA, TGL_ONE_MINUS_SRC_ALPHA);
+
+	tglColor4f(0, 0, 0, 1 - level);
+
+	tglBegin(TGL_QUADS);
+	tglVertex2f(x, y);
+	tglVertex2f(x + w, y);
+	tglVertex2f(x + w, y + h);
+	tglVertex2f(x, y + h);
+	tglEnd();
+
+	tglColor3f(1.0f, 1.0f, 1.0f);
+
+	tglDisable(TGL_BLEND);
+	tglDepthMask(TGL_TRUE);
+	tglEnable(TGL_DEPTH_TEST);
+	tglEnable(TGL_LIGHTING);
 }
 
 void GfxTinyGL::irisAroundRegion(int x1, int y1, int x2, int y2) {
-	for (int ly = 0; ly < _gameHeight; ly++) {
-		for (int lx = 0; lx < _gameWidth; lx++) {
-			// Don't do anything with the data in the region we draw Around
-			if (lx > x1 && lx < x2 && ly > y1 && ly < y2)
-				continue;
-			// But set everything around it to black.
-			_zb->writePixel(ly * _gameWidth + lx, 0);
-		}
-	}
+	tglMatrixMode(TGL_PROJECTION);
+	tglLoadIdentity();
+	tglOrtho(0.0, _gameWidth, _gameHeight, 0.0, 0.0, 1.0);
+	tglMatrixMode(TGL_MODELVIEW);
+	tglLoadIdentity();
+
+	tglDisable(TGL_DEPTH_TEST);
+	tglDisable(TGL_TEXTURE_2D);
+	tglDisable(TGL_BLEND);
+	tglDisable(TGL_LIGHTING);
+	tglDepthMask(TGL_FALSE);
+
+	tglColor3f(0.0f, 0.0f, 0.0f);
+
+	//Explicitly cast to avoid problems with C++11
+	float fx1 = x1;
+	float fx2 = x2;
+	float fy1 = y1;
+	float fy2 = y2;
+	float width = _screenWidth;
+	float height = _screenHeight;
+	float points[20] = {
+		0.0f, 0.0f,
+		0.0f, fy1,
+		width, 0.0f,
+		fx2, fy1,
+		width, height,
+		fx2, fy2,
+		0.0f, height,
+		fx1, fy2,
+		0.0f, fy1,
+		fx1, fy1
+	};
+
+	tglEnableClientState(TGL_VERTEX_ARRAY);
+	tglVertexPointer(2, TGL_FLOAT, 0, points);
+	tglDrawArrays(TGL_TRIANGLE_STRIP, 0, 10);
+	tglDisableClientState(TGL_VERTEX_ARRAY);
+
+	tglColor3f(1.0f, 1.0f, 1.0f);
+	tglEnable(TGL_DEPTH_TEST);
+	tglEnable(TGL_LIGHTING);
+	tglDepthMask(TGL_TRUE);
 }
 
 void GfxTinyGL::drawRectangle(const PrimitiveObject *primitive) {
-	int x1 = primitive->getP1().x;
-	int y1 = primitive->getP1().y;
-	int x2 = primitive->getP2().x;
-	int y2 = primitive->getP2().y;
+	float x1 = primitive->getP1().x * _scaleW;
+	float y1 = primitive->getP1().y * _scaleH;
+	float x2 = primitive->getP2().x * _scaleW;
+	float y2 = primitive->getP2().y * _scaleH;
+	const Color color(primitive->getColor());
 
-	const Color &color = primitive->getColor();
-	uint32 c = _pixelFormat.RGBToColor(color.getRed(), color.getGreen(), color.getBlue());
+	tglMatrixMode(TGL_PROJECTION);
+	tglLoadIdentity();
+	tglOrtho(0, _screenWidth, _screenHeight, 0, 0, 1);
+	tglMatrixMode(TGL_MODELVIEW);
+	tglLoadIdentity();
+
+	tglDisable(TGL_LIGHTING);
+	tglDisable(TGL_DEPTH_TEST);
+	tglDepthMask(TGL_FALSE);
+
+	tglColor3ub(color.getRed(), color.getGreen(), color.getBlue());
 
 	if (primitive->isFilled()) {
-		for (; y1 <= y2; y1++)
-			if (y1 >= 0 && y1 < _gameHeight)
-				for (int x = x1; x <= x2; x++)
-					if (x >= 0 && x < _gameWidth)
-						_zb->writePixel(_gameWidth * y1 + x, c);
+		tglBegin(TGL_QUADS);
+		tglVertex2f(x1, y1);
+		tglVertex2f(x2 + 1, y1);
+		tglVertex2f(x2 + 1, y2 + 1);
+		tglVertex2f(x1, y2 + 1);
+		tglEnd();
 	} else {
-		if (y1 >= 0 && y1 < _gameHeight)
-			for (int x = x1; x <= x2; x++)
-				if (x >= 0 && x < _gameWidth)
-					_zb->writePixel(_gameWidth * y1 + x, c);
-		if (y2 >= 0 && y2 < _gameHeight)
-			for (int x = x1; x <= x2; x++)
-				if (x >= 0 && x < _gameWidth)
-					_zb->writePixel(_gameWidth * y2 + x, c);
-		if (x1 >= 0 && x1 < _gameWidth)
-			for (int y = y1; y <= y2; y++)
-				if (y >= 0 && y < _gameHeight)
-					_zb->writePixel(_gameWidth * y + x1, c);
-		if (x2 >= 0 && x2 < _gameWidth)
-			for (int y = y1; y <= y2; y++)
-				if (y >= 0 && y < _gameHeight)
-					_zb->writePixel(_gameWidth * y + x2, c);
+		tglBegin(TGL_LINE_LOOP);
+		tglVertex2f(x1, y1);
+		tglVertex2f(x2 + 1, y1);
+		tglVertex2f(x2 + 1, y2 + 1);
+		tglVertex2f(x1, y2 + 1);
+		tglEnd();
 	}
+
+	tglColor3f(1.0f, 1.0f, 1.0f);
+
+	tglDepthMask(TGL_TRUE);
+	tglEnable(TGL_DEPTH_TEST);
+	tglEnable(TGL_LIGHTING);
 }
 
 void GfxTinyGL::drawLine(const PrimitiveObject *primitive) {
-	int x1 = primitive->getP1().x;
-	int y1 = primitive->getP1().y;
-	int x2 = primitive->getP2().x;
-	int y2 = primitive->getP2().y;
+	float x1 = primitive->getP1().x * _scaleW;
+	float y1 = primitive->getP1().y * _scaleH;
+	float x2 = primitive->getP2().x * _scaleW;
+	float y2 = primitive->getP2().y * _scaleH;
 
 	const Color &color = primitive->getColor();
 
-	if (x2 == x1) {
-		for (int y = y1; y <= y2; y++) {
-			if (x1 >= 0 && x1 < _gameWidth && y >= 0 && y < _gameHeight)
-				_zb->writePixel(_gameWidth * y + x1, color.getRed(), color.getGreen(), color.getBlue());
-		}
-	} else {
-		float m = (y2 - y1) / (float)(x2 - x1);
-		int b = (int)(-m * x1 + y1);
-		for (int x = x1; x <= x2; x++) {
-			int y = (int)(m * x) + b;
-			if (x >= 0 && x < _gameWidth && y >= 0 && y < _gameHeight)
-				_zb->writePixel(_gameWidth * y + x, color.getRed(), color.getGreen(), color.getBlue());
-		}
-	}
+	tglMatrixMode(TGL_PROJECTION);
+	tglLoadIdentity();
+	tglOrtho(0, _screenWidth, _screenHeight, 0, 0, 1);
+	tglMatrixMode(TGL_MODELVIEW);
+	tglLoadIdentity();
+
+	tglDisable(TGL_LIGHTING);
+	tglDisable(TGL_DEPTH_TEST);
+	tglDepthMask(TGL_FALSE);
+
+	tglColor3ub(color.getRed(), color.getGreen(), color.getBlue());
+
+//	tglLineWidth(_scaleW); // Not implemented in TinyGL
+
+	tglBegin(TGL_LINES);
+	tglVertex2f(x1, y1);
+	tglVertex2f(x2, y2);
+	tglEnd();
+
+	tglColor3f(1.0f, 1.0f, 1.0f);
+
+	tglDepthMask(TGL_TRUE);
+	tglEnable(TGL_DEPTH_TEST);
+	tglEnable(TGL_LIGHTING);
 }
 
 void GfxTinyGL::drawDimPlane() {
@@ -1679,34 +1475,41 @@ void GfxTinyGL::drawDimPlane() {
 }
 
 void GfxTinyGL::drawPolygon(const PrimitiveObject *primitive) {
-	int x1 = primitive->getP1().x;
-	int y1 = primitive->getP1().y;
-	int x2 = primitive->getP2().x;
-	int y2 = primitive->getP2().y;
-	int x3 = primitive->getP3().x;
-	int y3 = primitive->getP3().y;
-	int x4 = primitive->getP4().x;
-	int y4 = primitive->getP4().y;
-	float m;
-	int b;
+	float x1 = primitive->getP1().x * _scaleW;
+	float y1 = primitive->getP1().y * _scaleH;
+	float x2 = primitive->getP2().x * _scaleW;
+	float y2 = primitive->getP2().y * _scaleH;
+	float x3 = primitive->getP3().x * _scaleW;
+	float y3 = primitive->getP3().y * _scaleH;
+	float x4 = primitive->getP4().x * _scaleW;
+	float y4 = primitive->getP4().y * _scaleH;
 
 	const Color &color = primitive->getColor();
-	uint32 c = _pixelFormat.RGBToColor(color.getRed(), color.getGreen(), color.getBlue());
 
-	m = (y2 - y1) / (x2 - x1);
-	b = (int)(-m * x1 + y1);
-	for (int x = x1; x <= x2; x++) {
-		int y = (int)(m * x) + b;
-		if (x >= 0 && x < _gameWidth && y >= 0 && y < _gameHeight)
-			_zb->writePixel(_gameWidth * y + x, c);
-	}
-	m = (y4 - y3) / (x4 - x3);
-	b = (int)(-m * x3 + y3);
-	for (int x = x3; x <= x4; x++) {
-		int y = (int)(m * x) + b;
-		if (x >= 0 && x < _gameWidth && y >= 0 && y < _gameHeight)
-			_zb->writePixel(_gameWidth * y + x, c);
-	}
+	tglMatrixMode(TGL_PROJECTION);
+	tglLoadIdentity();
+	tglOrtho(0, _screenWidth, _screenHeight, 0, 0, 1);
+	tglMatrixMode(TGL_MODELVIEW);
+	tglLoadIdentity();
+
+	tglDisable(TGL_LIGHTING);
+	tglDisable(TGL_DEPTH_TEST);
+	tglDepthMask(TGL_FALSE);
+
+	tglColor3ub(color.getRed(), color.getGreen(), color.getBlue());
+
+	tglBegin(TGL_LINES);
+	tglVertex2f(x1, y1);
+	tglVertex2f(x2 + 1, y2 + 1);
+	tglVertex2f(x3, y3 + 1);
+	tglVertex2f(x4 + 1, y4);
+	tglEnd();
+
+	tglColor3f(1.0f, 1.0f, 1.0f);
+
+	tglDepthMask(TGL_TRUE);
+	tglEnable(TGL_DEPTH_TEST);
+	tglEnable(TGL_LIGHTING);
 }
 
 void GfxTinyGL::readPixels(int x, int y, int width, int height, uint8 *buffer) {
