@@ -27,10 +27,6 @@
 
 #include "graphics/tinygl/zgl.h"
 
-namespace TinyGL {
-	struct Buffer;
-}
-
 namespace Grim {
 
 class ModelNode;
@@ -126,13 +122,6 @@ public:
 	void drawMovieFrame(int offsetX, int offsetY) override;
 	void releaseMovieFrame() override;
 
-	int genBuffer() override;
-	void delBuffer(int buffer) override;
-	void selectBuffer(int buffer) override;
-	void clearBuffer(int buffer) override;
-	void drawBuffers() override;
-	void refreshBuffers() override;
-
 	void setBlendMode(bool additive) override;
 
 protected:
@@ -140,21 +129,15 @@ protected:
 
 private:
 	TinyGL::FrameBuffer *_zb;
-	Graphics::PixelBuffer _smushBitmap;
-	int _smushWidth;
-	int _smushHeight;
+	Graphics::PixelFormat _pixelFormat;
+	Graphics::BlitImage *_emergFont[96];
+	Graphics::BlitImage *_smushImage;
 	Graphics::PixelBuffer _storedDisplay;
 	float _alpha;
-	Common::HashMap<int, TinyGL::Buffer *> _buffers;
-	uint _bufferId;
 	const Actor *_currentActor;
 	TGLenum _depthFunc;
 
 	void readPixels(int x, int y, int width, int height, uint8 *buffer);
-	void blit(const Graphics::PixelFormat &format, BlitImage *blit, byte *dst, byte *src, int x, int y, int width, int height, bool trans);
-	void blit(const Graphics::PixelFormat &format, BlitImage *blit, byte *dst, byte *src, int dstX, int dstY, int srcX, int srcY, int width, int height, int srcWidth, int srcHeight, bool trans);
-	void blitScreen(const Graphics::PixelFormat &format, BlitImage *blit, byte *src, int x, int y, int width, int height, bool trans, bool dimSprites);
-	void blitScreen(const Graphics::PixelFormat &format, BlitImage *blit, byte *src, int dstX, int dstY, int srcX, int srcY, int width, int height, int srcWidth, int srcHeight, bool trans, bool dimSprites);
 };
 
 } // end of namespace Grim
