@@ -31,13 +31,21 @@
 #include "common/foreach.h"
 #include "common/fs.h"
 #include "common/config-manager.h"
+#include "common/translation.h"
 
 #include "graphics/pixelbuffer.h"
+#include "graphics/renderer.h"
+
+#ifdef USE_OPENGL
+#include "graphics/opengl/context.h"
+#endif
 
 #include "gui/error.h"
 #include "gui/gui-manager.h"
 #include "gui/message.h"
 
+#include "image/png.h"
+
 #include "engines/engine.h"
 
 #include "engines/grim/md5check.h"
@@ -96,10 +104,6 @@ GrimEngine::GrimEngine(OSystem *syst, uint32 gameFlags, GrimGameType gameType, C
 	g_imuse = nullptr;
 
 	//Set default settings
-	ConfMan.registerDefault("soft_renderer", false);
-	ConfMan.registerDefault("engine_speed", 60);
-	ConfMan.registerDefault("fullscreen", false);
-	ConfMan.registerDefault("show_fps", false);
 	ConfMan.registerDefault("use_arb_shaders", true);
 
 	_showFps = ConfMan.getBool("show_fps");
@@ -119,16 +123,23 @@ GrimEngine::GrimEngine(OSystem *syst, uint32 gameFlags, GrimGameType gameType, C
 		_controlsEnabled[i] = false;
 		_controlsState[i] = false;
 	}
+	_joyAxisPosition = new float[NUM_JOY_AXES];
+	for (int i = 0; i < NUM_JOY_AXES; i++) {
+		_joyAxisPosition[i] = 0;
+	}
 	_speechMode = TextAndVoice;
 	_textSpeed = 7;
 	_mode = _previousMode = NormalMode;
 	_flipEnable = true;
 	int speed = ConfMan.getInt("engine_speed");
-	if (speed <= 0 || speed > 100)
+	if (speed == 0) {
+		_speedLimitMs = 0;
+	} else if (speed < 0 || speed > 100) {
 		_speedLimitMs = 1000 / 60;
-	else
+		ConfMan.setInt("engine_speed", 1000 / _speedLimitMs);
+	} else {
 		_speedLimitMs = 1000 / speed;
-	ConfMan.setInt("engine_speed", 1000 / _speedLimitMs);
+	}
 	_listFilesIter = nullptr;
 	_savedState = nullptr;
 	_fps[0] = 0;
@@ -164,6 +175,7 @@ GrimEngine::GrimEngine(OSystem *syst, uint32 gameFlags, GrimGameType gameType, C
 	const Common::FSNode gameDataDir(ConfMan.get("path"));
 	SearchMan.addSubDirectoryMatching(gameDataDir, "movies"); // Add 'movies' subdirectory for the demo
 	SearchMan.addSubDirectoryMatching(gameDataDir, "credits");
+	SearchMan.addSubDirectoryMatching(gameDataDir, "widescreen");
 
 	Debug::registerDebugChannels();
 }
@@ -171,6 +183,7 @@ GrimEngine::GrimEngine(OSystem *syst, uint32 gameFlags, GrimGameType gameType, C
 GrimEngine::~GrimEngine() {
 	delete[] _controlsEnabled;
 	delete[] _controlsState;
+	delete[] _joyAxisPosition;
 
 	clearPools();
 
@@ -219,23 +232,48 @@ LuaBase *GrimEngine::createLua() {
 	return new Lua_V1();
 }
 
-void GrimEngine::createRenderer() {
-#ifdef USE_OPENGL
-	_softRenderer = ConfMan.getBool("soft_renderer");
+GfxBase *GrimEngine::createRenderer(int screenW, int screenH, bool fullscreen) {
+	Common::String rendererConfig = ConfMan.get("renderer");
+	Graphics::RendererType desiredRendererType = Graphics::parseRendererTypeCode(rendererConfig);
+	Graphics::RendererType matchingRendererType = Graphics::getBestMatchingAvailableRendererType(desiredRendererType);
+
+	_softRenderer = matchingRendererType == Graphics::kRendererTypeTinyGL;
+	_system->setupScreen(screenW, screenH, fullscreen, !_softRenderer);
+
+#if defined(USE_OPENGL)
+	// Check the OpenGL context actually supports shaders
+	if (matchingRendererType == Graphics::kRendererTypeOpenGLShaders && !OpenGLContext.shadersSupported) {
+		matchingRendererType = Graphics::kRendererTypeOpenGL;
+	}
 #endif
 
-	if (!_softRenderer && !g_system->hasFeature(OSystem::kFeatureOpenGL)) {
-		warning("gfx backend doesn't support hardware rendering");
-		_softRenderer = true;
+	if (matchingRendererType != desiredRendererType && desiredRendererType != Graphics::kRendererTypeDefault) {
+		// Display a warning if unable to use the desired renderer
+		warning("Unable to create a '%s' renderer", rendererConfig.c_str());
 	}
 
-	if (_softRenderer) {
-		g_driver = CreateGfxTinyGL();
-#ifdef USE_OPENGL
-	} else {
-		g_driver = CreateGfxOpenGL();
-#endif
+	GfxBase *renderer = nullptr;
+#if defined(USE_GLES2) || defined(USE_OPENGL_SHADERS)
+	if (matchingRendererType == Graphics::kRendererTypeOpenGLShaders) {
+		renderer = CreateGfxOpenGLShader();
 	}
+#endif
+#if defined(USE_OPENGL) && !defined(USE_GLES2)
+	if (matchingRendererType == Graphics::kRendererTypeOpenGL) {
+		renderer = CreateGfxOpenGL();
+	}
+#endif
+	if (matchingRendererType == Graphics::kRendererTypeTinyGL) {
+		renderer = CreateGfxTinyGL();
+	}
+
+	if (!renderer) {
+		error("Unable to create a '%s' renderer", rendererConfig.c_str());
+	}
+
+	renderer->setupScreen(screenW, screenH, fullscreen);
+	renderer->loadEmergFont();
+	return renderer;
 }
 
 const char *GrimEngine::getUpdateFilename() {
@@ -272,11 +310,13 @@ Common::Error GrimEngine::run() {
 	if (ConfMan.getBool("check_gamedata")) {
 		MD5CheckDialog d;
 		if (!d.runModal()) {
-			Common::String confirmString("ResidualVM found some problems with your game data files.\n"
-										 "Running ResidualVM nevertheless may cause game bugs or even crashes.\n"
-										 "Do you still want to run ");
-			confirmString += (GType_MONKEY4 == getGameType() ? "Escape From Monkey Island?" : "Grim Fandango?");
-			GUI::MessageDialog msg(confirmString, "Yes", "No");
+			Common::String confirmString = Common::String::format(_(
+				"ResidualVM found some problems with your game data files.\n"
+				"Running ResidualVM nevertheless may cause game bugs or even crashes.\n"
+				"Do you still want to run %s?"),
+			GType_MONKEY4 == getGameType() ? _("Escape From Monkey Island") : _("Grim Fandango")
+			 );
+			GUI::MessageDialog msg(confirmString, _("Yes"), _("No"));
 			if (!msg.runModal()) {
 				return Common::kUserCanceled;
 			}
@@ -306,8 +346,7 @@ Common::Error GrimEngine::run() {
 	g_sound = new SoundPlayer();
 
 	bool fullscreen = ConfMan.getBool("fullscreen");
-	createRenderer();
-	g_driver->setupScreen(640, 480, fullscreen);
+	g_driver = createRenderer(640, 480, fullscreen);
 
 	if (getGameType() == GType_MONKEY4 && SearchMan.hasFile("AMWI.m4b")) {
 		// Play EMI Mac Aspyr logo
@@ -337,6 +376,11 @@ Common::Error GrimEngine::run() {
 	lua->registerOpcodes();
 	lua->registerLua();
 
+	// One version of the demo doesn't set the demo flag in scripts.
+	if (getGameType() == GType_GRIM && _gameFlags & ADGF_DEMO) {
+		lua->forceDemo();
+	}
+
 	//Initialize Localizer first. In system-script are already localizeable Strings
 	g_localizer = new Localizer();
 	lua->loadSystemScript();
@@ -406,7 +450,11 @@ void GrimEngine::playAspyrLogo() {
 Common::Error GrimEngine::loadGameState(int slot) {
 	assert(slot >= 0);
 	if (getGameType() == GType_MONKEY4) {
-		_savegameFileName = Common::String::format("efmi%03d.gsv", slot);
+		if (getGamePlatform() == Common::kPlatformPS2) {
+			_savegameFileName = Common::String::format("efmi%03d.ps2", slot);
+		} else {
+			_savegameFileName = Common::String::format("efmi%03d.gsv", slot);
+		}
 	} else {
 		_savegameFileName = Common::String::format("grim%02d.gsv", slot);
 	}
@@ -563,7 +611,7 @@ void GrimEngine::updateDisplayScene() {
 				if (frame != _prevSmushFrame) {
 					_prevSmushFrame = g_movie->getFrame();
 					g_driver->drawMovieFrame(g_movie->getX(), g_movie->getY());
-					if (_showFps && _mode != DrawMode)
+					if (_showFps)
 						g_driver->drawEmergString(550, 25, _fps, Color(255, 255, 255));
 				} else
 					_doFlip = false;
@@ -583,15 +631,13 @@ void GrimEngine::updateDisplayScene() {
 }
 
 void GrimEngine::updateNormalMode() {
-	if (!_currSet)
+	if (!_currSet || !_flipEnable)
 		return;
 
 	g_driver->clearScreen();
 
 	drawNormalMode();
 
-	g_driver->drawBuffers();
-
 	_iris->draw();
 	drawTextObjects();
 }
@@ -652,6 +698,7 @@ void GrimEngine::drawNormalMode() {
 	if (_setupChanged) {
 		cameraPostChangeHandle(_currSet->getSetup());
 		_setupChanged = false;
+		setSideTextures(_currSet->getCurrSetup()->_name.c_str());
 	}
 
 	// Draw actors
@@ -685,7 +732,7 @@ void GrimEngine::doFlip() {
 		unsigned int currentTime = g_system->getMillis();
 		unsigned int delta = currentTime - _lastFrameTime;
 		if (delta > 500) {
-			sprintf(_fps, "%7.2f", (double)(_frameCounter * 1000) / (double)delta);
+			snprintf(_fps, sizeof(_fps), "%7.2f", (double)(_frameCounter * 1000) / (double)delta);
 			_frameCounter = 0;
 			_lastFrameTime = currentTime;
 		}
@@ -725,15 +772,24 @@ void GrimEngine::mainLoop() {
 			savegameSave();
 		}
 
+		// If the backend can keep the OpenGL context when switching to fullscreen,
+		// just toggle the fullscreen feature (SDL2 path).
+		if (_changeFullscreenState &&
+				_system->hasFeature(OSystem::kFeatureFullscreenToggleKeepsContext)) {
+			bool fullscreen = _system->getFeatureState(OSystem::kFeatureFullscreenMode);
+			_system->setFeatureState(OSystem::kFeatureFullscreenMode, !fullscreen);
+			_changeFullscreenState = false;
+		}
+
+		// If the backend destroys the OpenGL context or the user switched to a different
+		// renderer, the GFX driver needs to be recreated (SDL1 path).
 		if (_changeHardwareState || _changeFullscreenState) {
 			_changeHardwareState = false;
 
-			bool fullscreen = g_driver->isFullscreen();
+			bool fullscreen = _system->getFeatureState(OSystem::kFeatureFullscreenMode);
 			if (_changeFullscreenState) {
 				fullscreen = !fullscreen;
 			}
-			g_system->setFeatureState(OSystem::kFeatureFullscreenMode, fullscreen);
-			ConfMan.setBool("fullscreen", fullscreen);
 
 			uint screenWidth = g_driver->getScreenWidth();
 			uint screenHeight = g_driver->getScreenHeight();
@@ -745,8 +801,7 @@ void GrimEngine::mainLoop() {
 			clearPools();
 
 			delete g_driver;
-			createRenderer();
-			g_driver->setupScreen(screenWidth, screenHeight, fullscreen);
+			g_driver = createRenderer(screenWidth, screenHeight, fullscreen);
 			savegameRestore();
 
 			if (mode == DrawMode) {
@@ -797,17 +852,28 @@ void GrimEngine::mainLoop() {
 
 				handleControls(type, event.kbd);
 
-				// Allow lua to react to the event.
-				// Without this lua_update switching the entries in the menu is slow because
-				// if the button is not kept pressed the KEYUP will arrive just after the KEYDOWN
-				// and it will break the lua scripts that checks for the state of the button
-				// with GetControlState()
+				if (getGameType() != GType_MONKEY4) {
+					// Allow lua to react to the event.
+					// Without this lua_update switching the entries in the menu is slow because
+					// if the button is not kept pressed the KEYUP will arrive just after the KEYDOWN
+					// and it will break the lua scripts that checks for the state of the button
+					// with GetControlState()
+					//
+					// This call seems to be only necessary to handle Grim's menu correctly.
+					// In EMI it would have the side-effect that luaUpdate() is sometimes called
+					// in the same millisecond which causes getPerSecond() to return 0 for
+					// any given rate which is not compatible with e.g. actor walking.
 
-				// We do not want the scripts to update while a movie is playing in the PS2-version.
-				if (!(getGamePlatform() == Common::kPlatformPS2 && _mode == SmushMode)) {
-					luaUpdate();
+					// We do not want the scripts to update while a movie is playing in the PS2-version.
+					if (!(getGamePlatform() == Common::kPlatformPS2 && _mode == SmushMode)) {
+						luaUpdate();
+					}
 				}
 			}
+			if (type == Common::EVENT_JOYAXIS_MOTION)
+				handleJoyAxis(event.joystick.axis, event.joystick.position);
+			if (type == Common::EVENT_JOYBUTTON_DOWN || type == Common::EVENT_JOYBUTTON_UP)
+				handleJoyButton(type, event.joystick.button);
 		}
 
 		if (_mode != PauseMode) {
@@ -954,12 +1020,8 @@ void GrimEngine::savegameRestore() {
 	invalidateActiveActorsList();
 	buildActiveActorsList();
 
-	g_driver->refreshBuffers();
 	_currSet->setupCamera();
 	g_driver->set3DMode();
-	foreach (Actor *a, Actor::getPool()) {
-		a->restoreCleanBuffer();
-	}
 }
 
 void GrimEngine::restoreGRIM() {
@@ -999,21 +1061,18 @@ void GrimEngine::restoreGRIM() {
 }
 
 void GrimEngine::storeSaveGameImage(SaveGame *state) {
+	const Graphics::PixelFormat image_format = Graphics::PixelFormat(2, 5, 6, 5, 0, 11, 5, 0, 0);
 	int width = 250, height = 188;
 	Bitmap *screenshot;
 
 	debug("GrimEngine::StoreSaveGameImage() started.");
 
-	EngineMode mode = g_grim->getMode();
-	g_grim->setMode(_previousMode);
-	g_grim->updateDisplayScene();
-	g_driver->storeDisplay();
-	screenshot = g_driver->getScreenshot(width, height, false);
-	g_grim->setMode(mode);
+	screenshot = g_driver->getScreenshot(width, height, true);
 	state->beginSection('SIMG');
 	if (screenshot) {
 		int size = screenshot->getWidth() * screenshot->getHeight();
 		screenshot->setActiveImage(0);
+		screenshot->getBitmapData()->convertToColorFormat(image_format);
 		uint16 *data = (uint16 *)screenshot->getData().getRawBuffer();
 		for (int l = 0; l < size; l++) {
 			state->writeLEUint16(data[l]);
@@ -1200,9 +1259,7 @@ void GrimEngine::setSet(Set *scene) {
 	// and coords change too.
 	foreach (Actor *a, Actor::getPool()) {
 		a->stopWalking();
-		a->clearCleanBuffer();
 	}
-	g_driver->refreshBuffers();
 
 	Set *lastSet = _currSet;
 	_currSet = scene;
@@ -1219,11 +1276,6 @@ void GrimEngine::setSet(Set *scene) {
 void GrimEngine::makeCurrentSetup(int num) {
 	int prevSetup = g_grim->getCurrSet()->getSetup();
 	if (prevSetup != num) {
-		foreach (Actor *a, Actor::getPool()) {
-			a->clearCleanBuffer();
-		}
-		g_driver->refreshBuffers();
-
 		getCurrSet()->setSetup(num);
 		getCurrSet()->setSoundParameters(20, 127);
 		cameraChangeHandle(prevSetup, num);
@@ -1242,6 +1294,10 @@ void GrimEngine::setTextSpeed(int speed) {
 }
 
 float GrimEngine::getControlAxis(int num) {
+	int idx = num - KEYCODE_AXIS_JOY1_X;
+	if (idx >= 0 && idx < NUM_JOY_AXES) {
+		return _joyAxisPosition[idx];
+	}
 	return 0;
 }
 
@@ -1269,7 +1325,7 @@ void GrimEngine::buildActiveActorsList() {
 
 	_activeActors.clear();
 	foreach (Actor *a, Actor::getPool()) {
-		if (((_mode == NormalMode || _mode == DrawMode) && a->isInSet(_currSet->getName())) ||
+		if (((_mode == NormalMode || _mode == DrawMode) && a->isDrawableInSet(_currSet->getName())) ||
 		    a->isInOverworld()) {
 			_activeActors.push_back(a);
 		}
@@ -1325,12 +1381,6 @@ bool GrimEngine::hasFeature(EngineFeature f) const {
 		(f == kSupportsLoadingDuringRuntime);
 }
 
-void GrimEngine::openMainMenuDialog() {
-	Common::KeyState key(Common::KEYCODE_F1, Common::ASCII_F1);
-	handleControls(Common::EVENT_KEYDOWN, key);
-	handleControls(Common::EVENT_KEYUP, key);
-}
-
 void GrimEngine::pauseEngineIntern(bool pause) {
 	if (g_imuse)
 		g_imuse->pause(pause);
@@ -1344,6 +1394,34 @@ void GrimEngine::pauseEngineIntern(bool pause) {
 	}
 }
 
+
+Graphics::Surface *loadPNG(const Common::String &filename) {
+	Image::PNGDecoder d;
+	Common::SeekableReadStream *s = SearchMan.createReadStreamForMember(filename);
+	if (!s)
+		return nullptr;
+	d.loadStream(*s);
+	delete s;
+
+	Graphics::Surface *srf = d.getSurface()->convertTo(Graphics::PixelFormat(4, 8, 8, 8, 8, 0, 8, 16, 24));
+	return srf;
+}
+
+void GrimEngine::setSideTextures(const Common::String &setup) {
+	if (! g_system->hasFeature(OSystem::kFeatureSideTextures))
+			return;
+	Graphics::Surface *t1 = loadPNG(Common::String::format("%s_left.png", setup.c_str()));
+	Graphics::Surface *t2 = loadPNG(Common::String::format("%s_right.png", setup.c_str()));
+	g_system->suggestSideTextures(t1, t2);
+	if (t1)
+		t1->free();
+	if (t2)
+		t2->free();
+	delete t1;
+	delete t2;
+}
+
+
 void GrimEngine::debugLua(const Common::String &str) {
 	lua_dostring(str.c_str());
 }
