@@ -28,7 +28,9 @@ void start_script();
 void stop_script();
 void next_script();
 void identify_script();
+void pause_script();
 void pause_scripts();
+void unpause_script();
 void unpause_scripts();
 void find_script();
 void break_here();
