@@ -34,6 +34,7 @@
 
 #include "engines/grim/debug.h"
 
+#include "engines/grim/movie/codecs/codec48.h"
 #include "engines/grim/movie/codecs/blocky8.h"
 #include "engines/grim/movie/codecs/blocky16.h"
 #include "engines/grim/movie/codecs/smush_decoder.h"
@@ -98,16 +99,17 @@ void SmushDecoder::initFrames() {
 		while (size > 0) {
 			uint32 subType = _file->readUint32BE();
 			uint32 subSize = _file->readUint32BE();
+			int32  subPos  = _file->pos();
 
 			if (subType == MKTAG('B', 'l', '1', '6')) {
 				_file->seek(18, SEEK_CUR);
 				if (_file->readByte() == 0) {
 					frame.keyframe = true;
 				}
-				_file->seek(subSize - 19, SEEK_CUR);
-			} else
-				_file->seek(subSize, SEEK_CUR);
+			}
+
 			size -= subSize + 8 + (subSize & 1);
+			_file->seek(subPos + subSize + (subSize & 1), SEEK_SET);
 		}
 
 		_file->seek(size, SEEK_CUR);
@@ -182,7 +184,8 @@ bool SmushDecoder::readHeader() {
 
 		_file->readUint32BE();
 		_file->readUint32BE();
-		_audioTrack = new SmushAudioTrack(false, audioRate, 2);
+		_audioTrack = new SmushAudioTrack(getSoundType(), false, audioRate, 2);
+		addTrack(_audioTrack);
 		return true;
 
 	} else if (tag == MKTAG('S', 'H', 'D', 'R')) { // Retail
@@ -242,7 +245,7 @@ bool SmushDecoder::handleFramesHeader() {
 	} while (pos < size);
 	delete[] f_header;
 
-	_audioTrack = new SmushAudioTrack(true, freq, channels);
+	_audioTrack = new SmushAudioTrack(getSoundType(), true, freq, channels);
 	addTrack(_audioTrack);
 	return true;
 }
@@ -449,9 +452,11 @@ SmushDecoder::SmushVideoTrack::SmushVideoTrack(int width, int height, int fps, i
 	// Which means 16 bpp, 565, shift of 11, 5, 0, 0 for RGBA
 	_format = Graphics::PixelFormat(2, 5, 6, 5, 0, 11, 5, 0, 0);
 	if (!is16Bit) { // Demo
+		_codec48 = new Codec48Decoder();
 		_blocky8 = new Blocky8();
 		_blocky16 = nullptr;
 	} else {
+		_codec48 = nullptr;
 		_blocky8 = nullptr;
 		_blocky16 = new Blocky16();
 		_blocky16->init(width, height);
@@ -472,6 +477,7 @@ SmushDecoder::SmushVideoTrack::SmushVideoTrack(int width, int height, int fps, i
 }
 
 SmushDecoder::SmushVideoTrack::~SmushVideoTrack() {
+	delete _codec48;
 	delete _blocky8;
 	delete _blocky16;
 	_surface.free();
@@ -530,7 +536,7 @@ void SmushDecoder::SmushVideoTrack::handleFrameObject(Common::SeekableReadStream
 	assert(!_is16Bit);
 	assert(size >= 14);
 	byte codec = stream->readByte();
-	assert(codec == 47);
+	assert(codec == 47 || codec == 48);
 	/* byte codecParam = */ stream->readByte();
 	_x = stream->readSint16LE();
 	_y = stream->readSint16LE();
@@ -540,6 +546,7 @@ void SmushDecoder::SmushVideoTrack::handleFrameObject(Common::SeekableReadStream
 		_width = width;
 		_height = height;
 		_surface.create(_width, _height, _format);
+		_codec48->init(_width, _height);
 		_blocky8->init(_width, _height);
 	}
 	stream->readUint16LE();
@@ -548,7 +555,12 @@ void SmushDecoder::SmushVideoTrack::handleFrameObject(Common::SeekableReadStream
 	size -= 14;
 	byte *ptr = new byte[size];
 	stream->read(ptr, size);
-	_blocky8->decode((byte *)_surface.getPixels(), ptr);
+
+	if (codec == 47) {
+		_blocky8->decode((byte *)_surface.getPixels(), ptr);
+	} else if (codec == 48) {
+		_codec48->decode((byte *)_surface.getPixels(), ptr);
+	}
 	delete[] ptr;
 }
 
@@ -580,7 +592,8 @@ Graphics::Surface *SmushDecoder::SmushVideoTrack::decodeNextFrame() {
 void SmushDecoder::SmushVideoTrack::setMsPerFrame(int ms) {
 	_frameRate = Common::Rational(1000000, ms);
 }
-SmushDecoder::SmushAudioTrack::SmushAudioTrack(bool isVima, int freq, int channels) {
+SmushDecoder::SmushAudioTrack::SmushAudioTrack(Audio::Mixer::SoundType soundType, bool isVima, int freq, int channels) :
+	AudioTrack(soundType) {
 	_isVima = isVima;
 	_channels = channels;
 	_freq = freq;
