@@ -24,6 +24,7 @@
 #define GRIM_STUFFIT_H
 
 #include "common/archive.h"
+#include "common/bitstream.h"
 #include "common/scummsys.h"
 #include "common/endian.h"
 #include "common/file.h"
@@ -31,10 +32,6 @@
 #include "common/hashmap.h"
 #include "common/str.h"
 
-namespace Common {
-class BitStream;
-}
-
 namespace Grim {
 
 struct SIT14Data;
@@ -72,7 +69,7 @@ private:
 
 	// Decompression Helpers
 	void update14(uint16 first, uint16 last, byte *code, uint16 *freq) const;
-	void readTree14(Common::BitStream *bits, SIT14Data *dat, uint16 codesize, uint16 *result) const;
+	void readTree14(Common::BitStream8LSB *bits, SIT14Data *dat, uint16 codesize, uint16 *result) const;
 };
 
 } // End of namespace Grim
