@@ -83,10 +83,10 @@ Common::MemoryReadStream *Archive::dumpToMemory(uint32 offset, uint32 size) {
 	return static_cast<Common::MemoryReadStream *>(_file.readStream(size));
 }
 
-const DirectorySubEntry *Archive::getDescription(const char *room, uint32 index, uint16 face, DirectorySubEntry::ResourceType type) {
+const DirectorySubEntry *Archive::getDescription(const Common::String &room, uint32 index, uint16 face,
+                                                 DirectorySubEntry::ResourceType type) {
 	for (uint i = 0; i < _directory.size(); i++) {
-		if (_directory[i].getIndex() == index
-				&& !strcmp(_directory[i].getRoom(), room)) {
+		if (_directory[i].getIndex() == index && _directory[i].getRoom() == room) {
 			return _directory[i].getItemDescription(face, type);
 		}
 	}
@@ -94,6 +94,17 @@ const DirectorySubEntry *Archive::getDescription(const char *room, uint32 index,
 	return 0;
 }
 
+DirectorySubEntryList Archive::listFilesMatching(const Common::String &room, uint32 index, uint16 face,
+                                                 DirectorySubEntry::ResourceType type) {
+	for (uint i = 0; i < _directory.size(); i++) {
+		if (_directory[i].getIndex() == index && _directory[i].getRoom() == room) {
+			return _directory[i].listItemsMatching(face, type);
+		}
+	}
+
+	return DirectorySubEntryList();
+}
+
 bool Archive::open(const char *fileName, const char *room) {
 	// Copy the room name if provided
 	// If the room name is not provided, it is assumed that
@@ -115,4 +126,11 @@ void Archive::close() {
 	_file.close();
 }
 
+Common::String Archive::getRoomName() const {
+	if (_multipleRoom) {
+		error("'%s' is a multi-room archive", _file.getName());
+	}
+
+	return _roomName;
+}
 } // End of namespace Myst3
