@@ -41,12 +41,18 @@ private:
 	void _readDirectory();
 public:
 
-	const DirectorySubEntry *getDescription(const char *room, uint32 index, uint16 face, DirectorySubEntry::ResourceType type);
+	const DirectorySubEntry *getDescription(const Common::String &room, uint32 index, uint16 face,
+	                                        DirectorySubEntry::ResourceType type);
+	DirectorySubEntryList listFilesMatching(const Common::String &room, uint32 index, uint16 face,
+	                                        DirectorySubEntry::ResourceType type);
+
 	Common::MemoryReadStream *dumpToMemory(uint32 offset, uint32 size);
 	void dumpToFiles();
 
 	bool open(const char *fileName, const char *room);
 	void close();
+
+	Common::String getRoomName() const;
 };
 
 } // End of namespace Myst3
