@@ -21,6 +21,8 @@
  */
 
 #include "engines/myst3/console.h"
+
+#include "engines/myst3/archive.h"
 #include "engines/myst3/database.h"
 #include "engines/myst3/effects.h"
 #include "engines/myst3/inventory.h"
@@ -56,31 +58,33 @@ void Console::describeScript(const Common::Array<Opcode> &script) {
 bool Console::Cmd_Infos(int argc, const char **argv) {
 	uint16 nodeId = _vm->_state->getLocationNode();
 	uint32 roomId = _vm->_state->getLocationRoom();
+	uint32 ageID = _vm->_state->getLocationAge();
 
 	if (argc >= 2) {
 		nodeId = atoi(argv[1]);
 	}
 
 	if (argc >= 3) {
-		roomId = _vm->_db->getRoomId(argv[2]);
-
-		if (roomId == 0) {
+		RoomKey roomKey = _vm->_db->getRoomKey(argv[2]);
+		if (roomKey.roomID == 0 || roomKey.ageID == 0) {
 			debugPrintf("Unknown room name %s\n", argv[2]);
 			return true;
 		}
+
+		roomId = roomKey.roomID;
+		ageID = roomKey.ageID;
 	}
 
-	NodePtr nodeData = _vm->_db->getNodeData(nodeId, roomId);
+	NodePtr nodeData = _vm->_db->getNodeData(nodeId, roomId, ageID);
 
 	if (!nodeData) {
 		debugPrintf("No node with id %d\n", nodeId);
 		return true;
 	}
 
-	char roomName[8];
-	_vm->_db->getRoomName(roomName, roomId);
+	Common::String roomName = _vm->_db->getRoomName(roomId, ageID);
 
-	debugPrintf("node: %s %d    ", roomName, nodeId);
+	debugPrintf("node: %s %d    ", roomName.c_str(), nodeId);
 
 	for (uint i = 0; i < nodeData->scripts.size(); i++) {
 		debugPrintf("\ninit %d > %s (%s)\n", i,
@@ -174,21 +178,23 @@ bool Console::Cmd_Var(int argc, const char **argv) {
 }
 
 bool Console::Cmd_ListNodes(int argc, const char **argv) {
-
-	uint32 roomID = 0;
+	uint32 roomID = _vm->_state->getLocationRoom();
+	uint32 ageID = _vm->_state->getLocationAge();
 
 	if (argc == 2) {
-		roomID = _vm->_db->getRoomId(argv[1]);
-
-		if (roomID == 0) {
+		RoomKey roomKey = _vm->_db->getRoomKey(argv[1]);
+		if (roomKey.roomID == 0 || roomKey.ageID == 0) {
 			debugPrintf("Unknown room name %s\n", argv[1]);
 			return true;
 		}
+
+		roomID = roomKey.roomID;
+		ageID = roomKey.ageID;
 	}
 
 	debugPrintf("Nodes:\n");
 
-	Common::Array<uint16> list = _vm->_db->listRoomNodes(roomID);
+	Common::Array<uint16> list = _vm->_db->listRoomNodes(roomID, ageID);
 	for (uint i = 0; i < list.size(); i++) {
 		debugPrintf("%d\n", list[i]);
 	}
@@ -198,22 +204,26 @@ bool Console::Cmd_ListNodes(int argc, const char **argv) {
 
 bool Console::Cmd_Run(int argc, const char **argv) {
 	uint16 nodeId = _vm->_state->getLocationNode();
-	uint32 roomId = 0;
+	uint32 roomId = _vm->_state->getLocationRoom();
+	uint32 ageId = _vm->_state->getLocationAge();
 
 	if (argc >= 2) {
 		nodeId = atoi(argv[1]);
 	}
 
 	if (argc >= 3) {
-		roomId = _vm->_db->getRoomId(argv[2]);
+		RoomKey roomKey = _vm->_db->getRoomKey(argv[2]);
 
-		if (roomId == 0) {
+		if (roomKey.roomID == 0 || roomKey.ageID == 0) {
 			debugPrintf("Unknown room name %s\n", argv[2]);
 			return true;
 		}
+
+		roomId = roomKey.roomID;
+		ageId = roomKey.ageID;
 	}
 
-	_vm->runScriptsFromNode(nodeId, roomId);
+	_vm->runScriptsFromNode(nodeId, roomId, ageId);
 
 	return false;
 }
@@ -247,15 +257,16 @@ bool Console::Cmd_Go(int argc, const char **argv) {
 		return true;
 	}
 
-	uint32 roomID = _vm->_db->getRoomId(argv[1]);
-	uint16 nodeId = atoi(argv[2]);
-
-	if (roomID == 0) {
+	RoomKey roomKey = _vm->_db->getRoomKey(argv[1]);
+	if (roomKey.roomID == 0 || roomKey.ageID == 0) {
 		debugPrintf("Unknown room name %s\n", argv[1]);
 		return true;
 	}
 
-	_vm->_state->setLocationNextRoom(roomID);
+	uint16 nodeId = atoi(argv[2]);
+
+	_vm->_state->setLocationNextAge(roomKey.ageID);
+	_vm->_state->setLocationNextRoom(roomKey.roomID);
 	_vm->_state->setLocationNextNode(nodeId);
 
 	_vm->goToNode(0, kTransitionFade);
@@ -279,7 +290,7 @@ bool Console::Cmd_Extract(int argc, const char **argv) {
 	uint16 face = atoi(argv[3]);
 	DirectorySubEntry::ResourceType type = (DirectorySubEntry::ResourceType) atoi(argv[4]);
 
-	const DirectorySubEntry *desc = _vm->getFileDescription(room.c_str(), id, face, type);
+	const DirectorySubEntry *desc = _vm->getFileDescription(room, id, face, type);
 
 	if (!desc) {
 		debugPrintf("File with room %s, id %d, face %d and type %d does not exist\n", room.c_str(), id, face, type);
@@ -381,23 +392,22 @@ bool Console::Cmd_DumpMasks(int argc, const char **argv) {
 }
 
 bool Console::dumpFaceMask(uint16 index, int face, DirectorySubEntry::ResourceType type) {
-	const DirectorySubEntry *maskDesc = _vm->getFileDescription(0, index, face, type);
+	const DirectorySubEntry *maskDesc = _vm->getFileDescription("", index, face, type);
 
 	if (!maskDesc)
 		return false;
 
 	Common::MemoryReadStream *maskStream = maskDesc->getData();
 
-	Graphics::Surface *mask = Effect::loadMask(maskStream);
+	Effect::FaceMask *mask = Effect::loadMask(maskStream);
 
 	delete maskStream;
 
 	Common::DumpFile outFile;
 	outFile.open(Common::String::format("dump/%d-%d.masku_%d", index, face, type));
-	outFile.write(mask->getPixels(), mask->pitch * mask->h);
+	outFile.write(mask->surface->getPixels(), mask->surface->pitch * mask->surface->h);
 	outFile.close();
 
-	mask->free();
 	delete mask;
 
 	return true;
