@@ -22,11 +22,13 @@
 
 #include "engines/myst3/cursor.h"
 #include "engines/myst3/directorysubentry.h"
+#include "engines/myst3/gfx.h"
 #include "engines/myst3/myst3.h"
 #include "engines/myst3/scene.h"
 #include "engines/myst3/state.h"
 
 #include "graphics/surface.h"
+
 #include "image/bmp.h"
 
 namespace Myst3 {
@@ -35,32 +37,36 @@ struct CursorData {
 	uint32 nodeID;
 	uint16 hotspotX;
 	uint16 hotspotY;
-	Texture *texture;
 	double transparency;
+	double transparencyXbox;
 };
 
-static CursorData availableCursors[13] = {
-	{ 1000,  8,  8, 0, 0.25 },
-	{ 1001,  8,  8, 0, 0.5  },
-	{ 1002,  8,  8, 0, 0.5  },
-	{ 1003,  1,  5, 0, 0.5  },
-	{ 1004, 14,  5, 0, 0.5  },
-	{ 1005, 16, 14, 0, 0.5  },
-	{ 1006, 16, 14, 0, 0.5  },
-	{ 1007,  8,  8, 0, 0.55 },
-	{ 1000,  8,  8, 0, 0.25 },
-	{ 1001,  8,  8, 0, 0.5  },
-	{ 1011, 16, 16, 0, 0.5  },
-	{ 1000,  6,  1, 0, 0.5  },
-	{    0,  0,  0, 0, 0    }
+static const CursorData availableCursors[] = {
+	{ 1000,  8,  8, 0.25, 0.00 },
+	{ 1001,  8,  8, 0.50, 0.50 },
+	{ 1002,  8,  8, 0.50, 0.50 },
+	{ 1003,  1,  5, 0.50, 0.50 },
+	{ 1004, 14,  5, 0.50, 0.50 },
+	{ 1005, 16, 14, 0.50, 0.50 },
+	{ 1006, 16, 14, 0.50, 0.50 },
+	{ 1007,  8,  8, 0.55, 0.55 },
+	{ 1000,  8,  8, 0.25, 0.00 },
+	{ 1001,  8,  8, 0.50, 0.50 },
+	{ 1011, 16, 16, 0.50, 0.50 },
+	{ 1000,  6,  1, 0.50, 0.50 },
+	{ 1000,  8,  8, 0.00, 0.25 }
 };
 
 Cursor::Cursor(Myst3Engine *vm) :
 	_vm(vm),
-	_position(vm->_gfx->frameCenter()),
+	_position(vm->_scene->getCenter()),
 	_hideLevel(0),
 	_lockedAtCenter(false) {
 
+	// The cursor is drawn unscaled
+	_scaled = false;
+	_isConstrainedToWindow = false;
+
 	// Load available cursors
 	loadAvailableCursors();
 
@@ -69,10 +75,15 @@ Cursor::Cursor(Myst3Engine *vm) :
 }
 
 void Cursor::loadAvailableCursors() {
-	// Load available cursors
-	for (uint i = 0; availableCursors[i].nodeID; i++) {
-		const DirectorySubEntry *cursorDesc = _vm->getFileDescription("GLOB", availableCursors[i].nodeID, 0, DirectorySubEntry::kRawData);
+	assert(_textures.empty());
 
+	// Load available cursors
+	for (uint i = 0; i < ARRAYSIZE(availableCursors); i++) {
+		// Check if a cursor sharing the same texture has already been loaded
+		if (_textures.contains(availableCursors[i].nodeID)) continue;
+
+		// Load the cursor bitmap
+		const DirectorySubEntry *cursorDesc = _vm->getFileDescription("GLOB", availableCursors[i].nodeID, 0, DirectorySubEntry::kRawData);
 		if (!cursorDesc)
 			error("Cursor %d does not exist", availableCursors[i].nodeID);
 
@@ -82,33 +93,40 @@ void Cursor::loadAvailableCursors() {
 		if (!bitmapDecoder.loadStream(*bmpStream))
 			error("Could not decode Myst III bitmap");
 		const Graphics::Surface *surfaceBGRA = bitmapDecoder.getSurface();
-		Graphics::Surface *surfaceRGBA = surfaceBGRA->convertTo(Graphics::PixelFormat(4, 8, 8, 8, 8, 0, 8, 16, 24));
+		Graphics::Surface *surfaceRGBA = surfaceBGRA->convertTo(Texture::getRGBAPixelFormat());
 
 		delete bmpStream;
 
 		// Apply the colorkey for transparency
-		for (uint u = 0; u < surfaceRGBA->w; u++) {
-			for (uint v = 0; v < surfaceRGBA->h; v++) {
-				uint32 *pixel = (uint32*)(surfaceRGBA->getBasePtr(u, v));
-				if (*pixel == 0xFF00FF00)
-					*pixel = 0x0000FF00;
+		for (uint y = 0; y < surfaceRGBA->h; y++) {
+			byte *pixels = (byte *)(surfaceRGBA->getBasePtr(0, y));
+			for (uint x = 0; x < surfaceRGBA->w; x++) {
+				byte *r = pixels + 0;
+				byte *g = pixels + 1;
+				byte *b = pixels + 2;
+				byte *a = pixels + 3;
 
+				if (*r == 0 && *g == 0xFF && *b == 0 && *a == 0xFF) {
+					*g = 0;
+					*a = 0;
+				}
+
+				pixels += 4;
 			}
 		}
 
-		availableCursors[i].texture = _vm->_gfx->createTexture(surfaceRGBA);
+		// Create and store the texture
+		_textures.setVal(availableCursors[i].nodeID, _vm->_gfx->createTexture(surfaceRGBA));
+
 		surfaceRGBA->free();
 		delete surfaceRGBA;
 	}
 }
 
 Cursor::~Cursor() {
-	// Free available cursors
-	for (uint i = 0; availableCursors[i].nodeID; i++) {
-		if (availableCursors[i].texture) {
-			_vm->_gfx->freeTexture(availableCursors[i].texture);
-			availableCursors[i].texture = 0;
-		}
+	// Free cursors textures
+	for (TextureMap::iterator it = _textures.begin(); it != _textures.end(); it++) {
+		_vm->_gfx->freeTexture(it->_value);
 	}
 }
 
@@ -116,9 +134,24 @@ void Cursor::changeCursor(uint32 index) {
 	if (index > 12)
 		return;
 
+	if (_vm->getPlatform() == Common::kPlatformXbox) {
+		// The cursor is hidden when it is not hovering hotspots
+		if ((index == 0 || index == 8) && _vm->_state->getViewType() != kCube)
+			index = 12;
+	}
+
 	_currentCursorID = index;
 }
 
+double Cursor::getTransparencyForId(uint32 cursorId) {
+	assert(cursorId < ARRAYSIZE(availableCursors));
+	if (_vm->getPlatform() == Common::kPlatformXbox) {
+		return availableCursors[cursorId].transparencyXbox;
+	} else {
+		return availableCursors[cursorId].transparency;
+	}
+}
+
 void Cursor::lockPosition(bool lock) {
 	if (_lockedAtCenter == lock)
 		return;
@@ -127,9 +160,9 @@ void Cursor::lockPosition(bool lock) {
 
 	g_system->lockMouse(lock);
 
-	Common::Point center = _vm->_gfx->frameCenter();
+	Common::Point center = _vm->_scene->getCenter();
 	if (_lockedAtCenter) {
-		// Locking, just mouve the cursor at the center of the screen
+		// Locking, just move the cursor at the center of the screen
 		_position = center;
 	} else {
 		// Unlocking, warp the actual mouse position to the cursor
@@ -137,36 +170,49 @@ void Cursor::lockPosition(bool lock) {
 	}
 }
 
-void Cursor::updatePosition(Common::Point &mouse) {
+void Cursor::updatePosition(const Common::Point &mouse) {
 	if (!_lockedAtCenter) {
 		_position = mouse;
+	} else {
+		_position = _vm->_scene->getCenter();
 	}
 }
 
-Common::Point Cursor::getPosition() {
-	Common::Rect viewport = _vm->_gfx->viewport();
+Common::Point Cursor::getPosition(bool scaled) {
+	if (scaled) {
+		Common::Rect viewport = _vm->_gfx->viewport();
 
-	// The rest of the engine expects 640x480 coordinates
-	Common::Point scaledPosition = _position;
-	scaledPosition.x -= viewport.left;
-	scaledPosition.y -= viewport.top;
-	scaledPosition.x = CLIP<int16>(scaledPosition.x, 0, viewport.width());
-	scaledPosition.y = CLIP<int16>(scaledPosition.y, 0, viewport.height());
-	scaledPosition.x *= Renderer::kOriginalWidth / (float)viewport.width();
-	scaledPosition.y *= Renderer::kOriginalHeight / (float)viewport.height();
+		// The rest of the engine expects 640x480 coordinates
+		Common::Point scaledPosition = _position;
+		scaledPosition.x -= viewport.left;
+		scaledPosition.y -= viewport.top;
+		scaledPosition.x = CLIP<int16>(scaledPosition.x, 0, viewport.width());
+		scaledPosition.y = CLIP<int16>(scaledPosition.y, 0, viewport.height());
+		scaledPosition.x *= Renderer::kOriginalWidth / (float) viewport.width();
+		scaledPosition.y *= Renderer::kOriginalHeight / (float) viewport.height();
 
-	return scaledPosition;
+		return scaledPosition;
+	} else {
+		return _position;
+	}
 }
 
 void Cursor::draw() {
-	CursorData &cursor = availableCursors[_currentCursorID];
+	assert(_currentCursorID < ARRAYSIZE(availableCursors));
+
+	const CursorData &cursor = availableCursors[_currentCursorID];
+
+	Texture *texture = _textures[cursor.nodeID];
+	if (!texture) {
+		error("No texture for cursor with id %d", cursor.nodeID);
+	}
 
 	// Rect where to draw the cursor
-	Common::Rect screenRect = Common::Rect(cursor.texture->width, cursor.texture->height);
+	Common::Rect screenRect = Common::Rect(texture->width, texture->height);
 	screenRect.translate(_position.x - cursor.hotspotX, _position.y - cursor.hotspotY);
 
 	// Rect where to draw the cursor
-	Common::Rect textureRect = Common::Rect(cursor.texture->width, cursor.texture->height);
+	Common::Rect textureRect = Common::Rect(texture->width, texture->height);
 
 	float transparency = 1.0;
 
@@ -175,10 +221,10 @@ void Cursor::draw() {
 		if (varTransparency >= 0)
 			transparency = varTransparency / 100.0;
 		else
-			transparency = cursor.transparency;
+			transparency = getTransparencyForId(_currentCursorID);
 	}
 
-	_vm->_gfx->drawTexturedRect2D(screenRect, textureRect, cursor.texture, transparency);
+	_vm->_gfx->drawTexturedRect2D(screenRect, textureRect, texture, transparency);
 }
 
 void Cursor::setVisible(bool show) {
@@ -197,7 +243,7 @@ void Cursor::getDirection(float &pitch, float &heading) {
 		pitch = _vm->_state->getLookAtPitch();
 		heading = _vm->_state->getLookAtHeading();
 	} else {
-		_vm->_gfx->screenPosToDirection(_position, pitch, heading);
+		_vm->_scene->screenPosToDirection(_position, pitch, heading);
 	}
 }
 
