@@ -25,40 +25,23 @@
 
 #include "engines/engine.h"
 
+#include "common/array.h"
+#include "common/ptr.h"
 #include "common/system.h"
 #include "common/random.h"
 
-#include "engines/myst3/archive.h"
-#include "engines/myst3/movie.h"
-#include "engines/myst3/node.h"
-#include "engines/myst3/scene.h"
+#include "engines/myst3/directorysubentry.h"
 
 namespace Graphics {
 struct Surface;
 }
 
+namespace Common {
+struct Event;
+}
+
 namespace Myst3 {
 
-enum GameVersionFlags {
-	kFlagNone      = 0,
-	kFlagVersion10 = (1 << 0), // v1.0
-	kFlagDVD       = (1 << 1)  // DVD version
-};
-
-typedef uint32 SafeDiskKey[4];
-
-struct ExecutableVersion {
-	const char *description;
-	int flags;
-	const char *executable;
-	uint32 baseOffset;
-	uint32 ageTableOffset;
-	uint32 nodeInitScriptOffset;
-	uint32 soundNamesOffset;
-	uint32 ambientCuesOffset;
-	const SafeDiskKey *safeDiskKey;
-};
-
 // Engine Debug Flags
 enum {
 	kDebugVariable = (1 << 0),
@@ -75,20 +58,28 @@ enum TransitionType {
 	kTransitionRightToLeft
 };
 
+class Archive;
 class Console;
+class Drawable;
 class GameState;
 class HotSpot;
 class Cursor;
 class Inventory;
 class Database;
+class Scene;
 class Script;
+class SpotItemFace;
+class SunSpot;
 class Renderer;
 class Menu;
+class Node;
 class Sound;
 class Ambient;
+class ScriptedMovie;
 class ShakeEffect;
 class RotationEffect;
 class Transition;
+class FrameLimiter;
 struct NodeData;
 struct Myst3GameDescription;
 
@@ -98,9 +89,11 @@ class Myst3Engine : public Engine {
 
 protected:
 	// Engine APIs
-	virtual Common::Error run();
-	virtual void syncSoundSettings();
-	virtual GUI::Debugger *getDebugger() { return (GUI::Debugger *)_console; }
+	virtual Common::Error run() override;
+	virtual void syncSoundSettings() override;
+	virtual GUI::Debugger *getDebugger() override { return (GUI::Debugger *)_console; }
+	virtual void pauseEngineIntern(bool pause) override;
+
 public:
 	GameState *_state;
 	Scene *_scene;
@@ -120,16 +113,21 @@ public:
 	Myst3Engine(OSystem *syst, const Myst3GameDescription *version);
 	virtual ~Myst3Engine();
 
-	bool hasFeature(EngineFeature f) const;
+	bool hasFeature(EngineFeature f) const override;
 	Common::Platform getPlatform() const;
-	Common::Language getDefaultLanguage() const;
-	bool isMonolingual() const;
-	const ExecutableVersion *getExecutableVersion() const;
+	Common::Language getGameLanguage() const;
+	uint32 getGameLocalizationType() const;
+	bool isWideScreenModEnabled() const;
 
-	bool canLoadGameStateCurrently();
-	Common::Error loadGameState(int slot);
+	bool canLoadGameStateCurrently() override;
+	Common::Error loadGameState(int slot) override;
+	Common::Error loadGameState(Common::String fileName, TransitionType transition);
+
+	const DirectorySubEntry *getFileDescription(const Common::String &room, uint32 index, uint16 face,
+	                                            DirectorySubEntry::ResourceType type);
+	DirectorySubEntryList listFilesMatching(const Common::String &room, uint32 index, uint16 face,
+	                                        DirectorySubEntry::ResourceType type);
 
-	const DirectorySubEntry *getFileDescription(const char* room, uint32 index, uint16 face, DirectorySubEntry::ResourceType type);
 	Graphics::Surface *loadTexture(uint16 id);
 	static Graphics::Surface *decodeJpeg(const DirectorySubEntry *jpegDesc);
 
@@ -156,12 +154,12 @@ public:
 	void loadMovie(uint16 id, uint16 condition, bool resetCond, bool loop);
 	void playMovieGoToNode(uint16 movie, uint16 node);
 	void playMovieFullFrame(uint16 movie);
-	void playSimpleMovie(uint16 id, bool fullframe = false);
+	void playSimpleMovie(uint16 id, bool fullframe = false, bool refreshAmbientSounds = false);
 	void removeMovie(uint16 id);
 	void setMovieLooping(uint16 id, bool loop);
 
-	void addSpotItem(uint16 id, uint16 condition, bool fade);
-	SpotItemFace *addMenuSpotItem(uint16 id, uint16 condition, const Common::Rect &rect);
+	void addSpotItem(uint16 id, int16 condition, bool fade);
+	SpotItemFace *addMenuSpotItem(uint16 id, int16 condition, const Common::Rect &rect);
 	void loadNodeSubtitles(uint32 id);
 
 	void addSunSpot(uint16 pitch, uint16 heading, uint16 intensity,
@@ -170,12 +168,15 @@ public:
 
 	void setMenuAction(uint16 action) { _menuAction = action; }
 
-	void animateDirectionChange(float pitch, float heading, uint16 speed);
+	void animateDirectionChange(float pitch, float heading, uint16 scriptTicks);
 	void getMovieLookAt(uint16 id, bool start, float &pitch, float &heading);
 
-	void processInput(bool lookOnly);
 	void drawFrame(bool noSwap = false);
 
+	void processInput(bool interactive);
+	void updateInputState();
+	void resetInput();
+
 	bool inputValidatePressed();
 	bool inputEscapePressed();
 	bool inputSpacePressed();
@@ -188,7 +189,7 @@ private:
 	OSystem *_system;
 	Console *_console;
 	const Myst3GameDescription *_gameDescription;
-	
+
 	Node *_node;
 
 	Common::Array<Archive *> _archivesCommon;
@@ -207,15 +208,27 @@ private:
 	// Used by Voltaic's spinning gears
 	RotationEffect *_rotationEffect;
 
+	FrameLimiter *_frameLimiter;
 	Transition *_transition;
 
 	bool _inputSpacePressed;
 	bool _inputEnterPressed;
 	bool _inputEscapePressed;
+	bool _inputEscapePressedNotConsumed;
 	bool _inputTildePressed;
 
+	bool _interactive;
+
 	uint32 _backgroundSoundScriptLastRoomId;
 
+	/**
+	 * When the widescreen mode is active, the user can manually hide
+	 * the inventory by clicking on an unused inventory space.
+	 * This allows interacting with the scene portion that is below
+	 * the inventory.
+	 */
+	bool _inventoryManualHide;
+
 	HotSpot *getHoveredHotspot(NodePtr nodeData, uint16 var = 0);
 	void updateCursor();
 
@@ -227,8 +240,8 @@ private:
 
 	bool isInventoryVisible();
 
-	void transitionDraw(TransitionType type, Graphics::Surface *source);
-	void transitionDrawStep(TransitionType type, uint32 *target, uint targetPitch, uint32 *source, uint sourcePitch, uint32 *destination, uint destinationPitch, uint destinationHeight, uint completion);
+	void interactWithHoveredElement();
+	void processEventForGamepad(const Common::Event &event);
 
 	friend class Console;
 };
