@@ -49,13 +49,15 @@ public:
 
 	void setTextureFromJPEG(const DirectorySubEntry *jpegDesc);
 
-	void markTextureDirty() { _textureDirty = true; }
+	void addTextureDirtyRect(const Common::Rect &rect);
 	bool isTextureDirty() { return _textureDirty; }
 
 	void uploadTexture();
 
 private:
 	bool _textureDirty;
+	Common::Rect _textureDirtyRect;
+
 	Myst3Engine *_vm;
 };
 
@@ -78,6 +80,8 @@ public:
 	uint16 getFadeValue() { return _fadeValue; }
 	void setFadeValue(uint16 value) { _fadeValue = value; }
 
+	Common::Rect getFaceRect() const;
+
 private:
 	Face *_face;
 	bool _drawn;
@@ -96,7 +100,7 @@ public:
 	SpotItem(Myst3Engine *vm);
 	~SpotItem();
 
-	void setCondition(uint16 condition) { _condition = condition; }
+	void setCondition(int16 condition) { _condition = condition; }
 	void setFade(bool fade) { _enableFade = fade; }
 	void setFadeVar(uint16 var) { _fadeVar = var; }
 	void addFace(SpotItemFace *face) { _faces.push_back(face); }
@@ -106,7 +110,7 @@ public:
 private:
 	Myst3Engine *_vm;
 
-	uint16 _condition;
+	int16 _condition;
 	uint16 _fadeVar;
 	bool _enableFade;
 
@@ -124,27 +128,28 @@ public:
 	float radius;
 };
 
-class Node : Drawable {
-protected:
-	Myst3Engine *_vm;
-	Face *_faces[6];
-	Common::Array<SpotItem *> _spotItems;
-	Subtitles *_subtitles;
-	Common::Array<Effect *> _effects;
-
+class Node : public Drawable {
 public:
 	Node(Myst3Engine *vm, uint16 id);
 	virtual ~Node();
 
 	void update();
-	virtual void draw() = 0;
 	void drawOverlay();
 
-	void loadSpotItem(uint16 id, uint16 condition, bool fade);
-	SpotItemFace *loadMenuSpotItem(uint16 condition, const Common::Rect &rect);
+	void loadSpotItem(uint16 id, int16 condition, bool fade);
+	SpotItemFace *loadMenuSpotItem(int16 condition, const Common::Rect &rect);
 
 	void loadSubtitles(uint32 id);
 	bool hasSubtitlesToDraw();
+
+protected:
+	virtual bool isFaceVisible(uint faceId) = 0;
+
+	Myst3Engine *_vm;
+	Face *_faces[6];
+	Common::Array<SpotItem *> _spotItems;
+	Subtitles *_subtitles;
+	Common::Array<Effect *> _effects;
 };
 
 } // end of namespace Myst3
