@@ -30,15 +30,16 @@ namespace Myst3 {
 
 NodeCube::NodeCube(Myst3Engine *vm, uint16 id) :
 		Node(vm, id) {
+	_is3D = true;
+
 	for (int i = 0; i < 6; i++) {
-		const DirectorySubEntry *jpegDesc = _vm->getFileDescription(0, id, i + 1, DirectorySubEntry::kCubeFace);
+		const DirectorySubEntry *jpegDesc = _vm->getFileDescription("", id, i + 1, DirectorySubEntry::kCubeFace);
 
 		if (!jpegDesc)
 			error("Face %d does not exist", id);
 
 		_faces[i] = new Face(_vm);
 		_faces[i]->setTextureFromJPEG(jpegDesc);
-		_faces[i]->markTextureDirty();
 	}
 }
 
@@ -47,14 +48,22 @@ NodeCube::~NodeCube() {
 
 void NodeCube::draw() {
 	// Update the OpenGL textures if needed
-	for (uint i = 0; i < 6; i++)
-		_faces[i]->uploadTexture();
+	for (uint i = 0; i < 6; i++) {
+		if (_faces[i]->isTextureDirty() && isFaceVisible(i)) {
+			_faces[i]->uploadTexture();
+		}
+	}
 
 	Texture *textures[6];
-	for (uint i = 0; i < 6; i++)
+	for (uint i = 0; i < 6; i++) {
 		textures[i] = _faces[i]->_texture;
+	}
 
 	_vm->_gfx->drawCube(textures);
 }
 
+bool NodeCube::isFaceVisible(uint faceId) {
+	return _vm->_gfx->isCubeFaceVisible(faceId);
+}
+
 } // End of namespace Myst3
