@@ -27,12 +27,15 @@
 
 namespace Myst3 {
 
-class NodeCube: public Myst3::Node {
+class NodeCube: public Node {
 public:
 	NodeCube(Myst3Engine *vm, uint16 id);
 	virtual ~NodeCube();
 
-	void draw();
+	void draw() override;
+
+protected:
+	virtual bool isFaceVisible(uint faceId) override;
 };
 
 } // End of namespace Myst3
