@@ -24,14 +24,19 @@
 #include "common/config-manager.h"
 
 #include "engines/myst3/scene.h"
+#include "engines/myst3/gfx.h"
 #include "engines/myst3/myst3.h"
+#include "engines/myst3/node.h"
 #include "engines/myst3/state.h"
 
 #include "graphics/colormasks.h"
 
+#include "math/vector2d.h"
+
 namespace Myst3 {
 
 Scene::Scene(Myst3Engine *vm) :
+		Window(),
 		_vm(vm),
 		_mouseSpeed(50) {
 	updateMouseSpeed();
@@ -93,31 +98,20 @@ void Scene::updateCamera(Common::Point &mouse) {
 	_vm->_state->setCameraHeading((int32)heading);
 }
 
-void Scene::drawBlackBorders() {
-	Common::Rect top = Common::Rect(Renderer::kOriginalWidth, Renderer::kTopBorderHeight);
-
-	Common::Rect bottom = Common::Rect(Renderer::kOriginalWidth, Renderer::kBottomBorderHeight);
-	bottom.translate(0, Renderer::kTopBorderHeight + Renderer::kFrameHeight);
-
-	uint32 black = Graphics::ARGBToColor< Graphics::ColorMasks<8888> >(255, 0, 0, 0);
-	_vm->_gfx->drawRect2D(top, black);
-	_vm->_gfx->drawRect2D(bottom, black);
-}
-
 void Scene::drawSunspotFlare(const SunSpot &s) {
 	Common::Rect frame = Common::Rect(Renderer::kOriginalWidth, Renderer::kFrameHeight);
-	frame.translate(0, Renderer::kTopBorderHeight);
 
 	uint8 a = (uint8)(s.intensity * s.radius);
 	uint8 r, g, b;
 	Graphics::colorToRGB< Graphics::ColorMasks<888> >(s.color, r, g, b);
 	uint32 color = Graphics::ARGBToColor< Graphics::ColorMasks<8888> >(a, r, g, b);
 
+	_vm->_gfx->selectTargetWindow(this, false, true);
 	_vm->_gfx->drawRect2D(frame, color);
 }
 
 
-static Math::Vector3d directionToVector(float pitch, float heading) {
+Math::Vector3d Scene::directionToVector(float pitch, float heading) {
 	Math::Vector3d v;
 
 	float radHeading = Common::deg2rad(heading);
@@ -143,4 +137,90 @@ void Scene::updateMouseSpeed() {
 	_mouseSpeed = ConfMan.getInt("mouse_speed");
 }
 
+Common::Rect Scene::getPosition() const {
+	Common::Rect screen = _vm->_gfx->viewport();
+
+	Common::Rect frame;
+	if (_vm->isWideScreenModEnabled()) {
+		int32 viewportWidth = Renderer::kOriginalWidth;
+
+		int32 viewportHeight;
+		if (_vm->_state->getViewType() == kMenu) {
+			viewportHeight = Renderer::kOriginalHeight;
+		} else {
+			viewportHeight = Renderer::kFrameHeight;
+		}
+
+		// Aspect ratio correction
+		frame = Common::Rect(MIN<int32>(screen.width(), screen.height() * viewportWidth / viewportHeight),
+		                     MIN<int32>(screen.height(), screen.width() * viewportHeight / viewportWidth));
+
+		// Pillarboxing
+		uint left = (screen.width() - frame.width()) / 2;
+
+		uint top;
+		if (_vm->_state->getViewType() == kMenu) {
+			top = (screen.height() - frame.height()) / 2;
+		} else {
+			top = (screen.height() - frame.height()) * Renderer::kTopBorderHeight / (Renderer::kTopBorderHeight + Renderer::kBottomBorderHeight);
+		}
+
+		frame.translate(left, top);
+	} else {
+		if (_vm->_state->getViewType() != kMenu) {
+			frame = Common::Rect(screen.width(), screen.height() * Renderer::kFrameHeight / Renderer::kOriginalHeight);
+			frame.translate(screen.left, screen.top + screen.height() * Renderer::kTopBorderHeight / Renderer::kOriginalHeight);
+		} else {
+			frame = screen;
+		}
+	}
+
+	return frame;
+}
+
+Common::Rect Scene::getOriginalPosition() const {
+	Common::Rect originalPosition;
+
+	if (_vm->_state->getViewType() != kMenu) {
+		originalPosition = Common::Rect(Renderer::kOriginalWidth, Renderer::kFrameHeight);
+		originalPosition.translate(0, Renderer::kTopBorderHeight);
+	} else {
+		originalPosition = Common::Rect(Renderer::kOriginalWidth, Renderer::kOriginalHeight);
+	}
+
+	return originalPosition;
+}
+
+void Scene::screenPosToDirection(const Common::Point &screen, float &pitch, float &heading) const {
+	Common::Rect frame = getPosition();
+
+	// Screen coords to window coords
+	Common::Point pos = screenPosToWindowPos(screen);
+
+	// Window coords to normalized coords
+	Math::Vector4d in;
+	in.x() = pos.x * 2 / (float) frame.width() - 1.0;
+	in.y() = 1.0 - pos.y * 2 / (float) frame.height();
+	in.z() = 1.0;
+	in.w() = 1.0;
+
+	// Normalized coords to direction
+	Math::Matrix4 A = _vm->_gfx->getMvpMatrix();
+	A.inverse();
+	Math::Vector4d out = A.transform(in);
+
+	Math::Vector3d direction(out.x(), out.y(), out.z());
+	direction.normalize();
+
+	// 3D coords to polar coords
+	Math::Vector2d horizontalProjection = Math::Vector2d(direction.x(), direction.z());
+	horizontalProjection.normalize();
+
+	pitch = 90 - Math::Angle::arcCosine(direction.y()).getDegrees();
+	heading = Math::Angle::arcCosine(horizontalProjection.getY()).getDegrees();
+
+	if (horizontalProjection.getX() > 0.0)
+		heading = 360 - heading;
+}
+
 } // end of namespace Myst3
