@@ -27,9 +27,11 @@
 #include "engines/myst3/state.h"
 
 #include "audio/audiostream.h"
+#include "audio/decoders/asf.h"
 #include "audio/decoders/mp3.h"
 #include "audio/decoders/wave.h"
 
+#include "common/archive.h"
 #include "common/config-manager.h"
 
 namespace Myst3 {
@@ -55,8 +57,12 @@ void Sound::playEffect(uint32 id, uint32 volume, uint16 heading, uint16 attenuat
 void Sound::playEffectLooping(uint32 id, uint32 volume, uint16 heading, uint16 attenuation) {
 	id = _vm->_state->valueOrVarValue(id);
 
-	SoundChannel *channel = getChannelForSound(id, kEffect);
-	channel->play(id, volume, heading, attenuation, true, kEffect);
+	bool alreadyPlaying;
+	SoundChannel *channel = getChannelForSound(id, kEffect, &alreadyPlaying);
+
+	if (!alreadyPlaying) {
+		channel->play(id, volume, heading, attenuation, true, kEffect);
+	}
 }
 
 void Sound::playEffectFadeInOut(uint32 id, uint32 volume, uint16 heading, uint16 attenuation,
@@ -269,11 +275,11 @@ void Sound::setupNextSound(SoundNextCommand command, int16 controlVar, int16 sta
 	_vm->_state->setSoundNextId(0);
 	_vm->_state->setSoundNextIsLast(false);
 
-	uint32 controlLastFrame = _vm->_state->getVar(controlVar);
+	uint32 controlLastTick = _vm->_state->getVar(controlVar);
 	int32 playingSoundId = _vm->_state->getVar(controlVar + 1) >> 16;
 	int32 soundDelay = _vm->_state->getVar(controlVar + 1) & 0xFFFF;
 
-	if (!controlLastFrame) {
+	if (!controlLastTick) {
 		if (!playSeveralSounds) {
 			for (int16 i = startSoundId; i < startSoundId + soundCount; i++) {
 				int16 soundVarValue = _vm->_state->getVar(i);
@@ -289,17 +295,17 @@ void Sound::setupNextSound(SoundNextCommand command, int16 controlVar, int16 sta
 		return;
 	}
 
-	uint currentFrame = _vm->_state->getFrameCount();
-	if (currentFrame == controlLastFrame) {
+	uint currentTick = _vm->_state->getTickCount();
+	if (currentTick == controlLastTick) {
 		return;
 	}
 
-	if (currentFrame < controlLastFrame) {
+	if (currentTick < controlLastTick) {
 		soundDelay = 0;
-	} else if (currentFrame > controlLastFrame + 10) {
+	} else if (currentTick > controlLastTick + 10) {
 		soundDelay -= 10;
 	} else {
-		soundDelay -= currentFrame - controlLastFrame;
+		soundDelay -= currentTick - controlLastTick;
 	}
 
 	if (soundDelay < 0) {
@@ -307,7 +313,7 @@ void Sound::setupNextSound(SoundNextCommand command, int16 controlVar, int16 sta
 	}
 
 	if (soundDelay) {
-		_vm->_state->setVar(controlVar, currentFrame);
+		_vm->_state->setVar(controlVar, currentTick);
 		_vm->_state->setVar(controlVar + 1, soundDelay | (playingSoundId << 16));
 		return;
 	}
@@ -464,13 +470,19 @@ Audio::RewindableAudioStream *SoundChannel::makeAudioStream(const Common::String
 	Common::SeekableReadStream *s = SearchMan.createReadStreamForMember(filename);
 
 	bool isMP3 = false;
+	bool isWMA = false;
 
 	if (!s)
 		s = SearchMan.createReadStreamForMember(filename + ".wav");
 
 	if (!s) {
 		s = SearchMan.createReadStreamForMember(filename + ".mp3");
-		isMP3 = true;
+		if (s) isMP3 = true;
+	}
+
+	if (!s) {
+		s = SearchMan.createReadStreamForMember(filename + ".wma");
+		if (s) isWMA = true;
 	}
 
 	if (!s)
@@ -484,6 +496,8 @@ Audio::RewindableAudioStream *SoundChannel::makeAudioStream(const Common::String
 		delete s;
 		return NULL;
 #endif
+	} else if (isWMA) {
+		return Audio::makeASFStream(s, DisposeAfterUse::YES);
 	} else {
 		return Audio::makeWAVStream(s, DisposeAfterUse::YES);
 	}
@@ -626,14 +640,20 @@ void SoundChannel::updateFading() {
 }
 
 uint32 SoundChannel::playedFrames() {
-	Audio::Timestamp elapsed = g_system->getMixer()->getElapsedTime(_handle);
-
-	// Don't count completed loops in
-	while (elapsed > _length) {
-		elapsed = elapsed - _length;
+	uint32 length = _length.msecs();
+	if (!length) {
+		warning("Unable to retrieve length for sound %d", _id);
+		return 0;
 	}
 
-	return elapsed.msecs() * 30 / 1000;
+	uint32 elapsed = g_system->getMixer()->getSoundElapsedTime(_handle);
+
+	// Don't count completed loops in
+	while (elapsed > length) {
+		elapsed -= length;
+	}
+
+	return elapsed * 30 / 1000;
 }
 
 } // End of namespace Myst3
