@@ -22,15 +22,19 @@
 
 #include "engines/myst3/state.h"
 #include "engines/myst3/database.h"
+#include "engines/myst3/gfx.h"
 
+#include "common/debug-channels.h"
 #include "common/savefile.h"
 
+#include "graphics/surface.h"
+
 namespace Myst3 {
 
 GameState::StateData::StateData() {
 	version = GameState::kSaveVersion;
 	gameRunning = true;
-	currentFrame = 0;
+	tickCount = 0;
 	nextSecondsUpdate = 0;
 	secondsPlayed = 0;
 	dword_4C2C44 = 0;
@@ -75,10 +79,11 @@ GameState::StateData::StateData() {
 	saveMinute = 0;
 }
 
-GameState::GameState(Myst3Engine *vm):
-		_vm(vm) {
+GameState::GameState(const Common::Platform platform, Database *database):
+		_platform(platform),
+		_db(database) {
 
-#define VAR(var, x, unk) _varDescriptions.setVal(var, VarDescription(var, #x, unk));
+#define VAR(var, x, unk) _varDescriptions.setVal(#x, VarDescription(var, #x, unk));
 
 	VAR(14, CursorTransparency, false)
 
@@ -104,7 +109,7 @@ GameState::GameState(Myst3Engine *vm):
 	VAR(69, MenuSavedNode, false)
 
 	VAR(70, SecondsCountdown, false)
-	VAR(71, FrameCountdown, false)
+	VAR(71, TickCountdown, false)
 
 	// Counters, unused by the game scripts
 	VAR(76, CounterUnk76, false)
@@ -147,7 +152,7 @@ GameState::GameState(Myst3Engine *vm):
 	VAR(111, MagnetEffectUnk3, false)
 
 	VAR(112, ShakeEffectAmpl, false)
-	VAR(113, ShakeEffectFramePeriod, false)
+	VAR(113, ShakeEffectTickPeriod, false)
 	VAR(114, RotationEffectSpeed, false)
 	VAR(115, SunspotIntensity, false)
 	VAR(116, SunspotColor, false)
@@ -174,8 +179,8 @@ GameState::GameState(Myst3Engine *vm):
 	VAR(140, CameraMinPitch, false)
 	VAR(141, CameraMaxPitch, false)
 
-	VAR(142, MovieOverrideStartFrame, false)
-	VAR(143, MovieOverrideEndFrame, false)
+	VAR(142, MovieStartFrame, false)
+	VAR(143, MovieEndFrame, false)
 	VAR(144, MovieVolume1, false)
 	VAR(145, MovieVolume2, false)
 	VAR(146, MovieOverrideSubtitles, false)
@@ -266,12 +271,16 @@ GameState::GameState(Myst3Engine *vm):
 
 	// Amateria ambient sound / movie counters (XXXX 1001 and XXXX 1002)
 	VAR(406, AmateriaSecondsCounter, false)
-	VAR(407, AmateriaFramesCounter, false)
+	VAR(407, AmateriaTicksCounter, false)
 
 	VAR(444, ResonanceRingsSolved, false)
 
 	VAR(460, PinballRemainingPegs, false)
 
+	VAR(475, OuterShieldUp, false)
+	VAR(476, InnerShieldUp, false)
+	VAR(479, SaavedroStatus, false)
+
 	VAR(480, BookStateTomahna, false)
 	VAR(481, BookStateReleeshahn, false)
 
@@ -296,24 +305,6 @@ GameState::GameState(Myst3Engine *vm):
 	VAR(1326, MouseSpeed, false)
 	VAR(1327, DialogResult, false)
 
-	VAR(1337, MenuEscapePressed, false)
-	VAR(1338, MenuNextAction, false)
-	VAR(1339, MenuLoadBack, false)
-	VAR(1340, MenuSaveBack, false)
-	VAR(1341, MenuSaveAction, false)
-	VAR(1342, MenuOptionsBack, false)
-
-	VAR(1350, MenuSaveLoadPageLeft, false)
-	VAR(1351, MenuSaveLoadPageRight, false)
-	VAR(1352, MenuSaveLoadSelectedItem, false)
-	VAR(1353, MenuSaveLoadCurrentPage, false)
-
-	VAR(1374, OverallVolume, false)
-	VAR(1377, MusicVolume, false)
-	VAR(1380, MusicFrequency, false)
-	VAR(1393, LanguageAudio, false)
-	VAR(1394, LanguageText, false)
-
 	VAR(1395, HotspotIgnoreClick, false)
 	VAR(1396, HotspotHovered, false)
 	VAR(1397, SpotSubtitle, false)
@@ -337,10 +328,61 @@ GameState::GameState(Myst3Engine *vm):
 	VAR(147, MovieUnk147, true)
 	VAR(148, MovieUnk148, true)
 
-	// Menu stuff does not look like it's too useful
-	VAR(1361, Unk1361, true)
-	VAR(1362, Unk1362, true)
-	VAR(1363, Unk1363, true)
+	if (_platform != Common::kPlatformXbox) {
+		VAR(1337, MenuEscapePressed, false)
+		VAR(1338, MenuNextAction, false)
+		VAR(1339, MenuLoadBack, false)
+		VAR(1340, MenuSaveBack, false)
+		VAR(1341, MenuSaveAction, false)
+		VAR(1342, MenuOptionsBack, false)
+
+		VAR(1350, MenuSaveLoadPageLeft, false)
+		VAR(1351, MenuSaveLoadPageRight, false)
+		VAR(1352, MenuSaveLoadSelectedItem, false)
+		VAR(1353, MenuSaveLoadCurrentPage, false)
+
+		// Menu stuff does not look like it's too useful
+		VAR(1361, Unk1361, true)
+		VAR(1362, Unk1362, true)
+		VAR(1363, Unk1363, true)
+
+		VAR(1374, OverallVolume, false)
+		VAR(1377, MusicVolume, false)
+		VAR(1380, MusicFrequency, false)
+		VAR(1393, LanguageAudio, false)
+		VAR(1394, LanguageText, false)
+
+		VAR(1406, ShieldEffectActive, false)
+
+	} else {
+		shiftVariables(927, 1);
+		shiftVariables(1031, 2);
+		shiftVariables(1395, -22);
+
+		VAR(1340, MenuSavesAvailable, false)
+		VAR(1341, MenuNextAction, false)
+		VAR(1342, MenuLoadBack, false)
+		VAR(1343, MenuSaveBack, false)
+		VAR(1344, MenuSaveAction, false)
+		VAR(1345, MenuOptionsBack, false)
+		VAR(1346, MenuSelectedSave, false)
+
+		VAR(1384, MovieOptional, false)
+		VAR(1386, VibrationEnabled, false)
+
+		VAR(1430, GamePadActionPressed, false)
+		VAR(1431, GamePadDownPressed, false)
+		VAR(1432, GamePadUpPressed, false)
+		VAR(1433, GamePadLeftPressed, false)
+		VAR(1434, GamePadRightPressed, false)
+		VAR(1435, GamePadCancelPressed, false)
+
+		VAR(1437, DragWithDirectionKeys, false)
+		VAR(1438, MenuAttractCountDown, false)
+		VAR(1439, ShieldEffectActive, false)
+
+		VAR(1445, StateCanSave, false)
+	}
 
 #undef VAR
 
@@ -369,7 +411,7 @@ void GameState::StateData::syncWithSaveGame(Common::Serializer &s) {
 		error("This savegame (v%d) is too recent (max %d) please get a newer version of ResidualVM", s.getVersion(), kSaveVersion);
 
 	s.syncAsUint32LE(gameRunning);
-	s.syncAsUint32LE(currentFrame);
+	s.syncAsUint32LE(tickCount);
 	s.syncAsUint32LE(nextSecondsUpdate);
 	s.syncAsUint32LE(secondsPlayed);
 	s.syncAsUint32LE(dword_4C2C44);
@@ -432,24 +474,57 @@ void GameState::StateData::syncWithSaveGame(Common::Serializer &s) {
 	s.syncAsByte(saveMinute, 149);
 	s.syncString(saveDescription, 149);
 
+#ifdef SCUMM_BIG_ENDIAN
+	Graphics::PixelFormat saveFormat = Graphics::PixelFormat(4, 8, 8, 8, 0, 8, 16, 24, 0);
+#else
+	Graphics::PixelFormat saveFormat = Graphics::PixelFormat(4, 8, 8, 8, 0, 16, 8, 0, 24);
+#endif
+
 	if (s.isLoading()) {
 		thumbnail = Common::SharedPtr<Graphics::Surface>(new Graphics::Surface(), Graphics::SharedPtrSurfaceDeleter());
-		thumbnail->create(kThumbnailWidth, kThumbnailHeight, Graphics::PixelFormat(4, 8, 8, 8, 8, 16, 8, 0, 24));
-	}
-	assert(thumbnail && thumbnail->w == kThumbnailWidth && thumbnail->h == kThumbnailHeight);
+		thumbnail->create(kThumbnailWidth, kThumbnailHeight, saveFormat);
 
-	s.syncBytes((byte *)thumbnail->getPixels(), kThumbnailWidth * kThumbnailHeight * 4);
+		s.syncBytes((byte *)thumbnail->getPixels(), kThumbnailWidth * kThumbnailHeight * 4);
+
+		thumbnail->convertToInPlace(Texture::getRGBAPixelFormat());
+	} else {
+		assert(thumbnail->format == Texture::getRGBAPixelFormat());
+		assert(thumbnail && thumbnail->w == kThumbnailWidth && thumbnail->h == kThumbnailHeight);
+
+		Graphics::Surface *converted = thumbnail->convertTo(saveFormat);
+
+		s.syncBytes((byte *)converted->getPixels(), kThumbnailWidth * kThumbnailHeight * 4);
+
+		converted->free();
+		delete converted;
+	}
+}
+
+void GameState::StateData::resizeThumbnail(Graphics::Surface *small) const {
+	Graphics::Surface *big = thumbnail.get();
+	assert(small->format == big->format && big->format.bytesPerPixel == 4);
+
+	uint32 *dst = (uint32 *)small->getPixels();
+	for (uint i = 0; i < small->h; i++) {
+		for (uint j = 0; j < small->w; j++) {
+			uint32 srcX = big->w * j / small->w;
+			uint32 srcY = big->h * i / small->h;
+			uint32 *src = (uint32 *)big->getBasePtr(srcX, srcY);
+
+			// Copy RGBA pixel
+			*dst++ = *src;
+		}
+	}
 }
 
 void GameState::newGame() {
 	_data = StateData();
+	_lastTickStartTime = g_system->getMillis();
 }
 
-bool GameState::load(const Common::String &file) {
-	Common::InSaveFile *saveFile = _vm->getSaveFileManager()->openForLoading(file);
+bool GameState::load(Common::InSaveFile *saveFile) {
 	Common::Serializer s = Common::Serializer(saveFile, 0);
 	_data.syncWithSaveGame(s);
-	delete saveFile;
 
 	_data.gameRunning = true;
 
@@ -486,6 +561,16 @@ void GameState::setSaveThumbnail(Graphics::Surface *thumb) {
 	_data.thumbnail = Common::SharedPtr<Graphics::Surface>(thumb, Graphics::SharedPtrSurfaceDeleter());
 }
 
+Common::String GameState::formatSaveTime() {
+	if (_data.saveYear == 0)
+		return "";
+
+	// TODO: Check the Xbox NTSC version, maybe it uses that strange MM/DD/YYYY format
+	return Common::String::format("%02d/%02d/%02d %02d:%02d",
+			_data.saveDay, _data.saveMonth, _data.saveYear,
+			_data.saveHour, _data.saveMinute);
+}
+
 Common::Array<uint16> GameState::getInventory() {
 	Common::Array<uint16> items;
 
@@ -510,6 +595,24 @@ void GameState::checkRange(uint16 var) {
 		error("Variable out of range %d", var);
 }
 
+const GameState::VarDescription GameState::findDescription(uint16 var) {
+	for (VarMap::const_iterator it = _varDescriptions.begin(); it != _varDescriptions.end(); it++) {
+		if (it->_value.var == var) {
+			return it->_value;
+		}
+	}
+
+	return VarDescription();
+}
+
+void GameState::shiftVariables(uint16 base, int32 value) {
+	for (VarMap::iterator it = _varDescriptions.begin(); it != _varDescriptions.end(); it++) {
+		if (it->_value.var >= base) {
+			it->_value.var += value;
+		}
+	}
+}
+
 int32 GameState::getVar(uint16 var) {
 	checkRange(var);
 
@@ -519,10 +622,12 @@ int32 GameState::getVar(uint16 var) {
 void GameState::setVar(uint16 var, int32 value) {
 	checkRange(var);
 
-	if (_varDescriptions.contains(var)) {
-		const VarDescription &d = _varDescriptions.getVal(var);
-		if (d.unknown)
-			debugC(kDebugScript, "A script is writing to the unimplemented engine-mapped var %d (%s)", var, d.name);
+	if (DebugMan.isDebugChannelEnabled(kDebugVariable)) {
+		const VarDescription &d = findDescription(var);
+
+		if (d.name && d.unknown) {
+			warning("A script is writing to the unimplemented engine-mapped var %d (%s)", var, d.name);
+		}
 	}
 
 	_data.vars[var] = value;
@@ -531,24 +636,6 @@ void GameState::setVar(uint16 var, int32 value) {
 bool GameState::evaluate(int16 condition) {
 	uint16 unsignedCond = abs(condition);
 	uint16 var = unsignedCond & 2047;
-
-	switch (var) {
-	case 84:
-		setInputMousePressed(_vm->inputValidatePressed());
-		break;
-	case 88:
-		setInputEscapePressed(_vm->inputEscapePressed());
-		break;
-	case 89:
-		setInputTildePressed(_vm->inputTilePressed());
-		break;
-	case 90:
-		setInputSpacePressed(_vm->inputSpacePressed());
-		break;
-	default:
-		break;
-	}
-
 	int32 varValue = getVar(var);
 	int32 targetValue = (unsignedCond >> 11) - 1;
 
@@ -572,24 +659,28 @@ int32 GameState::valueOrVarValue(int16 value) {
 	return value;
 }
 
-int32 GameState::engineGet(uint16 var) {
-	if (!_varDescriptions.contains(var))
-		error("The engine is trying to access an undescribed var (%d)", var);
+int32 GameState::engineGet(const Common::String &varName) {
+	if (!_varDescriptions.contains(varName))
+		error("The engine is trying to access an undescribed var (%s)", varName.c_str());
 
-	return _data.vars[var];
+	const VarDescription &d = _varDescriptions.getVal(varName);
+
+	return _data.vars[d.var];
 }
 
-void GameState::engineSet(uint16 var, int32 value) {
-	if (!_varDescriptions.contains(var))
-		error("The engine is trying to access an undescribed var (%d)", var);
+void GameState::engineSet(const Common::String &varName, int32 value) {
+	if (!_varDescriptions.contains(varName))
+		error("The engine is trying to access an undescribed var (%s)", varName.c_str());
 
-	_data.vars[var] = value;
+	const VarDescription &d = _varDescriptions.getVal(varName);
+
+	_data.vars[d.var] = value;
 }
 
 const Common::String GameState::describeVar(uint16 var) {
-	if (_varDescriptions.contains(var)) {
-		const VarDescription &d = _varDescriptions.getVal(var);
+	const VarDescription &d = findDescription(var);
 
+	if (d.name) {
 		return Common::String::format("v%s", d.name);
 	} else {
 		return Common::String::format("v%d", var);
@@ -616,19 +707,18 @@ void GameState::limitCubeCamera(float minPitch, float maxPitch, float minHeading
 }
 
 void GameState::updateFrameCounters() {
-	_data.currentFrame++;
-
 	if (!_data.gameRunning)
 		return;
 
-	int32 frameCountdown = getFrameCountdown();
-	if (frameCountdown > 0)
-		setFrameCountdown(--frameCountdown);
-
-	if (getAmateriaFramesCounter() > 0)
-		setAmateriaFramesCounter(getAmateriaFramesCounter() - 1);
-
 	uint32 currentTime = g_system->getMillis();
+	int32 timeToNextTick = _lastTickStartTime + kTickDuration - currentTime;
+
+	if (timeToNextTick <= 0) {
+		_data.tickCount++;
+		updateTickCounters();
+		_lastTickStartTime = currentTime + timeToNextTick;
+	}
+
 	if (currentTime > _data.nextSecondsUpdate || ABS<int32>(_data.nextSecondsUpdate - currentTime) > 2000) {
 		_data.secondsPlayed++;
 		_data.nextSecondsUpdate = currentTime + 1000;
@@ -642,29 +732,51 @@ void GameState::updateFrameCounters() {
 
 		if (getSoundScriptsTimer() > 0)
 			setSoundScriptsTimer(getSoundScriptsTimer() - 1);
-	}
 
-	if (getSweepEnabled()) {
-		if (getSweepValue() + getSweepStep() > getSweepMax()) {
-			setSweepValue(getSweepMax());
-
-			if (getSweepStep() > 0) {
-				setSweepStep(-getSweepStep());
-			}
-		} else if (getSweepValue() + getSweepStep() < getSweepMin()) {
-			setSweepValue(getSweepMin());
-
-			if (getSweepStep() < 0) {
-				setSweepStep(-getSweepStep());
-			}
-		} else {
-			setSweepValue(getSweepValue() + getSweepStep());
-		}
+		if (hasVarMenuAttractCountDown() && getMenuAttractCountDown() > 0)
+			setMenuAttractCountDown(getMenuAttractCountDown() - 1);
 	}
 }
 
-bool GameState::isZipDestinationAvailable(uint16 node, uint16 room) {
-	int32 zipBitIndex = _vm->_db->getNodeZipBitIndex(node, room);
+void GameState::updateTickCounters() {
+	int32 tickCountdown = getTickCountdown();
+	if (tickCountdown > 0)
+			setTickCountdown(--tickCountdown);
+
+	if (getAmateriaTicksCounter() > 0)
+			setAmateriaTicksCounter(getAmateriaTicksCounter() - 1);
+
+	if (getSweepEnabled()) {
+			if (getSweepValue() + getSweepStep() > getSweepMax()) {
+				setSweepValue(getSweepMax());
+
+				if (getSweepStep() > 0) {
+					setSweepStep(-getSweepStep());
+				}
+			} else if (getSweepValue() + getSweepStep() < getSweepMin()) {
+				setSweepValue(getSweepMin());
+
+				if (getSweepStep() < 0) {
+					setSweepStep(-getSweepStep());
+				}
+			} else {
+				setSweepValue(getSweepValue() + getSweepStep());
+			}
+		}
+}
+
+uint GameState::getTickCount() const {
+	return _data.tickCount;
+}
+
+void GameState::pauseEngine(bool pause) {
+	if (!pause) {
+		_lastTickStartTime = g_system->getMillis();
+	}
+}
+
+bool GameState::isZipDestinationAvailable(uint16 node, uint16 room, uint32 age) {
+	int32 zipBitIndex = _db->getNodeZipBitIndex(node, room, age);
 
 	int32 arrayIndex = zipBitIndex / 32;
 	assert(arrayIndex < 64);
@@ -672,8 +784,8 @@ bool GameState::isZipDestinationAvailable(uint16 node, uint16 room) {
 	return (_data.zipDestinations[arrayIndex] & (1 << (zipBitIndex % 32))) != 0;
 }
 
-void GameState::markNodeAsVisited(uint16 node, uint16 room) {
-	int32 zipBitIndex = _vm->_db->getNodeZipBitIndex(node, room);
+void GameState::markNodeAsVisited(uint16 node, uint16 room, uint32 age) {
+	int32 zipBitIndex = _db->getNodeZipBitIndex(node, room, age);
 
 	int32 arrayIndex = zipBitIndex / 32;
 	assert(arrayIndex < 64);
@@ -681,4 +793,25 @@ void GameState::markNodeAsVisited(uint16 node, uint16 room) {
 	_data.zipDestinations[arrayIndex] |= 1 << (zipBitIndex % 32);
 }
 
+Common::String Saves::buildName(const char *name, Common::Platform platform) {
+	const char *format;
+
+	if (platform == Common::kPlatformXbox) {
+		format = "%s.m3x";
+	} else {
+		format = "%s.m3s";
+	}
+
+	return Common::String::format(format, name);
+}
+
+Common::StringArray Saves::list(Common::SaveFileManager *saveFileManager, Common::Platform platform) {
+	Common::String searchPattern = Saves::buildName("*", platform);
+	Common::StringArray filenames = saveFileManager->listSavefiles(searchPattern);
+
+	// The saves are sorted alphabetically
+	Common::sort(filenames.begin(), filenames.end());
+
+	return filenames;
+}
 } // End of namespace Myst3
