@@ -25,8 +25,12 @@
 
 #include "engines/myst3/myst3.h"
 
+#include "engines/myst3/gfx.h"
+
 namespace Myst3 {
 
+class FrameLimiter;
+
 class Transition {
 public:
 	Transition(Myst3Engine *vm);
@@ -40,6 +44,7 @@ private:
 	void playSound();
 
 	Myst3Engine *_vm;
+	FrameLimiter *_frameLimiter;
 	TransitionType _type;
 
 	Graphics::Surface *_sourceScreenshot;
