@@ -50,7 +50,7 @@ void tglVertex3f(float x, float y, float z)  {
 	tglVertex4f(x, y, z, 1);
 }
 
-void tglVertex3fv(float *v)  {
+void tglVertex3fv(const float *v)  {
 	tglVertex4f(v[0], v[1], v[2], 1);
 }
 
@@ -67,7 +67,7 @@ void tglNormal3f(float x, float y, float z) {
 	TinyGL::gl_add_op(p);
 }
 
-void tglNormal3fv(float *v)  {
+void tglNormal3fv(const float *v)  {
 	tglNormal3f(v[0], v[1], v[2]);
 }
 
@@ -81,38 +81,25 @@ void tglColor4f(float r, float g, float b, float a) {
 	p[2].f = g;
 	p[3].f = b;
 	p[4].f = a;
-	// direct convertion to integer to go faster if no shading
-	p[5].ui = (unsigned int)(r * (ZB_POINT_RED_MAX - ZB_POINT_RED_MIN) + ZB_POINT_RED_MIN);
-	p[6].ui = (unsigned int)(g * (ZB_POINT_GREEN_MAX - ZB_POINT_GREEN_MIN) + ZB_POINT_GREEN_MIN);
-	p[7].ui = (unsigned int)(b * (ZB_POINT_BLUE_MAX - ZB_POINT_BLUE_MIN) + ZB_POINT_BLUE_MIN);
-	p[8].ui = (unsigned int)(a * (ZB_POINT_ALPHA_MAX - ZB_POINT_ALPHA_MIN) + ZB_POINT_ALPHA_MIN);
 	gl_add_op(p);
 }
 
-void tglColor4fv(float *v) {
-	TinyGL::GLParam p[9];
-
-	p[0].op = TinyGL::OP_Color;
-	p[1].f = v[0];
-	p[2].f = v[1];
-	p[3].f = v[2];
-	p[4].f = v[3];
-	// direct convertion to integer to go faster if no shading
-	p[5].ui = (unsigned int)(v[0] * (ZB_POINT_RED_MAX - ZB_POINT_RED_MIN) + ZB_POINT_RED_MIN);
-	p[6].ui = (unsigned int)(v[1] * (ZB_POINT_GREEN_MAX - ZB_POINT_GREEN_MIN) + ZB_POINT_GREEN_MIN);
-	p[7].ui = (unsigned int)(v[2] * (ZB_POINT_BLUE_MAX - ZB_POINT_BLUE_MIN) + ZB_POINT_BLUE_MIN);
-	p[8].ui = (unsigned int)(v[3] * (ZB_POINT_ALPHA_MAX - ZB_POINT_ALPHA_MIN) + ZB_POINT_ALPHA_MIN);
-	TinyGL::gl_add_op(p);
+void tglColor4fv(const float *v) {
+	tglColor4f(v[0], v[1], v[2], v[3]);
 }
 
 void tglColor3f(float x, float y, float z) {
 	tglColor4f(x, y, z, 1);
 }
 
-void tglColor3fv(float *v)  {
+void tglColor3fv(const float *v)  {
 	tglColor4f(v[0], v[1], v[2], 1);
 }
 
+void tglColor3ub(unsigned char r, unsigned char g, unsigned char b) {
+	tglColor4f(r / 255.0f, g / 255.0f, b / 255.0f, 1.0f);
+}
+
 void tglColor4ub(unsigned char r, unsigned char g, unsigned char b, unsigned char a) {
 	tglColor4f(r / 255.0f, g / 255.0f, b / 255.0f, a / 255.0f);
 }
@@ -135,7 +122,7 @@ void tglTexCoord2f(float s, float t) {
 	tglTexCoord4f(s, t, 0, 1);
 }
 
-void tglTexCoord2fv(float *v) {
+void tglTexCoord2fv(const float *v) {
 	tglTexCoord4f(v[0], v[1], 0, 1);
 }
 
@@ -415,7 +402,7 @@ void tglOrtho(double left, double right, double bottom, double top, double zNear
 
 // lightening
 
-void tglMaterialfv(int mode, int type, float *v) {
+void tglMaterialfv(int mode, int type, const float *v) {
 	TinyGL::GLParam p[7];
 	int n;
 
@@ -458,7 +445,7 @@ void tglColorMaterial(int mode, int type) {
 	TinyGL::gl_add_op(p);
 }
 
-void tglLightfv(int light, int type, float *v) {
+void tglLightfv(int light, int type, const float *v) {
 	TinyGL::GLParam p[7];
 
 	p[0].op = TinyGL::OP_Light;
@@ -497,7 +484,7 @@ void tglLightModeli(int pname, int param) {
 	TinyGL::gl_add_op(p);
 }
 
-void tglLightModelfv(int pname, float *param) {
+void tglLightModelfv(int pname, const float *param) {
 	TinyGL::GLParam p[6];
 
 	p[0].op = TinyGL::OP_LightModel;
@@ -698,3 +685,8 @@ void tglSetShadowColor(unsigned char r, unsigned char g, unsigned char b) {
 	c->fb->shadow_color_g = g << 8;
 	c->fb->shadow_color_b = b << 8;
 }
+
+void tglEnableDirtyRects(bool enable) {
+	TinyGL::GLContext *c = TinyGL::gl_get_context();
+	c->_enableDirtyRectangles = enable;
+}
