@@ -49,27 +49,15 @@ void gl_transform_to_viewport(GLContext *c, GLVertex *v) {
 	v->zp.y = (int)(v->pc.Y * winv * c->viewport.scale.Y + c->viewport.trans.Y);
 	v->zp.z = (int)(v->pc.Z * winv * c->viewport.scale.Z + c->viewport.trans.Z);
 	// color
-	if (c->lighting_enabled) {
-		v->zp.r = (int)(v->color.X * (ZB_POINT_RED_MAX - ZB_POINT_RED_MIN)
-					+ ZB_POINT_RED_MIN);
-		v->zp.g = (int)(v->color.Y * (ZB_POINT_GREEN_MAX - ZB_POINT_GREEN_MIN)
-					+ ZB_POINT_GREEN_MIN);
-		v->zp.b = (int)(v->color.Z * (ZB_POINT_BLUE_MAX - ZB_POINT_BLUE_MIN)
-					+ ZB_POINT_BLUE_MIN);
-		v->zp.a = (int)(v->color.W * (ZB_POINT_ALPHA_MAX - ZB_POINT_ALPHA_MIN)
-			+ ZB_POINT_ALPHA_MIN);
-	} else {
-		// no need to convert to integer if no lighting : take current color
-		v->zp.r = c->longcurrent_color[0];
-		v->zp.g = c->longcurrent_color[1];
-		v->zp.b = c->longcurrent_color[2];
-		v->zp.a = c->longcurrent_color[3];
-	}
+	v->zp.r = (int)(v->color.X * ZB_POINT_RED_MAX);
+	v->zp.g = (int)(v->color.Y * ZB_POINT_GREEN_MAX);
+	v->zp.b = (int)(v->color.Z * ZB_POINT_BLUE_MAX);
+	v->zp.a = (int)(v->color.W * ZB_POINT_ALPHA_MAX);
 
 	// texture
 	if (c->texture_2d_enabled) {
-		v->zp.s = (int)(v->tex_coord.X * (ZB_POINT_ST_MAX - ZB_POINT_ST_MIN) + ZB_POINT_ST_MIN);
-		v->zp.t = (int)(v->tex_coord.Y * (ZB_POINT_ST_MAX - ZB_POINT_ST_MIN) + ZB_POINT_ST_MIN);
+		v->zp.s = (int)(v->tex_coord.X * ZB_POINT_ST_MAX);
+		v->zp.t = (int)(v->tex_coord.Y * ZB_POINT_ST_MAX);
 	}
 }
 
@@ -103,9 +91,16 @@ void gl_draw_point(GLContext *c, GLVertex *p0) {
 
 // line
 
-static inline void interpolate(GLVertex *q, GLVertex *p0, GLVertex *p1, float t) {
+static inline void interpolate_color(GLContext *c, GLVertex *q, GLVertex *p0, GLVertex *p1, float t) {
+	if (c->current_shade_model == TGL_SMOOTH)
+		q->color = p0->color + (p1->color - p0->color) * t;
+	else
+		q->color = p0->color;
+}
+
+static inline void interpolate(GLContext *c, GLVertex *q, GLVertex *p0, GLVertex *p1, float t) {
 	q->pc = p0->pc + (p1->pc - p0->pc) * t;
-	q->color = p0->color + (p1->color - p0->color) * t;
+	interpolate_color(c, q, p0, p1, t);
 }
 
 // Line Clipping
@@ -171,8 +166,8 @@ void gl_draw_line(GLContext *c, GLVertex *p1, GLVertex *p2) {
 				ClipLine1(-dy + dw, y1 - w1, &tmin, &tmax) &&
 				ClipLine1(dz + dw, -z1 - w1, &tmin, &tmax) &&
 				ClipLine1(-dz + dw, z1 - w1, &tmin, &tmax)) {
-			interpolate(&q1, p1, p2, tmin);
-			interpolate(&q2, p1, p2, tmax);
+			interpolate(c, &q1, p1, p2, tmin);
+			interpolate(c, &q2, p1, p2, tmax);
 			gl_transform_to_viewport(c, &q1);
 			gl_transform_to_viewport(c, &q2);
 
@@ -225,15 +220,7 @@ float(*clip_proc[6])(Vector4 *, Vector4 *, Vector4 *) =  {
 };
 
 static inline void updateTmp(GLContext *c, GLVertex *q, GLVertex *p0, GLVertex *p1, float t) {
-	if (c->current_shade_model == TGL_SMOOTH) {
-		float a = q->color.W;
-		q->color = p0->color + (p1->color - p0->color) * t;
-		q->color.W = a;
-	} else {
-		q->color.X = (p0->color.X);
-		q->color.Y = (p0->color.Y);
-		q->color.Z = (p0->color.Z);
-	}
+	interpolate_color(c, q, p0, p1, t);
 
 	if (c->texture_2d_enabled) {
 		// NOTE: This could be implemented with operator overloading,
