@@ -27,6 +27,8 @@
  */
 
 #include "graphics/tinygl/zgl.h"
+#include "graphics/tinygl/zblit.h"
+#include "graphics/tinygl/zdirtyrect.h"
 
 namespace TinyGL {
 
@@ -70,6 +72,7 @@ void glInit(void *zbuffer1, int textureSize) {
 
 	c->fb->_textureSize = c->_textureSize = textureSize;
 	c->fb->_textureSizeMask = (textureSize - 1) << ZB_POINT_ST_FRAC_BITS;
+	c->renderRect = Common::Rect(0, 0, zbuffer->xsize, zbuffer->ysize);
 
 	// allocate GLVertex array
 	c->vertex_max = POLYGON_MAX_VERTEX;
@@ -98,18 +101,28 @@ void glInit(void *zbuffer1, int textureSize) {
 	for (int i = 0; i < T_MAX_LIGHTS; i++) {
 		GLLight *l = &c->lights[i];
 		l->ambient = Vector4(0, 0, 0, 1);
-		l->diffuse = Vector4(1, 1, 1, 1);
-		l->specular = Vector4(1, 1, 1, 1);
+		if (i == 0) {
+			l->diffuse = Vector4(1, 1, 1, 1);
+			l->specular = Vector4(1, 1, 1, 1);
+			l->has_specular = true;
+		} else {
+			l->diffuse = Vector4(0, 0, 0, 1);
+			l->specular = Vector4(0, 0, 0, 1);
+			l->has_specular = false;
+		}
 		l->position = Vector4(0, 0, 1, 0);
-		l->norm_position = Vector3(0, 0, 1);
 		l->spot_direction = Vector3(0, 0, -1);
-		l->norm_spot_direction = Vector3(0, 0, -1);
 		l->spot_exponent = 0;
 		l->spot_cutoff = 180;
 		l->attenuation[0] = 1;
 		l->attenuation[1] = 0;
 		l->attenuation[2] = 0;
+		l->cos_spot_cutoff = -1.0f;
+		l->norm_spot_direction = Vector3(0, 0, -1);
+		l->norm_position = Vector3(0, 0, 1);
 		l->enabled = 0;
+		l->next = NULL;
+		l->prev = NULL;
 	}
 	c->first_light = NULL;
 	c->ambient_light_model = Vector4(0.2f, 0.2f, 0.2f, 1);
@@ -124,6 +137,7 @@ void glInit(void *zbuffer1, int textureSize) {
 		m->ambient = Vector4(0.2f, 0.2f, 0.2f, 1);
 		m->diffuse = Vector4(0.8f, 0.8f, 0.8f, 1);
 		m->specular = Vector4(0, 0, 0, 1);
+		m->has_specular = false;
 		m->shininess = 0;
 	}
 	c->current_color_material_mode = TGL_FRONT_AND_BACK;
@@ -135,10 +149,6 @@ void glInit(void *zbuffer1, int textureSize) {
 
 	// default state
 	c->current_color = Vector4(1.0f, 1.0f, 1.0f, 1.0f);
-	c->longcurrent_color[0] = 65535;
-	c->longcurrent_color[1] = 65535;
-	c->longcurrent_color[2] = 65535;
-	c->longcurrent_color[3] = 65535;
 
 	c->current_normal = Vector4(1.0f, 0.0f, 0.0f, 0.0f);
 
@@ -156,7 +166,7 @@ void glInit(void *zbuffer1, int textureSize) {
 
 	// clear
 	c->clear_color = Vector4(0.0f, 0.0f, 0.0f, 0.0f);
-	c->clear_depth = 0;
+	c->clear_depth = 1.0f;
 
 	// selection
 	c->render_mode = TGL_RENDER;
@@ -217,11 +227,23 @@ void glInit(void *zbuffer1, int textureSize) {
 	c->depth_test = 0;
 
 	c->color_mask = (1 << 24) | (1 << 16) | (1 << 8) | (1 << 0);
+
+	const int kDrawCallMemory = 5 * 1024 * 1024;
+
+	c->_currentAllocatorIndex = 0;
+	c->_drawCallAllocator[0].initialize(kDrawCallMemory);
+	c->_drawCallAllocator[1].initialize(kDrawCallMemory);
+	c->_enableDirtyRectangles = true;
+
+	Graphics::Internal::tglBlitResetScissorRect();
 }
 
 void glClose() {
 	GLContext *c = gl_get_context();
 
+	tglDisposeDrawCallLists(c);
+	tglDisposeResources(c);
+
 	specbuf_cleanup(c);
 	for (int i = 0; i < 3; i++)
 		gl_free(c->matrix_stack[i]);
