@@ -41,22 +41,20 @@ Common::Rect rotateRectangle(int x, int y, int width, int height, int rotation,
 
 struct BlitImage {
 public:
-	BlitImage() : _isDisposed(false), _version(0), _binaryTransparent(false) { }
+	BlitImage() : _isDisposed(false), _version(0), _binaryTransparent(false), _refcount(1) { }
 
 	void loadData(const Graphics::Surface &surface, uint32 colorKey, bool applyColorKey) {
 		const Graphics::PixelFormat textureFormat(4, 8, 8, 8, 8, 0, 8, 16, 24);
+		int size = surface.w * surface.h;
 		_surface.create(surface.w, surface.h, textureFormat);
 		Graphics::PixelBuffer buffer(surface.format, (byte *)const_cast<void *>(surface.getPixels()));
 		Graphics::PixelBuffer dataBuffer(textureFormat, (byte *)const_cast<void *>(_surface.getPixels()));
-		dataBuffer.copyBuffer(0, 0, surface.w * surface.h, buffer);
+		dataBuffer.copyBuffer(0, 0, size, buffer);
 		if (applyColorKey) {
-			for (int x = 0;  x < surface.w; x++) {
-				for (int y = 0; y < surface.h; y++) {
-					uint32 pixel = dataBuffer.getValueAt(y * surface.w + x);
-					if (pixel == colorKey) {
-						// Color keyed pixels become transparent white.
-						dataBuffer.setPixelAt(y * surface.w + x, 0, 255, 255, 255); 
-					}
+			for (int x = 0; x < size; x++) {
+				if (buffer.getValueAt(x) == colorKey) {
+					// Color keyed pixels become transparent white.
+					dataBuffer.setPixelAt(x, 0, 255, 255, 255);
 				}
 			}
 		}
@@ -77,7 +75,7 @@ public:
 					_binaryTransparent = false;
 				}
 				if (a == 0 && start >= 0) {
-					_lines.push_back(Line(start, y, x - start, srcBuf.getRawBuffer(start)));
+					_lines.push_back(Line(start, y, x - start, srcBuf.getRawBuffer(start), textureFormat));
 					start = -1;
 				} else if (a != 0 && start == -1) {
 					start = x;
@@ -85,7 +83,7 @@ public:
 			}
 			// end of the bitmap line. if start is an actual pixel save the line.
 			if (start >= 0) {
-				_lines.push_back(Line(start, y, surface.w - start, srcBuf.getRawBuffer(start)));
+				_lines.push_back(Line(start, y, surface.w - start, srcBuf.getRawBuffer(start), textureFormat));
 			}
 			srcBuf.shiftBy(surface.w);
 		}
@@ -109,16 +107,15 @@ public:
 		Graphics::PixelBuffer _buf; // This is needed for the conversion.
 
 		Line() : _x(0), _y(0), _length(0), _pixels(nullptr) { }
-		Line(int x, int y, int length, byte *pixels) : _buf(TinyGL::gl_get_context()->fb->cmode, length * TinyGL::gl_get_context()->fb->cmode.bytesPerPixel, DisposeAfterUse::NO),
+		Line(int x, int y, int length, byte *pixels, const Graphics::PixelFormat &textureFormat) : _buf(TinyGL::gl_get_context()->fb->cmode, length, DisposeAfterUse::NO),
 					_x(x), _y(y), _length(length) {
 			// Performing texture to screen conversion.
-			const Graphics::PixelFormat textureFormat(4, 8, 8, 8, 8, 0, 8, 16, 24);
 			Graphics::PixelBuffer srcBuf(textureFormat, pixels);
 			_buf.copyBuffer(0, 0, length, srcBuf);
 			_pixels = _buf.getRawBuffer();
 		}
 
-		Line(const Line& other) : _buf(TinyGL::gl_get_context()->fb->cmode, other._length * TinyGL::gl_get_context()->fb->cmode.bytesPerPixel, DisposeAfterUse::NO),
+		Line(const Line& other) : _buf(other._buf.getFormat(), other._length, DisposeAfterUse::NO),
 					_x(other._x), _y(other._y), _length(other._length){
 			_buf.copyBuffer(0, 0, _length, other._buf);
 			_pixels = _buf.getRawBuffer();
@@ -163,13 +160,13 @@ public:
 			return false;
 		}
 
-		if (dstX + width > c->_scissorRect.right) {
+		if (dstX + width >= c->_scissorRect.right) {
 			clampWidth = c->_scissorRect.right - dstX;
 		} else {
 			clampWidth = width;
 		}
 
-		if (dstY + height > c->_scissorRect.bottom) {
+		if (dstY + height >= c->_scissorRect.bottom) {
 			clampHeight = c->_scissorRect.bottom - dstY;
 		} else {
 			clampHeight = height;
@@ -249,7 +246,8 @@ public:
 
 	int getWidth() const { return _surface.w; }
 	int getHeight() const { return _surface.h; }
-	void dispose() { _isDisposed = true; }
+	void incRefCount() { _refcount++; }
+	void dispose() { if (--_refcount == 0) _isDisposed = true; }
 	bool isDisposed() const { return _isDisposed; }
 private:
 	bool _isDisposed;
@@ -257,6 +255,7 @@ private:
 	Common::Array<Line> _lines;
 	Graphics::Surface _surface;
 	int _version;
+	int _refcount;
 };
 
 void tglGetBlitImageSize(BlitImage *blitImage, int &width, int &height) {
@@ -264,6 +263,10 @@ void tglGetBlitImageSize(BlitImage *blitImage, int &width, int &height) {
 	height = blitImage->getHeight();
 }
 
+void tglIncBlitImageRef(BlitImage *blitImage) {
+	blitImage->incRefCount();
+}
+
 int tglGetBlitImageVersion(BlitImage *blitImage) {
 	return blitImage->getVersion();
 }
@@ -721,16 +724,13 @@ void tglCleanupImages() {
 	}
 }
 
-void tglBlitSetScissorRect(int left, int top, int right, int bottom) {
+void tglBlitSetScissorRect(const Common::Rect &rect) {
+	TinyGL::gl_get_context()->_scissorRect = rect;
+}
+
+void tglBlitResetScissorRect(void) {
 	TinyGL::GLContext *c = TinyGL::gl_get_context();
-	c->_scissorRect.left = left;
-	c->_scissorRect.right = right;
-	c->_scissorRect.top = top;
-	c->_scissorRect.bottom = bottom;
-	if (c->_scissorRect.right == 0 || c->_scissorRect.bottom == 0) {
-		c->_scissorRect.right = c->fb->xsize;
-		c->_scissorRect.bottom = c->fb->ysize;
-	}
+	c->_scissorRect = c->renderRect;
 }
 
 } // end of namespace Internal
