@@ -43,19 +43,29 @@ namespace TinyGL {
 
 #define ZB_POINT_ST_FRAC_BITS 14
 #define ZB_POINT_ST_FRAC_SHIFT     (ZB_POINT_ST_FRAC_BITS - 1)
-#define ZB_POINT_ST_MIN            ( (1 << ZB_POINT_ST_FRAC_SHIFT) )
-#define ZB_POINT_ST_MAX            ( (c->_textureSize << ZB_POINT_ST_FRAC_BITS) - (1 << ZB_POINT_ST_FRAC_SHIFT) )
+#define ZB_POINT_ST_MAX            ( (c->_textureSize << ZB_POINT_ST_FRAC_BITS) - 1 )
 
-#define ZB_POINT_RED_MIN ( (1 << 10) )
-#define ZB_POINT_RED_MAX ( (1 << 16) - (1 << 10) )
-#define ZB_POINT_GREEN_MIN ( (1 << 9) )
-#define ZB_POINT_GREEN_MAX ( (1 << 16) - (1 << 9) )
-#define ZB_POINT_BLUE_MIN ( (1 << 10) )
-#define ZB_POINT_BLUE_MAX ( (1 << 16) - (1 << 10) )
-#define ZB_POINT_ALPHA_MIN ( (1 << 10) )
-#define ZB_POINT_ALPHA_MAX ( (1 << 16) - (1 << 10) )
+#define ZB_POINT_RED_BITS         16
+#define ZB_POINT_RED_FRAC_BITS    8
+#define ZB_POINT_RED_FRAC_SHIFT   (ZB_POINT_RED_FRAC_BITS - 1)
+#define ZB_POINT_RED_MAX          ( (1 << ZB_POINT_RED_BITS) - 1 )
 
-#define RGB_TO_PIXEL(r, g, b) cmode.ARGBToColor(255, r >> 8, g >> 8, b >> 8) // Default to 255 alpha aka solid colour.
+#define ZB_POINT_GREEN_BITS       16
+#define ZB_POINT_GREEN_FRAC_BITS  8
+#define ZB_POINT_GREEN_FRAC_SHIFT (ZB_POINT_GREEN_FRAC_BITS - 1)
+#define ZB_POINT_GREEN_MAX        ( (1 << ZB_POINT_GREEN_BITS) - 1 )
+
+#define ZB_POINT_BLUE_BITS        16
+#define ZB_POINT_BLUE_FRAC_BITS   8
+#define ZB_POINT_BLUE_FRAC_SHIFT  (ZB_POINT_BLUE_FRAC_BITS - 1)
+#define ZB_POINT_BLUE_MAX         ( (1 << ZB_POINT_BLUE_BITS) - 1 )
+
+#define ZB_POINT_ALPHA_BITS       16
+#define ZB_POINT_ALPHA_FRAC_BITS  8
+#define ZB_POINT_ALPHA_FRAC_SHIFT (ZB_POINT_ALPHA_FRAC_BITS - 1)
+#define ZB_POINT_ALPHA_MAX        ( (1 << ZB_POINT_ALPHA_BITS) - 1 )
+
+#define RGB_TO_PIXEL(r, g, b) cmode.ARGBToColor(255, r, g, b) // Default to 255 alpha aka solid colour.
 
 static const int DRAW_DEPTH_ONLY = 0;
 static const int DRAW_FLAT = 1;
@@ -63,8 +73,6 @@ static const int DRAW_SMOOTH = 2;
 static const int DRAW_SHADOW_MASK = 3;
 static const int DRAW_SHADOW = 4;
 
-extern uint8 PSZB;
-
 struct Buffer {
 	byte *pbuf;
 	unsigned int *zbuf;
@@ -188,40 +196,80 @@ struct FrameBuffer {
 
 	template <bool kEnableAlphaTest, bool kBlendingEnabled>
 	FORCEINLINE void writePixel(int pixel, int value) {
-		byte rSrc, gSrc, bSrc, aSrc;
-		this->pbuf.getFormat().colorToARGB(value, aSrc, rSrc, gSrc, bSrc);
+		writePixel<kEnableAlphaTest, kBlendingEnabled, false>(pixel, value, 0);
+	}
 
+	template <bool kEnableAlphaTest, bool kBlendingEnabled, bool kDepthWrite>
+	FORCEINLINE void writePixel(int pixel, int value, unsigned int z) {
 		if (kBlendingEnabled == false) {
 			this->pbuf.setPixelAt(pixel, value);
+			if (kDepthWrite) {
+				_zbuf[pixel] = z;
+			}
 		} else {
-			writePixel<kEnableAlphaTest, kBlendingEnabled>(pixel, aSrc, rSrc, gSrc, bSrc);
+			byte rSrc, gSrc, bSrc, aSrc;
+			this->pbuf.getFormat().colorToARGB(value, aSrc, rSrc, gSrc, bSrc);
+
+			writePixel<kEnableAlphaTest, kBlendingEnabled, kDepthWrite>(pixel, aSrc, rSrc, gSrc, bSrc, z);
 		}
 	}
 
 	FORCEINLINE void writePixel(int pixel, int value) {
-		writePixel<true, true>(pixel, value);
+		if (_alphaTestEnabled) {
+			writePixel<true>(pixel, value);
+		} else {
+			writePixel<false>(pixel, value);
+		}
+	}
+
+	template <bool kEnableAlphaTest>
+	FORCEINLINE void writePixel(int pixel, int value) {
+		if (_blendingEnabled) {
+			writePixel<kEnableAlphaTest, true>(pixel, value);
+		} else {
+			writePixel<kEnableAlphaTest, false>(pixel, value);
+		}
 	}
 
 	FORCEINLINE void writePixel(int pixel, byte rSrc, byte gSrc, byte bSrc) {
 		writePixel(pixel, 255, rSrc, gSrc, bSrc);
 	}
 
-	FORCEINLINE bool scissorPixel(int pixel) {
-		int x = pixel % xsize;
-		int y = pixel / xsize;
-		return x < _clipRectangle.left || x > _clipRectangle.right || y < _clipRectangle.top || y > _clipRectangle.bottom;
+	FORCEINLINE bool scissorPixel(int x, int y) {
+		return !_clipRectangle.contains(x, y);
 	}
 
 	FORCEINLINE void writePixel(int pixel, byte aSrc, byte rSrc, byte gSrc, byte bSrc) {
-		writePixel<true, true>(pixel, aSrc, rSrc, gSrc, bSrc);
+		if (_alphaTestEnabled) {
+			writePixel<true>(pixel, aSrc, rSrc, gSrc, bSrc);
+		} else {
+			writePixel<false>(pixel, aSrc, rSrc, gSrc, bSrc);
+		}
+	}
+
+	template <bool kEnableAlphaTest>
+	FORCEINLINE void writePixel(int pixel, byte aSrc, byte rSrc, byte gSrc, byte bSrc) {
+		if (_blendingEnabled) {
+			writePixel<kEnableAlphaTest, true>(pixel, aSrc, rSrc, gSrc, bSrc);
+		} else {
+			writePixel<kEnableAlphaTest, false>(pixel, aSrc, rSrc, gSrc, bSrc);
+		}
 	}
 
 	template <bool kEnableAlphaTest, bool kBlendingEnabled>
 	FORCEINLINE void writePixel(int pixel, byte aSrc, byte rSrc, byte gSrc, byte bSrc) {
+		writePixel<kEnableAlphaTest, kBlendingEnabled, false>(pixel, aSrc, rSrc, gSrc, bSrc, 0);
+	}
+
+	template <bool kEnableAlphaTest, bool kBlendingEnabled, bool kDepthWrite>
+	FORCEINLINE void writePixel(int pixel, byte aSrc, byte rSrc, byte gSrc, byte bSrc, unsigned int z) {
 		if (kEnableAlphaTest) {
 			if (!checkAlphaTest(aSrc))
 				return;
 		}
+		if (kDepthWrite) {
+			_zbuf[pixel] = z;
+		}
 		
 		if (kBlendingEnabled == false) {
 			this->pbuf.setPixelAt(pixel, aSrc, rSrc, gSrc, bSrc);
@@ -377,10 +425,7 @@ struct FrameBuffer {
 	void clearOffscreenBuffer(Buffer *buffer);
 	void setTexture(const Graphics::PixelBuffer &texture);
 
-	template <bool kInterpRGB, bool kInterpZ, bool kInterpST, bool kInterpSTZ, int kDrawLogic, bool kDepthWrite, bool enableAlphaTest, bool kEnableScissor, bool kBlendingEnabled, bool kRGB565Target>
-	void fillTriangle(ZBufferPoint *p0, ZBufferPoint *p1, ZBufferPoint *p2);
-
-	template <bool kInterpRGB, bool kInterpZ, bool kInterpST, bool kInterpSTZ, int kDrawLogic, bool kDepthWrite, bool enableAlphaTest, bool kEnableScissor, bool kBlendingEnabled>
+	template <bool kInterpRGB, bool kInterpZ, bool kInterpST, bool kInterpSTZ, int kDrawLogic, bool kDepthWrite, bool enableAlphaTest, bool kEnableScissor, bool enableBlending>
 	void fillTriangle(ZBufferPoint *p0, ZBufferPoint *p1, ZBufferPoint *p2);
 
 	template <bool kInterpRGB, bool kInterpZ, bool kInterpST, bool kInterpSTZ, int kDrawMode, bool kDepthWrite, bool enableAlphaTest, bool kEnableScissor>
@@ -395,9 +440,6 @@ struct FrameBuffer {
 	template <bool kInterpRGB, bool kInterpZ, bool kInterpST, bool kInterpSTZ, int kDrawMode>
 	void fillTriangle(ZBufferPoint *p0, ZBufferPoint *p1, ZBufferPoint *p2);
 
-	template <bool kInterpRGB, bool kInterpZ, bool kDepthWrite>
-	void fillLineGeneric(ZBufferPoint *p1, ZBufferPoint *p2, int color);
-
 	void fillTriangleTextureMappingPerspectiveSmooth(ZBufferPoint *p0, ZBufferPoint *p1, ZBufferPoint *p2);
 	void fillTriangleTextureMappingPerspectiveFlat(ZBufferPoint *p0, ZBufferPoint *p1, ZBufferPoint *p2);
 	void fillTriangleDepthOnly(ZBufferPoint *p0, ZBufferPoint *p1, ZBufferPoint *p2);
@@ -409,23 +451,24 @@ struct FrameBuffer {
 	void plot(ZBufferPoint *p);
 	void fillLine(ZBufferPoint *p1, ZBufferPoint *p2);
 	void fillLineZ(ZBufferPoint *p1, ZBufferPoint *p2);
-	void fillLineFlatZ(ZBufferPoint *p1, ZBufferPoint *p2, int color);
+	void fillLineFlatZ(ZBufferPoint *p1, ZBufferPoint *p2);
 	void fillLineInterpZ(ZBufferPoint *p1, ZBufferPoint *p2);
-	void fillLineFlat(ZBufferPoint *p1, ZBufferPoint *p2, int color);
+	void fillLineFlat(ZBufferPoint *p1, ZBufferPoint *p2);
 	void fillLineInterp(ZBufferPoint *p1, ZBufferPoint *p2);
 
-	void setScissorRectangle(int left, int right, int top, int bottom) {
-		_clipRectangle.left = left;
-		_clipRectangle.right = right;
-		_clipRectangle.top = top;
-		_clipRectangle.bottom = bottom;
+	void setScissorRectangle(const Common::Rect &rect) {
+		_clipRectangle = rect;
+		_enableScissor = true;
+	}
+	void resetScissorRectangle() {
+		_enableScissor = false;
 	}
 
 	Common::Rect _clipRectangle;
+	bool _enableScissor;
 	int xsize, ysize;
 	int linesize; // line size, in bytes
 	Graphics::PixelFormat cmode;
-	int pixelbits;
 	int pixelbytes;
 
 	Buffer buffer;
@@ -454,6 +497,21 @@ struct FrameBuffer {
 
 private:
 
+	template <bool kDepthWrite>
+	FORCEINLINE void putPixel(unsigned int pixelOffset, int color, int x, int y, unsigned int z);
+
+	template <bool kDepthWrite, bool kEnableScissor>
+	FORCEINLINE void putPixel(unsigned int pixelOffset, int color, int x, int y, unsigned int z);
+
+	template <bool kEnableScissor>
+	FORCEINLINE void putPixel(unsigned int pixelOffset, int color, int x, int y);
+
+	template <bool kInterpRGB, bool kInterpZ, bool kDepthWrite>
+	void drawLine(const ZBufferPoint *p1, const ZBufferPoint *p2);
+
+	template <bool kInterpRGB, bool kInterpZ, bool kDepthWrite, bool kEnableScissor>
+	void drawLine(const ZBufferPoint *p1, const ZBufferPoint *p2);
+
 	unsigned int *_zbuf;
 	bool _depthWrite;
 	Graphics::PixelBuffer pbuf;
