@@ -31,10 +31,14 @@
 
 #include "common/util.h"
 #include "common/textconsole.h"
+#include "common/array.h"
+#include "common/list.h"
 
 #include "graphics/tinygl/gl.h"
 #include "graphics/tinygl/zbuffer.h"
 #include "graphics/tinygl/zmath.h"
+#include "graphics/tinygl/zblit.h"
+#include "graphics/tinygl/zdirtyrect.h"
 
 namespace TinyGL {
 
@@ -84,6 +88,7 @@ struct GLLight {
 	Vector4 ambient;
 	Vector4 diffuse;
 	Vector4 specular;
+	bool has_specular;
 	Vector4 position;
 	Vector3 spot_direction;
 	float spot_exponent;
@@ -103,6 +108,7 @@ struct GLMaterial {
 	Vector4 ambient;
 	Vector4 diffuse;
 	Vector4 specular;
+	bool has_specular;
 	float shininess;
 
 	// computed values
@@ -177,7 +183,7 @@ struct GLImage {
 
 struct GLTexture {
 	GLImage images[MAX_TEXTURE_LEVELS];
-	int handle;
+	unsigned int handle;
 	int versionNumber;
 	struct GLTexture *next, *prev;
 	bool disposed;
@@ -250,6 +256,7 @@ typedef void (*gl_draw_triangle_func)(GLContext *c, GLVertex *p0, GLVertex *p1,
 struct GLContext {
 	// Z buffer
 	FrameBuffer *fb;
+	Common::Rect renderRect;
 
 	// Internal texture size
 	int _textureSize;
@@ -324,7 +331,6 @@ struct GLContext {
 
 	// current vertex state
 	Vector4 current_color;
-	unsigned int longcurrent_color[4]; // precomputed integer color
 	Vector4 current_normal;
 	Vector4 current_tex_coord;
 	int current_edge_flag;
@@ -373,6 +379,17 @@ struct GLContext {
 	int color_mask;
 
 	Common::Rect _scissorRect;
+
+	bool _enableDirtyRectangles;
+
+	// blit test
+	Common::List<Graphics::BlitImage *> _blitImages;
+
+	// Draw call queue
+	Common::List<Graphics::DrawCall *> _drawCallsQueue;
+	Common::List<Graphics::DrawCall *> _previousFrameDrawCallsQueue;
+	int _currentAllocatorIndex;
+	LinearAllocator _drawCallAllocator[2];
 };
 
 extern GLContext *gl_ctx;
@@ -402,6 +419,7 @@ void glInitTextures(GLContext *c);
 void glEndTextures(GLContext *c);
 GLTexture *alloc_texture(GLContext *c, int h);
 void free_texture(GLContext *c, int h);
+void free_texture(GLContext *c, GLTexture *t);
 
 // image_util.c
 void gl_resizeImage(unsigned char *dest, int xsize_dest, int ysize_dest,
@@ -409,6 +427,12 @@ void gl_resizeImage(unsigned char *dest, int xsize_dest, int ysize_dest,
 void gl_resizeImageNoInterpolate(unsigned char *dest, int xsize_dest, int ysize_dest,
 								 unsigned char *src, int xsize_src, int ysize_src);
 
+void tglIssueDrawCall(Graphics::DrawCall *drawCall);
+
+// zdirtyrect.cpp
+void tglDisposeResources(GLContext *c);
+void tglDisposeDrawCallLists(TinyGL::GLContext *c);
+
 GLContext *gl_get_context();
 
 // specular buffer "api"
