@@ -34,46 +34,32 @@ namespace TinyGL {
 
 static const int NB_INTERP = 8;
 
-#define SAR_RND_TO_ZERO(v,n) (v / (1 << n))
-
 template <bool kDepthWrite, bool kEnableAlphaTest, bool kEnableScissor, bool kEnableBlending>
 FORCEINLINE static void putPixelFlat(FrameBuffer *buffer, int buf, unsigned int *pz, int _a,
-                                     unsigned int &z, int color, unsigned int &a, int &dzdx) {
-	if ((!kEnableScissor || !buffer->scissorPixel(buf + _a)) && buffer->compareDepth(z, pz[_a])) {
-		unsigned int r = (color & 0xF800) >> 8;
-		unsigned int g = (color & 0x07E0) >> 3;
-		unsigned int b = (color & 0x001F) << 3;
-		buffer->writePixel<kEnableAlphaTest, kEnableBlending>(buf + _a, a / 256, r, g, b);
-		if (kDepthWrite) {
-			pz[_a] = z;
-		}
+                                     int x, int y, unsigned int &z, unsigned int &r, unsigned int &g, unsigned int &b, unsigned int &a, int &dzdx) {
+	if ((!kEnableScissor || !buffer->scissorPixel(x + _a, y)) && buffer->compareDepth(z, pz[_a])) {
+		buffer->writePixel<kEnableAlphaTest, kEnableBlending, kDepthWrite>(buf + _a, a >> (ZB_POINT_ALPHA_BITS - 8), r >> (ZB_POINT_RED_BITS - 8), g >> (ZB_POINT_GREEN_BITS - 8), b >> (ZB_POINT_BLUE_BITS - 8), z);
 	}
 	z += dzdx;
 }
 
 template <bool kDepthWrite, bool kEnableAlphaTest, bool kEnableScissor, bool kEnableBlending>
 FORCEINLINE static void putPixelSmooth(FrameBuffer *buffer, int buf, unsigned int *pz, int _a,
-                                       unsigned int &z, int &tmp, unsigned int &rgb, unsigned int &a,
-                                       int &dzdx, unsigned int &drgbdx, unsigned int dadx) {
-	if ((!kEnableScissor || !buffer->scissorPixel(buf + _a)) && buffer->compareDepth(z, pz[_a])) {
-		tmp = rgb & 0xF81F07E0;
-		unsigned int color = tmp | (tmp >> 16);
-		unsigned int r = (color & 0xF800) >> 8;
-		unsigned int g = (color & 0x07E0) >> 3;
-		unsigned int b = (color & 0x001F) << 3;
-		buffer->writePixel<kEnableAlphaTest, kEnableBlending>(buf + _a, a / 256, r, g, b);
-		if (kDepthWrite) {
-			pz[_a] = z;
-		}
+                                       int x, int y, unsigned int &z, unsigned int &r, unsigned int &g, unsigned int &b, unsigned int &a,
+                                       int &dzdx, int &drdx, int &dgdx, int &dbdx, unsigned int dadx) {
+	if ((!kEnableScissor || !buffer->scissorPixel(x + _a, y)) && buffer->compareDepth(z, pz[_a])) {
+		buffer->writePixel<kEnableAlphaTest, kEnableBlending, kDepthWrite>(buf + _a, a >> (ZB_POINT_ALPHA_BITS - 8), r >> (ZB_POINT_RED_BITS - 8), g >> (ZB_POINT_GREEN_BITS - 8), b >> (ZB_POINT_BLUE_BITS - 8), z);
 	}
 	z += dzdx;
 	a += dadx;
-	rgb = (rgb + drgbdx) & (~0x00200800);
+	r += drdx;
+	g += dgdx;
+	b += dbdx;
 }
 
 template <bool kDepthWrite, bool kEnableScissor>
-FORCEINLINE static void putPixelDepth(FrameBuffer *buffer, int buf, unsigned int *pz, int _a, unsigned int &z, int &dzdx) {
-	if ((!kEnableScissor || !buffer->scissorPixel(buf + _a)) && buffer->compareDepth(z, pz[_a])) {
+FORCEINLINE static void putPixelDepth(FrameBuffer *buffer, int buf, unsigned int *pz, int _a, int x, int y, unsigned int &z, int &dzdx) {
+	if ((!kEnableScissor || !buffer->scissorPixel(x + _a, y)) && buffer->compareDepth(z, pz[_a])) {
 		if (kDepthWrite) {
 			pz[_a] = z;
 		}
@@ -81,12 +67,20 @@ FORCEINLINE static void putPixelDepth(FrameBuffer *buffer, int buf, unsigned int
 	z += dzdx;
 }
 
-template <bool kDepthWrite, bool kLightsMode, bool kSmoothMode, bool kEnableAlphaTest, bool kEnableScissor, bool kEnableBlending, bool kRGB565Target>
+template <bool kDepthWrite, bool kAlphaTestEnabled, bool kEnableScissor, bool kBlendingEnabled>
+FORCEINLINE static void putPixelShadow(FrameBuffer *buffer, int buf, unsigned int *pz, int _a, int x, int y, unsigned int &z, unsigned int &r, unsigned int &g, unsigned int &b, int &dzdx, unsigned char *pm) {
+	if ((!kEnableScissor || !buffer->scissorPixel(x + _a, y)) && buffer->compareDepth(z, pz[_a]) && pm[_a]) {
+		buffer->writePixel<kAlphaTestEnabled, kBlendingEnabled, kDepthWrite>(buf + _a, 255, r >> (ZB_POINT_RED_BITS - 8), g >> (ZB_POINT_GREEN_BITS - 8), b >> (ZB_POINT_BLUE_BITS - 8), z);
+	}
+	z += dzdx;
+}
+
+template <bool kDepthWrite, bool kLightsMode, bool kSmoothMode, bool kEnableAlphaTest, bool kEnableScissor, bool kEnableBlending>
 FORCEINLINE static void putPixelTextureMappingPerspective(FrameBuffer *buffer, int buf,
                         Graphics::PixelFormat &textureFormat, Graphics::PixelBuffer &texture, unsigned int *pz, int _a,
-                        unsigned int &z, unsigned int &t, unsigned int &s, int &tmp, unsigned int &rgba, unsigned int &a,
-                        int &dzdx, int &dsdx, int &dtdx, unsigned int &drgbdx, unsigned int dadx) {
-	if ((!kEnableScissor || !buffer->scissorPixel(buf + _a)) && buffer->compareDepth(z, pz[_a])) {
+                        int x, int y, unsigned int &z, unsigned int &t, unsigned int &s, unsigned int &r, unsigned int &g, unsigned int &b, unsigned int &a,
+                        int &dzdx, int &dsdx, int &dtdx, int &drdx, int &dgdx, int &dbdx, unsigned int dadx) {
+	if ((!kEnableScissor || !buffer->scissorPixel(x + _a, y)) && buffer->compareDepth(z, pz[_a])) {
 		unsigned sss = (s & buffer->_textureSizeMask) >> ZB_POINT_ST_FRAC_BITS;
 		unsigned ttt = (t & buffer->_textureSizeMask) >> ZB_POINT_ST_FRAC_BITS;
 		int pixel = ttt * buffer->_textureSize + sss;
@@ -94,52 +88,45 @@ FORCEINLINE static void putPixelTextureMappingPerspective(FrameBuffer *buffer, i
 		uint32 *textureBuffer = (uint32 *)texture.getRawBuffer(pixel);
 		uint32 col = *textureBuffer;
 		c_a = (col >> textureFormat.aShift) & 0xFF;
-		if (kRGB565Target && !kEnableBlending && !kEnableAlphaTest && c_a == 0) // we have RGB565 target currently so no alpha channel here, so skip pixel
-			return;
 		c_r = (col >> textureFormat.rShift) & 0xFF;
 		c_g = (col >> textureFormat.gShift) & 0xFF;
 		c_b = (col >> textureFormat.bShift) & 0xFF;
-		unsigned int l_a = (a / 256);
-		c_a = (c_a * l_a) / 256;
 		if (kLightsMode) {
-			tmp = rgba & 0xF81F07E0;
-			unsigned int light = tmp | (tmp >> 16);
-			unsigned int l_r = (light & 0xF800) >> 8;
-			unsigned int l_g = (light & 0x07E0) >> 3;
-			unsigned int l_b = (light & 0x001F) << 3;
-			c_r = (c_r * l_r) / 256;
-			c_g = (c_g * l_g) / 256;
-			c_b = (c_b * l_b) / 256;
-		}
-		buffer->writePixel<kEnableAlphaTest, kEnableBlending>(buf + _a, c_a, c_r, c_g, c_b);
-		if (kDepthWrite) {
-			pz[_a] = z;
+			unsigned int l_a = (a >> (ZB_POINT_ALPHA_BITS - 8));
+			unsigned int l_r = (r >> (ZB_POINT_RED_BITS - 8));
+			unsigned int l_g = (g >> (ZB_POINT_GREEN_BITS - 8));
+			unsigned int l_b = (b >> (ZB_POINT_BLUE_BITS - 8));
+			c_a = (c_a * l_a) >> (ZB_POINT_ALPHA_BITS - 8);
+			c_r = (c_r * l_r) >> (ZB_POINT_RED_BITS - 8);
+			c_g = (c_g * l_g) >> (ZB_POINT_GREEN_BITS - 8);
+			c_b = (c_b * l_b) >> (ZB_POINT_BLUE_BITS - 8);
 		}
+		buffer->writePixel<kEnableAlphaTest, kEnableBlending, kDepthWrite>(buf + _a, c_a, c_r, c_g, c_b, z);
 	}
 	z += dzdx;
 	s += dsdx;
 	t += dtdx;
 	if (kSmoothMode) {
 		a += dadx;
-		rgba = (rgba + drgbdx) & (~0x00200800);
+		r += drdx;
+		g += dgdx;
+		b += dbdx;
 	}
 }
 
-template <bool kInterpRGB, bool kInterpZ, bool kInterpST, bool kInterpSTZ, int kDrawLogic, bool kDepthWrite, bool kAlphaTestEnabled, bool kEnableScissor, bool kBlendingEnabled, bool kRGB565Target>
+template <bool kInterpRGB, bool kInterpZ, bool kInterpST, bool kInterpSTZ, int kDrawLogic, bool kDepthWrite, bool kAlphaTestEnabled, bool kEnableScissor, bool kBlendingEnabled>
 void FrameBuffer::fillTriangle(ZBufferPoint *p0, ZBufferPoint *p1, ZBufferPoint *p2) {
 	Graphics::PixelBuffer texture;
 	Graphics::PixelFormat textureFormat;
 	float fdzdx = 0, fndzdx = 0, ndszdx = 0, ndtzdx = 0;
-	int _drgbdx = 0;
 
 	ZBufferPoint *tp, *pr1 = 0, *pr2 = 0, *l1 = 0, *l2 = 0;
 	float fdx1, fdx2, fdy1, fdy2, fz0, d1, d2;
 	unsigned int *pz1 = NULL;
 	unsigned char *pm1 = NULL;
-	int part, update_left, update_right;
-	int color = 0;
+	int part, update_left = 1, update_right = 1;
 
-	int nb_lines, dx1, dy1, tmp, dx2, dy2;
+	int nb_lines, dx1, dy1, tmp, dx2, dy2, y;
 
 	int error = 0, derror = 0;
 	int x1 = 0, dxdy_min = 0, dxdy_max = 0;
@@ -153,9 +140,6 @@ void FrameBuffer::fillTriangle(ZBufferPoint *p0, ZBufferPoint *p1, ZBufferPoint
 	int b1 = 0, dbdx = 0, dbdy = 0, dbdl_min = 0, dbdl_max = 0;
 	int a1 = 0, dadx = 0, dady = 0, dadl_min = 0, dadl_max = 0;
 
-	int s1 = 0, dsdx = 0, dsdy = 0, dsdl_min = 0, dsdl_max = 0;
-	int t1 = 0, dtdx = 0, dtdy = 0, dtdl_min = 0, dtdl_max = 0;
-
 	float sz1 = 0.0, dszdx = 0, dszdy = 0, dszdl_min = 0.0, dszdl_max = 0.0;
 	float tz1 = 0.0, dtzdx = 0, dtzdy = 0, dtzdl_min = 0.0, dtzdl_max = 0.0;
 
@@ -223,29 +207,26 @@ void FrameBuffer::fillTriangle(ZBufferPoint *p0, ZBufferPoint *p1, ZBufferPoint
 		dady = (int)(fdx1 * d2 - fdx2 * d1);
 	}
 
-	if (kInterpST) {
-		d1 = (float)(p1->s - p0->s);
-		d2 = (float)(p2->s - p0->s);
-		dsdx = (int)(fdy2 * d1 - fdy1 * d2);
-		dsdy = (int)(fdx1 * d2 - fdx2 * d1);
-
-		d1 = (float)(p1->t - p0->t);
-		d2 = (float)(p2->t - p0->t);
-		dtdx = (int)(fdy2 * d1 - fdy1 * d2);
-		dtdy = (int)(fdx1 * d2 - fdx2 * d1);
-	}
-
-	if (kInterpSTZ) {
-		float zz;
-		zz = (float)p0->z;
-		p0->sz = (float)p0->s * zz;
-		p0->tz = (float)p0->t * zz;
-		zz = (float)p1->z;
-		p1->sz = (float)p1->s * zz;
-		p1->tz = (float)p1->t * zz;
-		zz = (float)p2->z;
-		p2->sz = (float)p2->s * zz;
-		p2->tz = (float)p2->t * zz;
+	if (kInterpST || kInterpSTZ) {
+		if (kInterpSTZ) {
+			float zz;
+			zz = (float)p0->z;
+			p0->sz = (float)p0->s * zz;
+			p0->tz = (float)p0->t * zz;
+			zz = (float)p1->z;
+			p1->sz = (float)p1->s * zz;
+			p1->tz = (float)p1->t * zz;
+			zz = (float)p2->z;
+			p2->sz = (float)p2->s * zz;
+			p2->tz = (float)p2->t * zz;
+		} else {
+			p0->sz = (float)p0->s;
+			p0->tz = (float)p0->t;
+			p1->sz = (float)p1->s;
+			p1->tz = (float)p1->t;
+			p2->sz = (float)p2->s;
+			p2->tz = (float)p2->t;
+		}
 
 		d1 = p1->sz - p0->sz;
 		d2 = p2->sz - p0->sz;
@@ -269,17 +250,19 @@ void FrameBuffer::fillTriangle(ZBufferPoint *p0, ZBufferPoint *p1, ZBufferPoint
 		break;
 	case DRAW_SHADOW:
 		pm1 = shadow_mask_buf + p0->y * xsize;
-		color = RGB_TO_PIXEL(shadow_color_r, shadow_color_g, shadow_color_b);
+		r1 = shadow_color_r;
+		g1 = shadow_color_g;
+		b1 = shadow_color_b;
 		break;
 	case DRAW_DEPTH_ONLY:
 		break;
 	case DRAW_FLAT:
-		color = RGB_TO_PIXEL(p2->r, p2->g, p2->b);
+		r1 = p2->r;
+		g1 = p2->g;
+		b1 = p2->b;
+		a1 = p2->a;
 		break;
 	case DRAW_SMOOTH:
-		_drgbdx = (SAR_RND_TO_ZERO(drdx, 6) << 22) & 0xFFC00000;
-		_drgbdx |= SAR_RND_TO_ZERO(dgdx, 5) & 0x000007FF;
-		_drgbdx |= (SAR_RND_TO_ZERO(dbdx, 7) << 12) & 0x001FF000;
 		break;
 	default:
 		break;
@@ -293,38 +276,29 @@ void FrameBuffer::fillTriangle(ZBufferPoint *p0, ZBufferPoint *p1, ZBufferPoint
 		fndzdx = NB_INTERP * fdzdx;
 		ndszdx = NB_INTERP * dszdx;
 		ndtzdx = NB_INTERP * dtzdx;
-		_drgbdx = ((drdx / (1 << 6)) << 22) & 0xFFC00000;
-		_drgbdx |= (dgdx / (1 << 5)) & 0x000007FF;
-		_drgbdx |= ((dbdx / (1 << 7)) << 12) & 0x001FF000;
 	}
 
+	if (fz0 > 0) {
+		l1 = p0;
+		l2 = p2;
+		pr1 = p0;
+		pr2 = p1;
+	} else {
+		l1 = p0;
+		l2 = p1;
+		pr1 = p0;
+		pr2 = p2;
+	}
+	nb_lines = p1->y - p0->y;
+	y = p0->y;
 	for (part = 0; part < 2; part++) {
-		if (part == 0) {
-			if (fz0 > 0) {
-				update_left = 1;
-				update_right = 1;
-				l1 = p0;
-				l2 = p2;
-				pr1 = p0;
-				pr2 = p1;
-			} else {
-				update_left = 1;
-				update_right = 1;
-				l1 = p0;
-				l2 = p1;
-				pr1 = p0;
-				pr2 = p2;
-			}
-			nb_lines = p1->y - p0->y;
-		} else {
+		if (part == 1) {
 			// second part
 			if (fz0 > 0) {
 				update_left = 0;
-				update_right = 1;
 				pr1 = p1;
 				pr2 = p2;
 			} else {
-				update_left = 1;
 				update_right = 0;
 				l1 = p1;
 				l2 = p2;
@@ -371,17 +345,7 @@ void FrameBuffer::fillTriangle(ZBufferPoint *p0, ZBufferPoint *p1, ZBufferPoint
 				dadl_max = dadl_min + dadx;
 			}
 
-			if (kInterpST) {
-				s1 = l1->s;
-				dsdl_min = (dsdy + dsdx * dxdy_min);
-				dsdl_max = dsdl_min + dsdx;
-
-				t1 = l1->t;
-				dtdl_min = (dtdy + dtdx * dxdy_min);
-				dtdl_max = dtdl_min + dtdx;
-			}
-
-			if (kInterpSTZ) {
+			if (kInterpST || kInterpSTZ) {
 				sz1 = l1->sz;
 				dszdl_min = (dszdy + dszdx * dxdy_min);
 				dszdl_max = dszdl_min + dszdx;
@@ -406,7 +370,7 @@ void FrameBuffer::fillTriangle(ZBufferPoint *p0, ZBufferPoint *p1, ZBufferPoint
 
 		// we draw all the scan line of the part
 		while (nb_lines > 0) {
-			nb_lines--;
+			int x = x1;
 			{
 				if (kDrawLogic == DRAW_DEPTH_ONLY ||
 						(kDrawLogic == DRAW_FLAT && !(kInterpST || kInterpSTZ))) {
@@ -415,46 +379,53 @@ void FrameBuffer::fillTriangle(ZBufferPoint *p0, ZBufferPoint *p1, ZBufferPoint
 					unsigned int *pz;
 					unsigned int z, a;
 					int buf = pp1 + x1;
+					unsigned int r = r1;
+					unsigned int g = g1;
+					unsigned int b = b1;
 					n = (x2 >> 16) - x1;
 					pp = pp1 + x1;
 					if (kInterpZ) {
 						pz = pz1 + x1;
 						z = z1;
 					}
-					a = a1;
+					if (kDrawLogic == DRAW_FLAT) {
+						a = a1;
+					}
 					while (n >= 3) {
 						if (kDrawLogic == DRAW_DEPTH_ONLY) {
-							putPixelDepth<kDepthWrite, kEnableScissor>(this, buf, pz, 0, z, dzdx);
-							putPixelDepth<kDepthWrite, kEnableScissor>(this, buf, pz, 1, z, dzdx);
-							putPixelDepth<kDepthWrite, kEnableScissor>(this, buf, pz, 2, z, dzdx);
-							putPixelDepth<kDepthWrite, kEnableScissor>(this, buf, pz, 3, z, dzdx);
+							putPixelDepth<kDepthWrite, kEnableScissor>(this, buf, pz, 0, x, y, z, dzdx);
+							putPixelDepth<kDepthWrite, kEnableScissor>(this, buf, pz, 1, x, y, z, dzdx);
+							putPixelDepth<kDepthWrite, kEnableScissor>(this, buf, pz, 2, x, y, z, dzdx);
+							putPixelDepth<kDepthWrite, kEnableScissor>(this, buf, pz, 3, x, y, z, dzdx);
 							buf += 4;
 						}
 						if (kDrawLogic == DRAW_FLAT) {
-							putPixelFlat<kDepthWrite, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, pp, pz, 0, z, color, a, dzdx);
-							putPixelFlat<kDepthWrite, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, pp, pz, 1, z, color, a, dzdx);
-							putPixelFlat<kDepthWrite, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, pp, pz, 2, z, color, a, dzdx);
-							putPixelFlat<kDepthWrite, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, pp, pz, 3, z, color, a, dzdx);
+							putPixelFlat<kDepthWrite, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, pp, pz, 0, x, y, z, r, g, b, a, dzdx);
+							putPixelFlat<kDepthWrite, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, pp, pz, 1, x, y, z, r, g, b, a, dzdx);
+							putPixelFlat<kDepthWrite, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, pp, pz, 2, x, y, z, r, g, g, a, dzdx);
+							putPixelFlat<kDepthWrite, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, pp, pz, 3, x, y, z, r, g, b, a, dzdx);
 						}
 						if (kInterpZ) {
 							pz += 4;
 						}
 						pp += 4;
 						n -= 4;
+						x += 4;
 					}
 					while (n >= 0) {
 						if (kDrawLogic == DRAW_DEPTH_ONLY) {
-							putPixelDepth<kDepthWrite, kEnableScissor>(this, buf, pz, 0, z, dzdx);
+							putPixelDepth<kDepthWrite, kEnableScissor>(this, buf, pz, 0, x, y, z, dzdx);
 							buf ++;
 						}
 						if (kDrawLogic == DRAW_FLAT) {
-							putPixelFlat<kDepthWrite, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, pp, pz, 0, z, color, a, dzdx);
+							putPixelFlat<kDepthWrite, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, pp, pz, 0, x, y, z, r, g, b, a, dzdx);
 						}
 						if (kInterpZ) {
 							pz += 1;
 						}
 						pp += 1;
 						n -= 1;
+						x += 1;
 					}
 				} else if (kDrawLogic == DRAW_SHADOW_MASK) {
 					unsigned char *pm;
@@ -463,22 +434,28 @@ void FrameBuffer::fillTriangle(ZBufferPoint *p0, ZBufferPoint *p1, ZBufferPoint
 					n = (x2 >> 16) - x1;
 					pm = pm1 + x1;
 					while (n >= 3) {
-						for (int a = 0; a <= 3; a++) {
-							pm[a] = 0xff;
-						}
+						pm[0] = 0xff;
+						pm[1] = 0xff;
+						pm[2] = 0xff;
+						pm[3] = 0xff;
 						pm += 4;
 						n -= 4;
+						x += 4;
 					}
 					while (n >= 0) {
 						pm[0] = 0xff;
 						pm += 1;
 						n -= 1;
+						x += 1;
 					}
 				} else if (kDrawLogic == DRAW_SHADOW) {
 					unsigned char *pm;
 					int n;
 					unsigned int *pz;
 					unsigned int z;
+					unsigned int r = r1;
+					unsigned int g = g1;
+					unsigned int b = b1;
 
 					n = (x2 >> 16) - x1;
 
@@ -488,65 +465,60 @@ void FrameBuffer::fillTriangle(ZBufferPoint *p0, ZBufferPoint *p1, ZBufferPoint
 					pz = pz1 + x1;
 					z = z1;
 					while (n >= 3) {
-						for (int a = 0; a < 4; a++) {
-							if ((!kEnableScissor || !scissorPixel(buf + a)) && compareDepth(z, pz[a]) && pm[0]) {
-								writePixel<kAlphaTestEnabled, kBlendingEnabled>(buf + a, color);
-								if (kDepthWrite) {
-									pz[a] = z;
-								}
-							}
-							z += dzdx;
-						}
+						putPixelShadow<kDepthWrite, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, buf, pz, 0, x, y, z, r, g, b, dzdx, pm);
+						putPixelShadow<kDepthWrite, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, buf, pz, 1, x, y, z, r, g, b, dzdx, pm);
+						putPixelShadow<kDepthWrite, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, buf, pz, 2, x, y, z, r, g, b, dzdx, pm);
+						putPixelShadow<kDepthWrite, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, buf, pz, 3, x, y, z, r, g, b, dzdx, pm);
 						pz += 4;
 						pm += 4;
 						buf += 4;
 						n -= 4;
+						x += 4;
 					}
 					while (n >= 0) {
-						if ((!kEnableScissor || !scissorPixel(buf)) && compareDepth(z, pz[0]) && pm[0]) {
-							writePixel<kAlphaTestEnabled, kBlendingEnabled>(buf, color);
-							if (kDepthWrite) {
-								pz[0] = z;
-							}
-						}
+						putPixelShadow<kDepthWrite, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, buf, pz, 0, x, y, z, r, g, b, dzdx, pm);
 						pz += 1;
 						pm += 1;
 						buf += 1;
 						n -= 1;
+						x += 1;
 					}
 				} else if (kDrawLogic == DRAW_SMOOTH && !(kInterpST || kInterpSTZ)) {
 					unsigned int *pz;
 					int buf = pp1 + x1;
-					unsigned int z, rgb, a, drgbdx;
+					unsigned int z, r, g, b, a;
 					int n;
 					n = (x2 >> 16) - x1;
 					pz = pz1 + x1;
 					z = z1;
-					rgb = (r1 << 16) & 0xFFC00000;
-					rgb |= (g1 >> 5) & 0x000007FF;
-					rgb |= (b1 << 5) & 0x001FF000;
-					drgbdx = _drgbdx;
+					r = r1;
+					g = g1;
+					b = b1;
 					a = a1;
 					while (n >= 3) {
-						putPixelSmooth<kDepthWrite, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, buf, pz, 0, z, tmp, rgb, a, dzdx, drgbdx, dadx);
-						putPixelSmooth<kDepthWrite, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, buf, pz, 1, z, tmp, rgb, a, dzdx, drgbdx, dadx);
-						putPixelSmooth<kDepthWrite, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, buf, pz, 2, z, tmp, rgb, a, dzdx, drgbdx, dadx);
-						putPixelSmooth<kDepthWrite, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, buf, pz, 3, z, tmp, rgb, a, dzdx, drgbdx, dadx);
+						putPixelSmooth<kDepthWrite, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, buf, pz, 0, x, y, z, r, g, b, a, dzdx, drdx, dgdx, dbdx, dadx);
+						putPixelSmooth<kDepthWrite, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, buf, pz, 1, x, y, z, r, g, b, a, dzdx, drdx, dgdx, dbdx, dadx);
+						putPixelSmooth<kDepthWrite, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, buf, pz, 2, x, y, z, r, g, b, a, dzdx, drdx, dgdx, dbdx, dadx);
+						putPixelSmooth<kDepthWrite, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, buf, pz, 3, x, y, z, r, g, b, a, dzdx, drdx, dgdx, dbdx, dadx);
 						pz += 4;
 						buf += 4;
 						n -= 4;
+						x += 4;
 					}
 					while (n >= 0) {
-						putPixelSmooth<kDepthWrite, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, buf, pz, 0, z, tmp, rgb, a, dzdx, drgbdx, dadx);
+						putPixelSmooth<kDepthWrite, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, buf, pz, 0, x, y, z, r, g, b, a, dzdx, drdx, dgdx, dbdx, dadx);
 						buf += 1;
 						pz += 1;
 						n -= 1;
+						x += 1;
 					}
 				} else if (kInterpST || kInterpSTZ) {
 					unsigned int *pz;
-					unsigned int s, t, z, rgb, a, drgbdx;
+					unsigned int s, t, z, r, g, b, a;
 					int n;
 					float sz, tz, fz, zinv;
+					int dsdx, dtdx;
+
 					n = (x2 >> 16) - x1;
 					fz = (float)z1;
 					zinv = (float)(1.0 / fz);
@@ -557,11 +529,10 @@ void FrameBuffer::fillTriangle(ZBufferPoint *p0, ZBufferPoint *p1, ZBufferPoint
 					z = z1;
 					sz = sz1;
 					tz = tz1;
-					rgb = (r1 << 16) & 0xFFC00000;
-					rgb |= (g1 >> 5) & 0x000007FF;
-					rgb |= (b1 << 5) & 0x001FF000;
+					r = r1;
+					g = g1;
+					b = b1;
 					a = a1;
-					drgbdx = _drgbdx;
 					while (n >= (NB_INTERP - 1)) {
 						{
 							float ss, tt;
@@ -574,13 +545,14 @@ void FrameBuffer::fillTriangle(ZBufferPoint *p0, ZBufferPoint *p1, ZBufferPoint
 							fz += fndzdx;
 							zinv = (float)(1.0 / fz);
 						}
-						for (int _a = 0; _a < 8; _a++) {
-							putPixelTextureMappingPerspective<kDepthWrite, kInterpRGB, kDrawLogic == DRAW_SMOOTH, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled, kRGB565Target>(this, buf, textureFormat, texture,
-							                           pz, _a, z, t, s, tmp, rgb, a, dzdx, dsdx, dtdx, drgbdx, dadx);
+						for (int _a = 0; _a < NB_INTERP; _a++) {
+							putPixelTextureMappingPerspective<kDepthWrite, kInterpRGB, kDrawLogic == DRAW_SMOOTH, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, buf, textureFormat, texture,
+							                           pz, _a, x, y, z, t, s, r, g, b, a, dzdx, dsdx, dtdx, drdx, dgdx, dbdx, dadx);
 						}
 						pz += NB_INTERP;
 						buf += NB_INTERP;
 						n -= NB_INTERP;
+						x += NB_INTERP;
 						sz += ndszdx;
 						tz += ndtzdx;
 					}
@@ -596,11 +568,12 @@ void FrameBuffer::fillTriangle(ZBufferPoint *p0, ZBufferPoint *p1, ZBufferPoint
 					}
 
 					while (n >= 0) {
-						putPixelTextureMappingPerspective<kDepthWrite, kInterpRGB, kDrawLogic == DRAW_SMOOTH, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled, kRGB565Target>(this, buf, textureFormat, texture,
-						                           pz, 0, z, t, s, tmp, rgb, a, dzdx, dsdx, dtdx, drgbdx, dadx);
+						putPixelTextureMappingPerspective<kDepthWrite, kInterpRGB, kDrawLogic == DRAW_SMOOTH, kAlphaTestEnabled, kEnableScissor, kBlendingEnabled>(this, buf, textureFormat, texture,
+						                           pz, 0, x, y, z, t, s, r, g, b, a, dzdx, dsdx, dtdx, drdx, dgdx, dbdx, dadx);
 						pz += 1;
 						buf += 1;
 						n -= 1;
+						x += 1;
 					}
 				}
 			}
@@ -621,12 +594,7 @@ void FrameBuffer::fillTriangle(ZBufferPoint *p0, ZBufferPoint *p1, ZBufferPoint
 					a1 += dadl_max;
 				}
 
-				if (kInterpST) {
-					s1 += dsdl_max;
-					t1 += dtdl_max;
-				}
-
-				if (kInterpSTZ) {
+				if (kInterpST || kInterpSTZ) {
 					sz1 += dszdl_max;
 					tz1 += dtzdl_max;
 				}
@@ -641,11 +609,7 @@ void FrameBuffer::fillTriangle(ZBufferPoint *p0, ZBufferPoint *p1, ZBufferPoint
 					b1 += dbdl_min;
 					a1 += dadl_min;
 				}
-				if (kInterpST) {
-					s1 += dsdl_min;
-					t1 += dtdl_min;
-				}
-				if (kInterpSTZ) {
+				if (kInterpST || kInterpSTZ) {
 					sz1 += dszdl_min;
 					tz1 += dtzdl_min;
 				}
@@ -660,19 +624,12 @@ void FrameBuffer::fillTriangle(ZBufferPoint *p0, ZBufferPoint *p1, ZBufferPoint
 
 			if (kDrawLogic == DRAW_SHADOW || kDrawLogic == DRAW_SHADOW_MASK)
 				pm1 = pm1 + xsize;
+			nb_lines--;
+			y++;
 		}
 	}
 }
 
-template <bool kInterpRGB, bool kInterpZ, bool kInterpST, bool kInterpSTZ, int kDrawMode, bool kDepthWrite, bool kEnableAlphaTest, bool kEnableScissor, bool kBlendingEnabled>
-void FrameBuffer::fillTriangle(ZBufferPoint *p0, ZBufferPoint *p1, ZBufferPoint *p2) {
-	if (pbuf.getFormat().bytesPerPixel == 2) {
-		fillTriangle<kInterpRGB, kInterpZ, kInterpST, kInterpSTZ, kDrawMode, kDepthWrite, kEnableAlphaTest, kEnableScissor, kBlendingEnabled, true>(p0, p1, p2);
-	} else {
-		fillTriangle<kInterpRGB, kInterpZ, kInterpST, kInterpSTZ, kDrawMode, kDepthWrite, kEnableAlphaTest, kEnableScissor, kBlendingEnabled, false>(p0, p1, p2);
-	}
-}
-
 template <bool kInterpRGB, bool kInterpZ, bool kInterpST, bool kInterpSTZ, int kDrawMode, bool kDepthWrite, bool kEnableAlphaTest, bool kEnableScissor>
 void FrameBuffer::fillTriangle(ZBufferPoint *p0, ZBufferPoint *p1, ZBufferPoint *p2) {
 	if (_blendingEnabled) {
@@ -684,8 +641,7 @@ void FrameBuffer::fillTriangle(ZBufferPoint *p0, ZBufferPoint *p1, ZBufferPoint
 
 template <bool kInterpRGB, bool kInterpZ, bool kInterpST, bool kInterpSTZ, int kDrawMode, bool kDepthWrite, bool kEnableAlphaTest>
 void FrameBuffer::fillTriangle(ZBufferPoint *p0, ZBufferPoint *p1, ZBufferPoint *p2) {
-	bool enableScissor = _clipRectangle.left != 0 || _clipRectangle.right != xsize || _clipRectangle.top != 0 || _clipRectangle.bottom != ysize;
-	if (enableScissor) {
+	if (_enableScissor) {
 		fillTriangle<kInterpRGB, kInterpZ, kInterpST, kInterpSTZ, kDrawMode, kDepthWrite, kEnableAlphaTest, true>(p0, p1, p2);
 	} else {
 		fillTriangle<kInterpRGB, kInterpZ, kInterpST, kInterpSTZ, kDrawMode, kDepthWrite, kEnableAlphaTest, false>(p0, p1, p2);
