@@ -17,6 +17,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *
  */
 
 #include "gui/console.h"
@@ -82,8 +83,6 @@ ConsoleDialog::ConsoleDialog(float widthPercent, float heightPercent)
 	_historyIndex = 0;
 	_historyLine = 0;
 	_historySize = 0;
-	for (int i = 0; i < kHistorySize; i++)
-		_history[i][0] = '\0';
 
 	// Display greetings & prompt
 	print(gScummVMFullVersion);
@@ -274,24 +273,19 @@ void ConsoleDialog::handleKeyDown(Common::KeyState state) {
 
 		if (len > 0) {
 
-			// We have to allocate the string buffer with new, since VC++ sadly does not
-			// comply to the C++ standard, so we can't use a dynamic sized stack array.
-			char *str = new char[len + 1];
+			Common::String str;
 
 			// Copy the user input to str
 			for (i = 0; i < len; i++)
-				str[i] = buffer(_promptStartPos + i);
-			str[len] = '\0';
+				str.insertChar(buffer(_promptStartPos + i), i);
 
 			// Add the input to the history
 			addToHistory(str);
 
 			// Pass it to the input callback, if any
 			if (_callbackProc)
-				keepRunning = (*_callbackProc)(this, str, _callbackRefCon);
+				keepRunning = (*_callbackProc)(this, str.c_str(), _callbackRefCon);
 
-				// Get rid of the string buffer
-			delete[] str;
 		}
 
 		print(PROMPT);
@@ -359,6 +353,7 @@ void ConsoleDialog::handleKeyDown(Common::KeyState state) {
 			defaultKeyDownHandler(state);
 			break;
 		}
+		// fall through
 	case Common::KEYCODE_DELETE:
 		if (_currentPos < _promptEndPos) {
 			killChar();
@@ -371,6 +366,7 @@ void ConsoleDialog::handleKeyDown(Common::KeyState state) {
 			defaultKeyDownHandler(state);
 			break;
 		}
+		// fall through
 	case Common::KEYCODE_END:
 		if (state.hasFlags(Common::KBD_SHIFT)) {
 			_scrollLine = _promptEndPos / kCharsPerLine;
@@ -388,6 +384,7 @@ void ConsoleDialog::handleKeyDown(Common::KeyState state) {
 			defaultKeyDownHandler(state);
 			break;
 		}
+		// fall through
 	case Common::KEYCODE_DOWN:
 		historyScroll(-1);
 		break;
@@ -397,6 +394,7 @@ void ConsoleDialog::handleKeyDown(Common::KeyState state) {
 			defaultKeyDownHandler(state);
 			break;
 		}
+		// fall through
 	case Common::KEYCODE_PAGEDOWN:
 		if (state.hasFlags(Common::KBD_SHIFT)) {
 			_scrollLine += _linesPerPage - 1;
@@ -415,6 +413,7 @@ void ConsoleDialog::handleKeyDown(Common::KeyState state) {
 			defaultKeyDownHandler(state);
 			break;
 		}
+		// fall through
 	case Common::KEYCODE_LEFT:
 		if (_currentPos > _promptStartPos)
 			_currentPos--;
@@ -426,6 +425,7 @@ void ConsoleDialog::handleKeyDown(Common::KeyState state) {
 			defaultKeyDownHandler(state);
 			break;
 		}
+		// fall through
 	case Common::KEYCODE_RIGHT:
 		if (_currentPos < _promptEndPos)
 			_currentPos++;
@@ -437,6 +437,7 @@ void ConsoleDialog::handleKeyDown(Common::KeyState state) {
 			defaultKeyDownHandler(state);
 			break;
 		}
+		// fall through
 	case Common::KEYCODE_HOME:
 		if (state.hasFlags(Common::KBD_SHIFT)) {
 			_scrollLine = _firstLineInBuffer + _linesPerPage - 1;
@@ -452,6 +453,7 @@ void ConsoleDialog::handleKeyDown(Common::KeyState state) {
 			defaultKeyDownHandler(state);
 			break;
 		}
+		// fall through
 	case Common::KEYCODE_UP:
 		historyScroll(+1);
 		break;
@@ -461,6 +463,7 @@ void ConsoleDialog::handleKeyDown(Common::KeyState state) {
 			defaultKeyDownHandler(state);
 			break;
 		}
+		// fall through
 	case Common::KEYCODE_PAGEUP:
 		if (state.hasFlags(Common::KBD_SHIFT)) {
 			_scrollLine -= _linesPerPage - 1;
@@ -575,8 +578,8 @@ void ConsoleDialog::killLastWord() {
 	}
 }
 
-void ConsoleDialog::addToHistory(const char *str) {
-	strcpy(_history[_historyIndex], str);
+void ConsoleDialog::addToHistory(const Common::String &str) {
+	_history[_historyIndex] = str;
 	_historyIndex = (_historyIndex + 1) % kHistorySize;
 	_historyLine = 0;
 	if (_historySize < kHistorySize)
@@ -590,8 +593,7 @@ void ConsoleDialog::historyScroll(int direction) {
 	if (_historyLine == 0 && direction > 0) {
 		int i;
 		for (i = 0; i < _promptEndPos - _promptStartPos; i++)
-			_history[_historyIndex][i] = buffer(_promptStartPos + i);
-		_history[_historyIndex][i] = '\0';
+			_history[_historyIndex].insertChar(buffer(_promptStartPos + i), i);
 	}
 
 	// Advance to the next line in the history
@@ -617,7 +619,8 @@ void ConsoleDialog::historyScroll(int direction) {
 		idx = (_historyIndex - _historyLine + _historySize) % _historySize;
 	else
 		idx = _historyIndex;
-	for (int i = 0; i < kLineBufferSize && _history[idx][i] != '\0'; i++)
+	int length = _history[idx].size();
+	for (int i = 0; i < length; i++)
 		printCharIntern(_history[idx][i]);
 	_promptEndPos = _currentPos;
 
