@@ -29,7 +29,7 @@
 #include "common/str.h"
 #include "audio/mididrv.h"
 
-#ifdef SMALL_SCREEN_DEVICE
+#ifdef GUI_ENABLE_KEYSDIALOG
 #include "gui/KeysDialog.h"
 #endif
 
@@ -37,9 +37,15 @@
 #include "gui/fluidsynth-dialog.h"
 #endif
 
+#ifdef USE_LIBCURL
+#include "backends/cloud/storage.h"
+#endif
+
 namespace GUI {
+class LauncherDialog;
 
 class CheckboxWidget;
+class EditTextWidget;
 class PopUpWidget;
 class SliderWidget;
 class StaticTextWidget;
@@ -61,6 +67,7 @@ public:
 	void init();
 
 	void open();
+	virtual void apply();
 	void close();
 	void handleCommand(CommandSender *sender, uint32 cmd, uint32 data);
 	const Common::String& getDomain() const { return _domain; }
@@ -74,8 +81,15 @@ protected:
 	ButtonWidget *_soundFontButton;
 	StaticTextWidget *_soundFont;
 	ButtonWidget *_soundFontClearButton;
+	
+	virtual void build();
+	virtual void clean();
+	void rebuild();
 
+
+	void addControlControls(GuiObject *boss, const Common::String &prefix);
 	void addGraphicControls(GuiObject *boss, const Common::String &prefix);
+	void addShaderControls(GuiObject *boss, const Common::String &prefix);
 	void addAudioControls(GuiObject *boss, const Common::String &prefix);
 	void addMIDIControls(GuiObject *boss, const Common::String &prefix);
 	void addMT32Controls(GuiObject *boss, const Common::String &prefix);
@@ -101,6 +115,23 @@ protected:
 	int _pathsTabId;
 
 private:
+	
+	//
+	// Control controls
+	//
+	bool _enableControlSettings;
+
+	CheckboxWidget *_touchpadCheckbox;
+	CheckboxWidget *_onscreenCheckbox;
+	CheckboxWidget *_swapMenuAndBackBtnsCheckbox;
+
+	StaticTextWidget *_kbdMouseSpeedDesc;
+	SliderWidget *_kbdMouseSpeedSlider;
+	StaticTextWidget *_kbdMouseSpeedLabel;
+	StaticTextWidget *_joystickDeadzoneDesc;
+	SliderWidget *_joystickDeadzoneSlider;
+	StaticTextWidget *_joystickDeadzoneLabel;
+
 	//
 	// Graphics controls
 	//
@@ -108,10 +139,20 @@ private:
 	StaticTextWidget *_gfxPopUpDesc;
 	PopUpWidget *_gfxPopUp;
 	CheckboxWidget *_fullscreenCheckbox;
+	CheckboxWidget *_filteringCheckbox;
 	CheckboxWidget *_aspectCheckbox;
-	CheckboxWidget *_softwareRenderingCheckbox; // ResidualVM specific
+	CheckboxWidget *_vsyncCheckbox; // ResidualVM specific
+	StaticTextWidget *_rendererTypePopUpDesc; // ResidualVM specific
+	PopUpWidget *_rendererTypePopUp; // ResidualVM specific
 	StaticTextWidget *_renderModePopUpDesc;
 	PopUpWidget *_renderModePopUp;
+	
+	//
+	// Shader controls
+	//
+	bool _enableShaderSettings;
+	StaticTextWidget *_shaderPopUpDesc;
+	PopUpWidget *_shaderPopUp;
 
 	//
 	// Audio controls
@@ -164,6 +205,9 @@ private:
 	//
 	// Volume controls
 	//
+	void updateMusicVolume(const int newValue) const;
+	void updateSfxVolume(const int newValue) const;
+	void updateSpeechVolume(const int newValue) const;
 	bool _enableVolumeSettings;
 
 	StaticTextWidget *_musicVolumeDesc;
@@ -187,11 +231,6 @@ protected:
 	Common::String _guioptions;
 	Common::String _guioptionsString;
 
-	//
-	//Theme Options
-	//
-	Common::String _oldTheme;
-
 	//
 	// Engine-specific controls
 	//
@@ -201,17 +240,23 @@ protected:
 
 class GlobalOptionsDialog : public OptionsDialog {
 public:
-	GlobalOptionsDialog();
+	GlobalOptionsDialog(LauncherDialog *launcher);
 	~GlobalOptionsDialog();
 
-	void open();
+	virtual void apply();
 	void close();
 	void handleCommand(CommandSender *sender, uint32 cmd, uint32 data);
+	void handleTickle();
 
 	virtual void reflowLayout();
 
 protected:
-#ifdef SMALL_SCREEN_DEVICE
+	virtual void build();
+	virtual void clean();
+
+	Common::String _newTheme;
+	LauncherDialog *_launcher;
+#ifdef GUI_ENABLE_KEYSDIALOG
 	KeysDialog *_keysDialog;
 #endif
 #ifdef USE_FLUIDSYNTH
@@ -237,6 +282,49 @@ protected:
 	PopUpWidget *_autosavePeriodPopUp;
 	StaticTextWidget *_guiLanguagePopUpDesc;
 	PopUpWidget *_guiLanguagePopUp;
+
+#ifdef USE_UPDATES
+	StaticTextWidget *_updatesPopUpDesc;
+	PopUpWidget *_updatesPopUp;
+#endif
+
+#ifdef USE_CLOUD
+	//
+	// Cloud controls
+	//
+	uint32 _selectedStorageIndex;
+	StaticTextWidget *_storagePopUpDesc;
+	PopUpWidget *_storagePopUp;
+	StaticTextWidget *_storageUsernameDesc;
+	StaticTextWidget *_storageUsername;
+	StaticTextWidget *_storageUsedSpaceDesc;
+	StaticTextWidget *_storageUsedSpace;
+	StaticTextWidget *_storageLastSyncDesc;
+	StaticTextWidget *_storageLastSync;
+	ButtonWidget	 *_storageConnectButton;
+	ButtonWidget	 *_storageRefreshButton;
+	ButtonWidget	 *_storageDownloadButton;
+	ButtonWidget	 *_runServerButton;
+	StaticTextWidget *_serverInfoLabel;
+	ButtonWidget	 *_rootPathButton;
+	StaticTextWidget *_rootPath;
+	ButtonWidget	 *_rootPathClearButton;
+	StaticTextWidget *_serverPortDesc;
+	EditTextWidget *_serverPort;
+	ButtonWidget	 *_serverPortClearButton;
+	bool _redrawCloudTab;
+#ifdef USE_SDL_NET
+	bool _serverWasRunning;
+#endif
+
+	void setupCloudTab();
+
+#ifdef USE_LIBCURL
+	void storageInfoCallback(Cloud::Storage::StorageInfoResponse response);
+	void storageListDirectoryCallback(Cloud::Storage::ListDirectoryResponse response);
+	void storageErrorCallback(Networking::ErrorResponse response);
+#endif
+#endif // USE_CLOUD
 };
 
 } // End of namespace GUI
