@@ -73,7 +73,7 @@ Quaternion Quaternion::zAxis(const Angle &angle) {
 	return q;
 }
 
-Quaternion Quaternion::slerpQuat(const Quaternion& to, const float t) {
+Quaternion Quaternion::slerpQuat(const Quaternion& to, const float t) const {
 	Quaternion dst;
 	float scale0, scale1;
 	float flip = 1.0f;
@@ -114,6 +114,11 @@ Quaternion& Quaternion::normalize() {
 	return *this;
 }
 
+void Quaternion::transform(Vector3d &v) const {
+	const Vector3d im = Vector3d(x(), y(), z());
+	v += 2.0 * Vector3d::crossProduct(im, Vector3d::crossProduct(im, v) + w() * v);
+}
+
 void Quaternion::fromMatrix(const Matrix3 &m) {
 	float qx, qy, qz, qw;
 	float tr = m.getValue(0, 0) + m.getValue(1, 1) + m.getValue(2, 2);
@@ -188,9 +193,13 @@ Matrix4 Quaternion::toMatrix() const {
 	return dst;
 }
 
-Quaternion Quaternion::inverse() {
-	normalize();
-	return Quaternion(-x(), -y(), -z(), w());
+Quaternion Quaternion::inverse() const {
+	Quaternion q = *this;
+	q.normalize();
+	q.x() = -q.x();
+	q.y() = -q.y();
+	q.z() = -q.z();
+	return q;
 }
 
 Vector3d Quaternion::directionVector(const int col) const {
