@@ -32,6 +32,7 @@
 #define VIDEO_BINK_DECODER_H
 
 #include "common/array.h"
+#include "common/bitstream.h"
 #include "common/rational.h"
 #include "graphics/surface.h" // ResidualVM specific
 
@@ -46,7 +47,6 @@ class QueuingAudioStream;
 
 namespace Common {
 class SeekableReadStream;
-class BitStream;
 class Huffman;
 
 class RDFT;
@@ -72,6 +72,9 @@ public:
 
 	bool loadStream(Common::SeekableReadStream *stream);
 	void close();
+
+	// ResidualVM-specific:
+	Common::Rational getFrameRate();
 protected:
 	void readNextPacket();
 	bool supportsAudioTrackSwitching() const { return true; }
@@ -104,7 +107,7 @@ private:
 
 		uint32 sampleCount;
 
-		Common::BitStream *bits;
+		Common::BitStream32LELSB *bits;
 
 		bool first;
 
@@ -137,7 +140,7 @@ private:
 		uint32 offset;
 		uint32 size;
 
-		Common::BitStream *bits;
+		Common::BitStream32LELSB *bits;
 
 		VideoFrame();
 		~VideoFrame();
@@ -163,7 +166,7 @@ private:
 		/** Decode a video packet. */
 		void decodePacket(VideoFrame &frame);
 
-	protected:
+	public: // ResidualVM
 		Common::Rational getFrameRate() const { return _frameRate; }
 
 	private:
@@ -263,6 +266,11 @@ private:
 		/** Value of the last decoded high nibble in color data types. */
 		int _colLastVal;
 
+		uint32 _yBlockWidth;   ///< Width of the Y plane in blocks
+		uint32 _yBlockHeight;  ///< Height of the Y plane in blocks
+		uint32 _uvBlockWidth;  ///< Width of the U and V planes in blocks
+		uint32 _uvBlockHeight; ///< Height of the U and V planes in blocks
+
 		byte *_curPlanes[4]; ///< The 4 color planes, YUVA, current frame.
 		byte *_oldPlanes[4]; ///< The 4 color planes, YUVA, last frame.
 
@@ -329,15 +337,16 @@ private:
 
 	class BinkAudioTrack : public AudioTrack {
 	public:
-		BinkAudioTrack(AudioInfo &audio);
+		BinkAudioTrack(AudioInfo &audio, Audio::Mixer::SoundType soundType);
 		~BinkAudioTrack();
 
 		/** Decode an audio packet. */
 		void decodePacket();
 
-		bool seek(const Audio::Timestamp &time) { return true; }  // ResidualVM-specific
+		bool seek(const Audio::Timestamp &time);  // ResidualVM-specific
 		bool isSeekable() const { return true; }  // ResidualVM-specific
 		void skipSamples(const Audio::Timestamp &length);  // ResidualVM-specific
+		int getRate(); // ResidualVM-specific
 	protected:
 		Audio::AudioStream *getAudioStream() const;
 
